/***************************************************************
 * 
 *  factory ext4 interface
 *
 *	date: 2019/06/24
 *
 * 	xxx_xxxx@apowertec.com
 *
 ****************************************************************/
#include <common.h>
#include <exports.h>
#include <part.h>
#include <config.h>
#include <command.h>
#include <image.h>
#include <linux/ctype.h>
#include <asm/byteorder.h>
#include <ext4fs.h>
#include <linux/stat.h>
#include <malloc.h>
#include <fs.h>
#include <fw_info.h>
#include <asm/arch/factorylib.h>
#include <asm/arch/system.h>
#include <gpt_parse.h>

/***************************************************************
*
*	marco define 
*
****************************************************************/
#define ENABLE_FACTORY_CONSOLE_MODE
#define ENABLE_FACTORY_DEBUG

#define FAC_PRINTF(fmt, args...)	printf(fmt, ##args)

#ifdef	ENABLE_FACTORY_DEBUG
#define FAC_DEBUG(fmt, args...)		printf(fmt, ##args)
#else
#define FAC_DEBUG(fmt, args...)
#endif

#define FACTORY_PART_NAME           "factory"
#define FACTORY_OLD_PATH_PREFIX		"tmp/factory"

#define FACTORY_BACKUP_PATH_PREFIX  "/.000@bk@"
#define FACTORY_BACKUP_PATH_PREFIX1  ".000@bk@"
/***************************************************************
*
*	variable define 
*
****************************************************************/
static char *factory_ext4_buffer = NULL;
static unsigned int factory_ext4_size = 0x0;

static disk_partition_t factory_ext4_info;
static block_dev_desc_t *factory_ext4_dev_desc = NULL;

static int factory_ext4_is_ready = -1;

/***************************************************************
*
*	factory ext4 api
*
****************************************************************/
static int factory_is_ready(void)
{
	return factory_ext4_is_ready;
}

static void factory_path_refine(char *path)
{
	char tmpname[256] = {0};
	char *pos = NULL;
	
	pos = strstr(path, FACTORY_OLD_PATH_PREFIX);
	if(pos == path){
		memset(tmpname, 0x00, sizeof(tmpname));
		memcpy(tmpname, path + strlen(FACTORY_OLD_PATH_PREFIX), strlen(path) - strlen(FACTORY_OLD_PATH_PREFIX));
		memset(path, 0x00, strlen(path));
		memcpy(path, tmpname, strlen(tmpname));
	}

	if(path[0] != '/'){
		memset(tmpname, 0x00, sizeof(tmpname));
		tmpname[0] = '/';
		memcpy(tmpname + 1, path, strlen(path));
		memset(path, 0x00, strlen(path));
		memcpy(path, tmpname, strlen(tmpname));
	}
	
	return;
}

static void factory_get_backup_filename(char *path)
{
	char tmpname[256] = {0};
	char *pos = NULL;
	unsigned int i = 0, max = 0;

	max = strlen(path);
	memset(tmpname, 0x00, sizeof(tmpname));
	memcpy(tmpname, FACTORY_BACKUP_PATH_PREFIX, strlen(FACTORY_BACKUP_PATH_PREFIX));

	pos = tmpname + strlen(tmpname);
	
	for(i = 0; i < max; i++){
		if(path[i] == '/')
			pos[i] = '\\';
		else
			pos[i] = path[i];
	}

	memset(path, 0x00, strlen(path));
	memcpy(path, tmpname, strlen(tmpname));
	
	return;
}

static int factory_parse_backup_filename(char *path)
{
	char tmpname[256] = {0};
	unsigned int i = 0, j = 0, max = 0;
	
	memset(tmpname, 0x00, sizeof(tmpname));

	if(strstr(path, FACTORY_BACKUP_PATH_PREFIX) == NULL)
		return -1;

	i = strlen(FACTORY_BACKUP_PATH_PREFIX);
	max = strlen(path);

	for( ; i < max; i++){
		if(path[i] == '\\')
			tmpname[j] = '/';
		else
			tmpname[j] = path[i];
		j++;
	}

	memset(path, 0x00,strlen(path));
	memcpy(path, tmpname, strlen(tmpname));
	
	return 0;
}

int factory_reset(void)
{
	if(factory_is_ready() < 0)
		return -1;
	
	/*format ext4 partition*/
	return 0;
}

int factory_delete(char *path)
{
	char filename[256] = {0};
	
	if(factory_is_ready() < 0)
		return -1;

	if((path == NULL) || ((strlen(path) + 1) > sizeof(filename)))
		return -1;

	memset(filename, 0x00, sizeof(filename));
	memcpy(filename, path, strlen(path));
	
	/*delete factory path header: tmp/factory*/
	factory_path_refine(filename);

	/* set the device as block device */
	ext4fs_set_blk_dev(factory_ext4_dev_desc, &factory_ext4_info);

	/* mount the filesystem */
	if (!ext4fs_mount(factory_ext4_info.size)) {
		FAC_PRINTF("mount ext4 partition %s fail!\r\n", FACTORY_PART_NAME);
		goto fail;
	}

	/* unlink file */
	if (ext4fs_unlink(filename)) {
		FAC_PRINTF("unlink %s fail!\r\n", filename);
		goto fail;
	}
	
	ext4fs_close();
	return 0;
fail:
	ext4fs_close();
	return -1;
}

int factory_read(char *path, char**buffer, int *length)
{
	int file_len = 0;
	int read_len = 0;
	char filename[256] = {0};
	
	if(factory_is_ready() < 0)
		return -1;

	if((path == NULL) || ((strlen(path) + 1) > sizeof(filename)))
		return -1;

	memset(filename, 0x00, sizeof(filename));
	memcpy(filename, path, strlen(path));
	
	/*delete factory path header: tmp/factory*/
	factory_path_refine(filename);

	/* set the device as block device */
	ext4fs_set_blk_dev(factory_ext4_dev_desc, &factory_ext4_info);

	/* mount the filesystem */
	if (!ext4fs_mount(factory_ext4_info.size)) {
		FAC_PRINTF("mount ext4 partition %s fail!\r\n", FACTORY_PART_NAME);
		goto fail;
	}

	/*open file and get file size*/
	file_len = ext4fs_open(filename);
	if (file_len < 0) {
		FAC_PRINTF("file %s not found in %s partition!\r\n", filename, FACTORY_PART_NAME);
		goto fail;
	}

	/*check file size*/
	if(file_len >= (int)factory_ext4_size){
		FAC_PRINTF("file size is too big, size = 0x%x!\r\n", (unsigned int)file_len);
		goto fail;
	}

	/*read data from factory*/
	if((factory_ext4_buffer + file_len) >= ((char *)rtk_mem_info->factory_data_addr + SYS_PARAM_FACTORY_SIZE)) {
		FAC_PRINTF("[EXT4] factory buffer reset! buff:%p, file:%s, len:0x%x\n", factory_ext4_buffer, filename, file_len);
		factory_ext4_buffer = (char *)rtk_mem_info->factory_data_addr;
	}

	read_len = ext4fs_read(factory_ext4_buffer, file_len);
	if(read_len != file_len){
		FAC_PRINTF("load file %s to ddr fail!\r\n", filename);
		goto fail;
	}

	*length = file_len;
	*buffer = factory_ext4_buffer;
	factory_ext4_buffer += file_len;
	
	ext4fs_close();
	return 0;
fail:
	ext4fs_close();
	return -1;
}

/***************************************************************
* function: factory_read_link_dest_path
* argument:
*         ln_path :       [IN]      link file source path
*         ln_dest_path:   [INOUT]   link file dest path
* return:
*         0 : return success
*         -1: return fail
*****************************************************************/
int factory_read_link_dest_path(char *ln_path, char *ln_dest_path)
{
	char filename[256] = {0};
	
	if(factory_is_ready() < 0)
		return -1;

	if((ln_path == NULL) || (ln_dest_path == NULL))
		return -1;

	memset(filename, 0x00, sizeof(filename));
	memcpy(filename, ln_path, strlen(ln_path));
	
	/*delete factory path header: tmp/factory*/
	factory_path_refine(filename);

	/* set the device as block device */
	ext4fs_set_blk_dev(factory_ext4_dev_desc, &factory_ext4_info);

	/* mount the filesystem */
	if (!ext4fs_mount(factory_ext4_info.size)) {
		FAC_PRINTF("mount ext4 partition %s fail!\r\n", FACTORY_PART_NAME);
		goto fail;
	}

	if(ext4fs_get_link_info(ln_path, ln_dest_path) < 0){
		FAC_PRINTF("read link file %s fail!\r\n", ln_path);
		goto fail;
	}
		
	ext4fs_close();
	return 0;
fail:
	ext4fs_close();
	return -1;
}

/***************************************************************
* function: factory_find_file
* argument:
*         dir :      [IN]      file directory    
*         prefix:    [IN]      file name prefix
*         filename:  [INOUT]   full file name
* return:
*         0 : return success
*         -1: return fail
*****************************************************************/
int factory_find_file(char *dir, char *prefix, char *filename)
{
	char dir_name[256] = {0};
	
	if(factory_is_ready() < 0)
		return -1;
	
	if(dir == NULL || prefix == NULL || filename == NULL)
		return -1;

	memset(dir_name, 0x00, sizeof(dir_name));
	memcpy(dir_name, dir, strlen(dir));
	
	/*delete factory path header: tmp/factory*/
	factory_path_refine(dir_name);
	
	/* set the device as block device */
	ext4fs_set_blk_dev(factory_ext4_dev_desc, &factory_ext4_info);

	/* mount the filesystem */
	if (!ext4fs_mount(factory_ext4_info.size)) {
		FAC_PRINTF("mount ext4 partition %s fail!\r\n", FACTORY_PART_NAME);
		goto fail;
	}

	if(ext4fs_find_filepath(dir_name, prefix, filename)){
		FAC_PRINTF("%s partition can not find %s%s***!\r\n",FACTORY_PART_NAME, dir_name, prefix);
		goto fail;
	}
	
	ext4fs_close();
	return 0;
fail:
	ext4fs_close();
	return -1;
}

int factory_write(char *path, char *buffer, int length)
{
	char filename[256] = {0};
	char tmp_filename[256] = {0};
	
	if(factory_is_ready() < 0)
		return -1;

	if(buffer == NULL)
		return -1;
	
	if((path == NULL) || ((strlen(path) + 1) > sizeof(filename)))
		return -1;

	memset(filename, 0x00, sizeof(filename));
	memcpy(filename, path, strlen(path));
	
	/*delete factory path header: tmp/factory*/
	factory_path_refine(filename);

	/*get tmp backup file path */
	memset(tmp_filename, 0x00, sizeof(tmp_filename));
	memcpy(tmp_filename, filename, strlen(filename));
	factory_get_backup_filename(tmp_filename);

	/* set the device as block device */
	ext4fs_set_blk_dev(factory_ext4_dev_desc, &factory_ext4_info);

	/* mount the filesystem */
	if (!ext4fs_mount(factory_ext4_info.size)) {
		FAC_PRINTF("mount ext4 partition %s fail!\r\n", FACTORY_PART_NAME);
		goto fail;
	}

	/*gen current timestamp*/
	ext4fs_timestamp_gen();
	
	/* first, update tmp backup file to filesystem*/
	if (ext4fs_write(tmp_filename, (unsigned char *)buffer, (unsigned long)length)) {
		FAC_PRINTF("write %s to %s partition fail!\r\n", tmp_filename, FACTORY_PART_NAME);
		goto fail;
	}

	/* second, update data to filesystem */
	if (ext4fs_write(filename, (unsigned char *)buffer, (unsigned long)length)) {
		FAC_PRINTF("write %s to %s partition fail!\r\n", filename, FACTORY_PART_NAME);
		goto fail;
	}

	/*last, delete tmp backup file from filesystem*/
	ext4fs_unlink(tmp_filename);
	
	ext4fs_close();
	return 0;
fail:
	ext4fs_close();
	return -1;
}

int factory_save(void)
{
	if(factory_is_ready() < 0)
		return -1;
	
	return 0;
}

/*factory file recovery mechanism*/
static int factory_get_file_mtime(char *path, unsigned int *mtime)
{
	char filename[256] = {0};
	
	if(factory_is_ready() < 0)
		return -1;

	if((path == NULL) || ((strlen(path) + 1) > sizeof(filename)))
		return -1;

	memset(filename, 0x00, sizeof(filename));
	memcpy(filename, path, strlen(path));
	
	/*delete factory path header: tmp/factory*/
	factory_path_refine(filename);

	/* set the device as block device */
	ext4fs_set_blk_dev(factory_ext4_dev_desc, &factory_ext4_info);

	/* mount the filesystem */
	if (!ext4fs_mount(factory_ext4_info.size)) {
		FAC_PRINTF("mount ext4 partition %s fail!\r\n", FACTORY_PART_NAME);
		goto fail;
	}

	/* get mtime */
	if (ext4fs_mtime(filename, mtime)) {
		FAC_PRINTF("get mtime %s fail!\r\n", filename);
		goto fail;
	}
		
	ext4fs_close();
	return 0;
fail:
	ext4fs_close();
	return -1;

}

static int factory_auto_check(void)
{
	int ret1 = 0, ret2 = 0;
	char *buffer1, *buffer2;
	int len1 = 0, len2 = 0;
	
	char filename[256] = {0};
	char tmp_filename[256] = {0};
	int recovery_flag = 0;

	memset(filename, 0x00, sizeof(filename));
	memset(tmp_filename, 0x00, sizeof(tmp_filename));

	/*get tmp backup file status*/
	ret1 = factory_find_file("/", FACTORY_BACKUP_PATH_PREFIX1, tmp_filename);
	if(ret1 < 0){
		FAC_PRINTF("Factory no need recovery!\r\n");
		return 0;
	}

	factory_path_refine(tmp_filename);

	/*get need recovery file path*/
	memcpy(filename, tmp_filename, strlen(tmp_filename));
	ret1 = factory_parse_backup_filename(filename);
	if(ret1 < 0){
		FAC_PRINTF("Cannot parse recovery file name!\r\n");
		return -1;
	}

	/*recovery */
	ret1 = factory_read(tmp_filename, &buffer1, &len1);
	ret2 = factory_read(filename, &buffer2, &len2);
	if((ret2 < 0) && (ret1 == 0)){
		FAC_PRINTF("Factory recovery run case1, save %s to %s!\r\n", tmp_filename, filename);
		factory_write(filename, buffer1, len1);
		recovery_flag = 1;
	}else if((ret1 == 0) && (ret2 == 0)){
		unsigned int mtime1 = 0, mtime2 = 0;

		factory_get_file_mtime(tmp_filename, &mtime1);
		factory_get_file_mtime(filename, &mtime2);

		FAC_DEBUG("mtime1 = 0x%x mtime2 = 0x%x len1 = 0x%x len2 = 0x%x\r\n", mtime1, mtime2, len1, len2);
		if(mtime1 == mtime2){
			FAC_PRINTF("Factory recovery run case2, save %s to %s!\r\n", tmp_filename, filename);
			factory_write(filename, buffer1, len1);
			recovery_flag = 1;
		}
	}

	if(recovery_flag == 0)
		factory_delete(tmp_filename);
	
	return 0;
}

int factory_init(void)
{
	int ret = 0;
	unsigned int dev_index = 0;
	char dev_string[32] = {0};
	int dev = -1, part = -1;
	
	if(boot_flash_type != BOOT_EMMC){
		FAC_PRINTF("Not support current flash, flash type = %d !\r\n", (int)boot_flash_type);
		return -1;
	}

	factory_ext4_buffer = (char *)rtk_mem_info->factory_data_addr;
	factory_ext4_size = SYS_PARAM_FACTORY_SIZE;

	FAC_PRINTF("[EXT4] factory_buffer:%p, factory_size:0x%x!\n", factory_ext4_buffer, factory_ext4_size);

	/*get factory partition index*/
	ret = get_dev_index_by_name(FACTORY_PART_NAME, &dev_index);
	if(ret < 0){
		FAC_PRINTF("Get %s partition block device index fail!\r\n", FACTORY_PART_NAME);
		FAC_PRINTF("No factory partition!\r\n");
		return 0;
	}
	
	/*get factory partition info*/
	memset(&factory_ext4_info, 0x00, sizeof(disk_partition_t));
	memset(dev_string, 0x00, sizeof(dev_string));
	snprintf(dev_string, sizeof(dev_string), "0:0x%x", dev_index);
	part = get_device_and_partition("mmc", dev_string, &factory_ext4_dev_desc, &factory_ext4_info, 1);
	if (part < 0){
		FAC_PRINTF("Get %s partition desc info fail!\r\n", FACTORY_PART_NAME);
		return -1;
	}
	dev = factory_ext4_dev_desc->dev;

	FAC_PRINTF("Partiton %s located at %d:%d \r\n", FACTORY_PART_NAME, dev, part);
	
	factory_ext4_is_ready = 1;

	factory_auto_check();

	return 0;
}

#ifdef ENABLE_FACTORY_CONSOLE_MODE
int factory_dump_info(void)
{
	if(factory_is_ready() < 0)
		return -1;
	
	FAC_PRINTF("****** dump factory info ******\n");
	FAC_PRINTF("boot_flash_type    = ");
	switch(boot_flash_type){
		case BOOT_NOR_SERIAL:
			FAC_PRINTF("NOR\n");
			break;

		case BOOT_NAND:
			FAC_PRINTF("NAND\n");
			break;

		case BOOT_EMMC:
			FAC_PRINTF("MMC\n");
			break;

		case BOOT_SPI_NAND:
            FAC_PRINTF("SPI NAND\n");
			break;

		default:
			FAC_PRINTF("UNKNOWN\n");
			break;
	}

	FAC_PRINTF("factory_buffer     = 0x%08x\n", (uint)factory_ext4_buffer);
	FAC_PRINTF("factory_total_size = 0x%08x\n", factory_ext4_size);

	return 0;
}

int factory_dump_list(void)
{
	if(factory_is_ready() < 0)
		return -1;
	
	/* set the device as block device */
	ext4fs_set_blk_dev(factory_ext4_dev_desc, &factory_ext4_info);

	/* mount the filesystem */
	if (!ext4fs_mount(factory_ext4_info.size)) {
		FAC_PRINTF("mount ext4 partition %s fail!\r\n", FACTORY_PART_NAME);
		goto fail;
	}

	/* show factory file list */
	if (ext4fs_ls("/")) {
		FAC_PRINTF("show %s partition file list fail!\r\n", FACTORY_PART_NAME);
		goto fail;
	}
	
	ext4fs_close();
	return 0;
fail:
	ext4fs_close();
	return -1;
}

int ext4_factory_testcase(int argc, char * const argv[])
{
	int ret = 0;

	if (strcmp(argv[1], "test1") == 0) {
		unsigned int cnt = simple_strtoul(argv[2], NULL, 16);
		unsigned int i = 0;
		
		char *write_buf = (char *)TEMP_BUFFER_FOR_FLASH_DATA_ADDR;
		int test_size = 0x400;
		
		char *read_buf;
		int read_len = 0;

		for(i = 0; i < cnt; i++){
			memset(write_buf, i % 0xFF, test_size);

			ret = factory_write("debug", write_buf, test_size);
			if(ret < 0){
				printf(" test factory_write fail!\r\n");
				return CMD_RET_FAILURE;
			}

			ret = factory_read("debug", &read_buf, &read_len);
			if(ret < 0){
				printf(" test factory_read fail!\r\n");
				return CMD_RET_FAILURE;
			}

			if(test_size != read_len){
				printf(" test factory_read size fail!\r\n");
				return CMD_RET_FAILURE;
			}

			ret = memcmp(write_buf, read_buf, test_size);
			if(ret != 0){
				printf(" test cmp fail!\r\n");
				return CMD_RET_FAILURE;
			}

			ret = factory_delete("debug");
			if(ret != 0){
				printf(" factory_delete fail!\r\n");
				return CMD_RET_FAILURE;
			}

			printf("test write-read-delete loop-%d value 0x%x pass\r\n", i, i % 0xFF);
		}

		printf("test write-read-delete value pass\r\n");
	}

	return CMD_RET_SUCCESS;
}

int do_ext4_factory(cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
	char *buffer;
	unsigned long dst_buffer;//for rtice tool factory read
	int length;
	int rc;

	if (argc < 2) {
		return CMD_RET_USAGE;
	}
	else {
		if (strcmp(argv[1], "save") == 0) {
			factory_save();
		}
		else if (strcmp(argv[1], "reset") == 0) {
			factory_reset();
		}
		else if (strcmp(argv[1], "info") == 0) {
			factory_dump_info();
		}
		else if (strcmp(argv[1], "list") == 0) {
			factory_dump_list();
		}
		else if (strcmp(argv[1], "delete") == 0) {
			factory_delete(argv[2]);
		}
		else if (strcmp(argv[1], "read") == 0) {
			rc = factory_read(argv[2], &buffer, &length);

			if (rc)
				FAC_PRINTF("%s: file not found\n", argv[2]);
			else
				FAC_PRINTF("%s: %d bytes at 0x%08x\n", argv[2], length, (uint)buffer);

			if (argc == 4) {
				dst_buffer = simple_strtoul(argv[3], NULL, 16);
				memcpy((void *)dst_buffer, (void *)buffer, length);
				FAC_PRINTF("src[0x%08x] dst[0x%08x]\n", (uint)buffer, (uint)dst_buffer);
			}
		}
		else if (strcmp(argv[1], "write") == 0) {

			buffer = (char *)simple_strtoul(argv[3], NULL, 16);
			length = (int)simple_strtoul(argv[4], NULL, 16);

			factory_write(argv[2], buffer, length);
		}
		else if (strcmp(argv[1], "linkinfo") == 0) {
			char destfile[256] = {0};

			memset(destfile, 0x00, sizeof(destfile));

			if(factory_read_link_dest_path(argv[2], destfile) < 0){
				FAC_PRINTF("get link dest file fail!\r\n");
				return CMD_RET_FAILURE;
			}else{
				FAC_PRINTF("%s --- > %s !\r\n", argv[2], destfile);
			}
		}else if (strstr(argv[1], "test")) {
			return ext4_factory_testcase(argc, argv);
		}else {
			return CMD_RET_USAGE;
		}
	}

	return 0;
}


U_BOOT_CMD(
	factory,	5,	1,	do_ext4_factory,
	"FACTORY sub system",
	"read path\n"
	"factory write path addr length\n"
	"factory delete path\n"
	"factory save\n"
	"factory reset\n"
	"factory info\n"
	"factory list\n"
	"factory linkinfo path\n"
);
#endif
