#include <common.h>
#include <asm/arch/io.h>
#include <asm/interrupt.h>
#include "smp.h"
#include "system.h"
#include "fw_info.h"
#ifdef CONFIG_SYS_SUPPORT_MULTITHREAD
#include <asm/thread.h>
#endif

void coproc_enter_pending(int cpu_id);
extern void v7_outer_cache_disable(void);
void Interrupt_request(InterruptNum irq_num, void* func, void * func_arg);
void ipi_send_target(unsigned int cpu_id, InterruptNum ipi_num);
extern void coproc_putc(char c);

extern int platform_cpu_wake(unsigned int cpu);

extern int megic_number_cleaned;
unsigned int cpu_id_list[4] = {0x81000000, 0x81000100, 0x81000200, 0x81000300};
unsigned int smp_stack_start = CONFIG_SYS_INIT_CORE1_SP_ADDR;
unsigned int smp_stack_size = 0x10000;
volatile unsigned int GlobalActive_Area[5] = {0, 0, 0, 0, 0};
volatile CoreWakeupFunc ActiveFuncPointer[5] = {0, 0, 0, 0, 0};
unsigned int active_cores = 0;
void smp_mutex_lock(void)
{
   unsigned int i;

   while(1)
   {
      if (rtd_inl(SMP_SHARED_SEM_REG) == 0x1)
      {
         break;
      }

      for(i = 0;i < 1024;i++); 
   }
}

void smp_mutex_unlock(void)
{
   rtd_outl(SMP_SHARED_SEM_REG, 0x0);
}

void smp_release_coproc(void)
{
    u32 cpuid;
    cpuid = smp_processor_id(get_MPIDR());
    coproc_enter_pending(cpuid);
}

int smp_active_cores(void)
{
	return active_cores;
}
void cpu_reset(void *jump);
/*
 * function executed by core 1~3
 */
void coproc_enter_pending(int cpu_id)
{
	void (*jump)(void);

	invalidate_dcache_range(0x500, 0x510);
	switch(cpu_id) {
		case 1:
			 if(rtd_inl(MAGIC_TAG_CORE1_reg) == MAGIC_COPROC_NOT_SUPPORT)
				return;
			 rtd_outl(MAGIC_TAG_CORE1_reg, MAGIC_COPROC_IN_REDIRECT);
			jump = (void (*)(void))rtd_inl(MAGIC_TAG_CORE1_reg);
			break;
		case 2:
			 if(rtd_inl(MAGIC_TAG_CORE2_reg) == MAGIC_COPROC_NOT_SUPPORT)
				return;
			rtd_outl(MAGIC_TAG_CORE2_reg, MAGIC_COPROC_IN_REDIRECT);
			jump = (void (*)(void))rtd_inl(MAGIC_TAG_CORE2_reg);
			break;
		case 3:
			 if(rtd_inl(MAGIC_TAG_CORE3_reg) == MAGIC_COPROC_NOT_SUPPORT)
				return;
			rtd_outl(MAGIC_TAG_CORE3_reg, MAGIC_COPROC_IN_REDIRECT);
			jump = (void (*)(void))rtd_inl(MAGIC_TAG_CORE3_reg);
			break;
		default:
			return;
    }

//#ifdef CONFIG_SUPPORT_TIMER_INTERRUPT
//	rtk_arch_timer_deinit();
//#endif

	/*
	 * Turn off I-cache and invalidate it
	 */
	icache_disable();
	invalidate_icache_all();

	/*
	 * turn off D-cache
	 * dcache_disable() in turn flushes the d-cache and disables MMU
	 */
	dcache_disable();
	v7_outer_cache_disable();

	/*
	 * After D-cache is flushed and before it is disabled there may
	 * be some new valid entries brought into the cache. We are sure
	 * that these lines are not dirty and will not affect our execution.
	 * (because unwinding the call-stack and setting a bit in CP15 SCTRL
	 * is all we did during this. We have not pushed anything on to the
	 * stack. Neither have we affected any static data)
	 * So just invalidate the entire d-cache again to avoid coherency
	 * problems for kernel
	 */
	invalidate_dcache_all();

	//jump to redirect code directly, not pending here
	//jump();
	cpu_reset(jump);

}


int smp_processor_id(unsigned int mpidr)
{
	int i = 0;
	while(cpu_id_list[i] != 0xdeaddead) {
		if (cpu_id_list[i] == mpidr)
			return i;
		i++;
	}
	return -1;
}

int smp_boot_secondary(unsigned int cpu_bitmap)
{
    unsigned int regValue = 0;
    unsigned int regBase = 0;
#if defined(CONFIG_USE_IRQ) ||defined(CONFIG_QC_VERIFY)
    unsigned char cpuid =0;
#endif
    int i = 0;
puts("smp_boot_secondary\n");
    active_cores = 1;
    for (i = 0;i < 3;i++)
    {
     
        if (i == 0) {
            if ((cpu_bitmap & _BIT1) != _BIT1)
                continue;
            else {
                regBase = MAGIC_TAG_CORE1_reg;
#if defined(CONFIG_USE_IRQ) ||defined(CONFIG_QC_VERIFY)
                cpuid = 1;
#endif
            }
        }
        else if (i == 1) {
            if ((cpu_bitmap & _BIT2) != _BIT2)
                continue;
            else {
                regBase = MAGIC_TAG_CORE2_reg;
#if defined(CONFIG_USE_IRQ) ||defined(CONFIG_QC_VERIFY)
                cpuid = 2;
#endif
            }
        }
        else if (i == 2) {
            if ((cpu_bitmap & _BIT3) != _BIT3)
                continue;
            else {
                regBase = MAGIC_TAG_CORE3_reg;
#if defined(CONFIG_USE_IRQ) ||defined(CONFIG_QC_VERIFY)
                cpuid = 3;
#endif
            }
        }
#ifdef CONFIG_QC_VERIFY
       puts("platform_cpu_wake \n");
	platform_cpu_wake(cpuid);
	mdelay(2);
#endif
	invalidate_dcache_range(0x500,0x520);
        regValue = rtd_inl(regBase);
        if(regValue == MAGIC_COPROC_NOT_SUPPORT)
		continue;

//        if (regValue == MAGIC_COPROC_IN_REDIRECT)
        {
            rtd_outl(regBase, MAGIC_COPROC_JUMP_TO_UBOOT);
            active_cores++;
            flush_dcache_range(0x500,0x520);
#if (CONFIG_FT == 1)
            printf("Core %d started\n", i + 1);
#endif
#ifdef CONFIG_USE_IRQ
            ipi_send_target(cpuid,0);
#else
            asm volatile("SEV"); 
#endif
            do {
		    invalidate_dcache_range(0x500,0x520);
	    } while(rtd_inl(regBase) != MAGIC_COPROC_OP_GET_READY) ;
	    smp_stack_start += smp_stack_size; 
        }
    }
    return 0;
}

unsigned int smp_magic_tag(unsigned int cpu_id)
{
	unsigned int regBase = 0;
	switch(cpu_id) {
		case 1:
			regBase = MAGIC_TAG_CORE1_reg;
			break;
		case 2:
			regBase = MAGIC_TAG_CORE2_reg;
			break;
		case 3:
			regBase = MAGIC_TAG_CORE3_reg;
			break;
		default:
			coproc_putc('E');
			coproc_putc('r');
			coproc_putc('r');
			coproc_putc('1');
			while(1);
	}
	return regBase;
}

void Core_Wakeup(CoreWakeupFunc __Addr, u32 CoreID)
{
	if(CoreID >= 4)
	{
		printf("Error!! in Core_Wakeup, Not correct CPUID %d\n", CoreID);
		while(1);
	}
       if(GlobalActive_Area[CoreID] == 1)
       {
            printf("Core %d is active\n", CoreID);
        
        }
        else
        {
#ifdef CONFIG_USE_IRQ
            ipi_send_target(CoreID,0);
#else
            asm volatile("SEV"); 
#endif
            ActiveFuncPointer[CoreID] =  __Addr;
            GlobalActive_Area[CoreID] = 1;         
        }
}

int smp_release_secondary(unsigned int cpu_bitmap)
{
	unsigned int regValue = 0;
	unsigned int regBase = 0;
	int i = 0;

	for (i = 0;i < 3;i++)
	{
		// decide which core
		if (i == 0) {
			if ((cpu_bitmap & _BIT1) != _BIT1) {
				continue;
			}
			else {
				regBase = MAGIC_TAG_CORE1_reg;
         	}
		}
		else if (i == 1) {
			if ((cpu_bitmap & _BIT2) != _BIT2) {
				continue;
			}
			else {
				regBase = MAGIC_TAG_CORE2_reg;
			}
		}
		else if (i == 2) {
			if ((cpu_bitmap & _BIT3) != _BIT3) {
				continue;
			}
			else {
				regBase = MAGIC_TAG_CORE3_reg;
			}
		}

		// get coreX current status
		regValue = rtd_inl(regBase);
		if(regValue == MAGIC_COPROC_NOT_SUPPORT)
			continue;
		if (regValue == MAGIC_COPROC_OP_GET_READY) {
			rtd_outl(regBase, MAGIC_COPROC_IN_REDIRECT);
			flush_dcache_range(0x500, 0x510);

#if (CONFIG_FT == 0)
			printf("Core %d stoped\n", i + 1);
#endif
		}
	}

	return 0;
}

void release_non_boot_core(void)
{
#ifdef CONFIG_SYS_SUPPORT_MULTITHREAD
	if(init_done[1] && init_done[2] && init_done[3])
	{
		megic_number_cleaned=1;
		smp_release_secondary(0xf);
		thread_cond_signal(&g_sub_cond[1]);
		thread_cond_signal(&g_sub_cond[2]);
		thread_cond_signal(&g_sub_cond[3]);
	}
	else
	{
		printf("%s Should not be here!!!\n", __FUNCTION__);
	}
#if 0  //RTK FIXME!!
	while((!smp_cpu_released[1]) || (!smp_cpu_released[2]) || (!smp_cpu_released[3]))
	{
		++waitCount;

		if (SMP_WAIT_NON_BOOT_CPU_RELEASE_COUNT < waitCount)
		{
			if (!smp_cpu_released[1])
				printf("smp_cpu_not_released_1??? !\n");
			if (!smp_cpu_released[2])
				printf("smp_cpu_not_released_2??? !\n");
			if (!smp_cpu_released[3])
				printf("smp_cpu_not_released_3??? !\n");

			do_reset(NULL, 0, 0, NULL);  //just in case, should never be here!
		}
	}
#endif
#endif
}

int smp_init(void)
{
    unsigned char boot_from_STR = 0;
#if defined (CONFIG_JUMP_TO_64BIT_KERNEL)
    unsigned int  cpuID;
#endif
    puts("smp_init\n");
#ifdef CONFIG_USE_IRQ
	 /* set up exceptions */
	interrupt_init();
	/* enable exceptions */
	enable_interrupts();
#endif
    if ((rtd_inl(REG_CRT_SUSPEND_reg) == KERNEL_RSV_MEM_MAGICNO) && (rtd_inl(REG_STR_JUMP_ADDR_reg) != 0))
        boot_from_STR = 1;
    if(!boot_from_STR)
        return smp_boot_secondary(0xf);
#if defined (CONFIG_JUMP_TO_64BIT_KERNEL)
    else
    {
        cpuID = smp_processor_id(get_MPIDR());
        rtd_outl(smp_magic_tag(cpuID), MAGIC_COPROC_OP_SMP_DISABLE);
    }
#endif
        return 0;
}

#ifdef CONFIG_USE_IRQ
void ipi_broadcast(InterruptNum ipi_num)
{
    if((ipi_num>=E_INT_IPI_0x130_START) && (ipi_num<=E_INT_IPI_0x130_END))
        mhal_ipi_broadcast(ipi_num-E_INT_IPI_0x130_START);
    else
        printf("Wrong ipi number: %d please refer to MsIRQ.h\n", ipi_num);
}

void Interrupt_request(InterruptNum irq_num, void* func, void * func_arg)
{
    if((irq_num>=E_INT_IPI_0x130_START) && (irq_num<=E_INT_IPI_0x130_END))
        mhal_ipi_attach( irq_num-E_INT_IPI_0x130_START, func, 0);
    else
        RtOS_AttachInterrupt(irq_num, func);
}

void ipi_send_target(unsigned int cpu_id, InterruptNum ipi_num)
{
    if((ipi_num >= E_INT_IPI_0x130_START) && (ipi_num <= E_INT_IPI_0x130_END))
        mhal_ipi_send_target(cpu_id, ipi_num - E_INT_IPI_0x130_START);
    else
        printf("Wrong ipi number: %d please refer to MsIRQ.h\n", ipi_num);
}

void ipi_send_self(InterruptNum ipi_num)
{
    if((ipi_num >= E_INT_IPI_0x130_START) && (ipi_num <= E_INT_IPI_0x130_END))
        mhal_ipi_send_self(ipi_num - E_INT_IPI_0x130_START);
    else
        printf("Wrong ipi number: %d please refer to MsIRQ.h\n", ipi_num);
}
#else

void Interrupt_request(InterruptNum irq_num, void* func, void * func_arg)
{
}
void ipi_broadcast(InterruptNum ipi_num)
{
}
void ipi_send_target(unsigned int cpu_id, InterruptNum 
ipi_num)
{
}
void ipi_send_self(InterruptNum ipi_num)
{
}
#endif

