/************************************************************************
 *     Include files
 ************************************************************************/

#include <common.h>
#include <asm/utils.h>
#include <asm/arch/system.h>
#include <rbus/sb2_reg.h>
#include <fw_info.h>
#include <watchdog.h>

/************************************************************************
 *      Definitions
 ************************************************************************/

DECLARE_GLOBAL_DATA_PTR;

#if defined(Config_Support_STR_Enable)
#define IS_ENABLE_STR
#endif

#define ENABLE_STRBOOT_DEBUG
#ifdef ENABLE_STRBOOT_DEBUG
#define ENABLE_STRBOOT_DEUBG_PRINT
#endif

/************************************************************************
 *      External variables
 ************************************************************************/
extern int is_kcpu_fw_exist; /*used to determine whether the kcpu fw existed*/
/************************************************************************
 *      External function prototype
 ************************************************************************/

extern void rtl8168_clock_enable(void);
extern int _f_exc_redirect_img, _e_exc_redirect_img;
extern int _f_v_entry_img, _e_v_entry_img;
extern int do_vcpu_bisr(unsigned int vcpuid);
extern int do_acpu_bisr(unsigned int acpuid);
extern int do_ve_bisr(void);
extern void enable_audio2_clock(void);
extern void disable_console(void);
extern void enable_console(void);
// For Compiler Warning
extern int rtk_rbus_timeout_thr_init(void);
extern int rtk_scpu2_bisr(void);
extern int rtk_check_ip_bisr(void);
uint bIsSTR = 0;

//used to determine whether need keep watchdog on in STR flow.
//g_wdog_env = 1: means watchdog should keep enable.
int g_wdog_env = -1;

#if defined (Config_TPV_AMP_TRUE)
#include <platform_lib/board/pcb_mgr.h>
#include <platform_lib/board/pcb.h>
#include <i2c.h>
#include <drivers/i2c/rtk_i2c_lib.h>
extern void AudioAmp_InitAudioOut2I2S_1st(void);
#endif
/************************************************************************
 *      Static function prototype
 ************************************************************************/

/************************************************************************
 *      Function body
 ************************************************************************/

static void strboot_putc(char ch)
{
        while ((rtd_inl(CONFIG_SYS_NS16550_COM1 + 0x14) & _BIT5) == 0)
                ;

        rtd_outl(CONFIG_SYS_NS16550_COM1, ch);
}


static void strboot_resume_kernel (void)
{
#ifdef CONFIG_BOOT_ATF
       extern void call_smc(unsigned int, unsigned int, unsigned int, unsigned int);

#ifdef CONFIG_JUMP_TO_64BIT_KERNEL
       printf("call_smc 64 entry:%x suspend_reg:%x jump_reg:%x \n",LINUX64_KERNEL_ENTRY,rtd_inl(REG_CRT_SUSPEND_reg),rtd_inl(REG_STR_JUMP_ADDR_reg));
       call_smc(0x23792379,LINUX64_KERNEL_ENTRY,rtd_inl(REG_CRT_SUSPEND_reg),rtd_inl(REG_STR_JUMP_ADDR_reg));
#else
       printf("call_smc 32 entry:%x suspend_reg:%x jump_reg:%x \n",LINUX_KERNEL_ENTRY,rtd_inl(REG_CRT_SUSPEND_reg),rtd_inl(REG_STR_JUMP_ADDR_reg));
       call_smc(0,LINUX_KERNEL_ENTRY,rtd_inl(REG_CRT_SUSPEND_reg),rtd_inl(REG_STR_JUMP_ADDR_reg));
#endif
       while(1);

#else
	unsigned int (*GoFuncPtr)(void);
	GoFuncPtr=(unsigned int (*)(void))(rtd_inl(REG_STR_JUMP_ADDR_reg));

	strboot_putc('*'); // print '*' for STR debug

	rtd_outl((REG_STR_JUMP_ADDR_reg),0);
	rtd_outl((REG_CRT_SUSPEND_reg),0);

	(*GoFuncPtr)();
	while(1)
	{
		printf("%s_%d\n", __FUNCTION__, __LINE__);
	}
#endif
}

void init_k_cpu (void)
{
	unsigned int address = rtd_inl(STB_WDOG_DATA6);
	printf("%s_%d  : address: %x\n", __FUNCTION__, __LINE__, address);
	rtd_outl((STB_WDOG_DATA10), address | 0x80000000);
	printf("%s_%d\n", __FUNCTION__, __LINE__);

}

static void hold_hw_semaphore(void)
{
	//printf("## Hold HW semaphore\n");
	while(!(rtd_inl(SB2_HD_SEM_reg)))
	{
		udelay(100);
	}
}

static void release_hw_semaphore(void)
{
//	printf("## Release HW semaphore\n");

	rtd_outl(SB2_HD_SEM_reg, 0x0);
}

void init_av_mem (void)
{
	unsigned char *a,*b;
        boot_av_info_t *boot_av;
	extern uint *boot_av_info_ptr;
	extern uint *uart_reg_base_ptr;

	a = (unsigned char *)&_e_exc_redirect_img;
	b = (unsigned char *)&_f_exc_redirect_img;
	memcpy((unsigned char *)MIPS_EXC_REDIRECT_CODE_ADDR, b, a-b);

	a = (unsigned char *)&_e_v_entry_img;
	b = (unsigned char *)&_f_v_entry_img;
	memcpy((unsigned char *)MIPS_V_ENTRY_CODE_ADDR, b, a-b);

	rtd_outl(VCPU_JUMP_ADDR_reg,  MIPS_V_ENTRY_CODE_ADDR | MIPS_KSEG1BASE);
#ifdef CONFIG_VCPU2_SUPPORT
	rtd_outl(VCPU2_JUMP_ADDR_reg, MIPS_V_ENTRY_CODE_ADDR | MIPS_KSEG1BASE);
#endif
	rtd_outl(ACPU_JUMP_ADDR_reg,  MIPS_A_ENTRY_CODE_ADDR | MIPS_KSEG1BASE);

	*uart_reg_base_ptr = SWAPEND32(CONFIG_SYS_NS16550_COM1 | 0xb0000000);

	*boot_av_info_ptr = 0;

	memset((void *)(0x000000d0), 0, (0x30)); // for sync RPC status

	boot_av = (boot_av_info_t *) (0x00001400);
	memset(boot_av, 0, sizeof(boot_av_info_t));
}

void init_av_cpu (void)
{
	extern uint *AudioFW_entry_ptr_A1;
#ifdef CONFIG_ACPU2_SUPPORT
	extern uint *AudioFW_entry_ptr_A2;
#endif
        extern uint *VideoFW_entry_ptr_V1;
	extern uint *VideoFW_entry_ptr_V2;

	hold_hw_semaphore();
	//rtd_outl(SB2_HD_SEM, 0x1);

	*AudioFW_entry_ptr_A1 = SWAPEND32(MIPS_AUDIO_FW_1_ENTRY_ADDR | MIPS_KSEG1BASE);
#ifdef CONFIG_ACPU2_SUPPORT
	*AudioFW_entry_ptr_A2 = SWAPEND32(MIPS_AUDIO_FW_2_ENTRY_ADDR | MIPS_KSEG1BASE);
#endif
	*VideoFW_entry_ptr_V1 = SWAPEND32(MIPS_VIDEO_FW_1_ENTRY_ADDR | MIPS_KSEG1BASE);
	*VideoFW_entry_ptr_V2 = SWAPEND32(MIPS_VIDEO_FW_2_ENTRY_ADDR | MIPS_KSEG1BASE);

	flush_dcache_all();

	udelay(50);

	enable_audio_clock();
#ifdef CONFIG_ACPU2_SUPPORT
	enable_audio2_clock();
#endif
#ifdef ENABLE_ACPU_BISR
	do_acpu_bisr(_BIT0 | _BIT1);
#endif

	enable_video_clock();
#ifdef CONFIG_VCPU2_SUPPORT
	enable_video2_clock();
#endif
#ifdef ENABLE_VCPU_BISR
	do_vcpu_bisr(_BIT0 | _BIT1);
#endif
#ifdef ENABLE_VE_BISR
	do_ve_bisr();
#endif
	release_hw_semaphore();
	//rtd_outl(SB2_HD_SEM, 0x0);
}


#if defined(CONFIG_ARM_DCC) && !defined(CONFIG_BAUDRATE)
#define CONFIG_BAUDRATE 115200
#endif

static int init_baudrate(void)
{
        gd->baudrate = getenv_ulong("baudrate", 10, CONFIG_BAUDRATE);
        return 0;
}

void    relocate_code (ulong, gd_t *, ulong) __attribute__ ((noreturn));
void strboot_init_f (void)
{

#ifdef IS_ENABLE_STR
	if ((rtd_inl(REG_CRT_SUSPEND_reg) != KERNEL_RSV_MEM_MAGICNO) || (rtd_inl(REG_STR_JUMP_ADDR_reg) == 0))
	{
		return;
	}

	gd = (gd_t *) _stack_start_str;
	__asm__ __volatile__("": : :"memory");
	memset((void *)gd, 0, sizeof(gd_t));
	timer_init();
	init_baudrate();
	serial_init();
	console_init_f();

	strboot_putc('#'); // print '#' for STR debug

#ifdef ENABLE_STRBOOT_DEUBG_PRINT
        enable_console(); // for debug
#else
        disable_console(); // default disable printf for speed up, but will be enable if occur error
#endif

	rtk_rbus_timeout_thr_init();
	clock_init();
#ifdef ENABLE_SCPU2_BISR
	rtk_scpu2_bisr();
#endif

	relocate_code(_stack_start_str, (gd_t *)gd, _TEXT_BASE );
#endif  //#ifdef IS_ENABLE_STR
}

/*
#if defined(CONFIG_REALTEK_AMP_TAS5711) || defined(CONFIG_REALTEK_AMP_AD83586) || defined(CONFIG_REALTEK_AMP_TAS5707_TV013) || defined(CONFIG_REALTEK_AMP_ACL1310) ||defined(CONFIG_REALTEK_AMP_ALC1311)
#include <platform_lib/board/pcb_mgr.h>
#include <platform_lib/board/pcb.h>
#include <i2c.h>
#include <drivers/i2c/rtk_i2c_lib.h>
extern void AudioAmp_InitAudioOut2I2S_1st(void);
extern void AudioAmp_InitAudioOut2I2S_2nd(void);

static void strboot_init_amp(void)
{
#if defined(PIN_AMP_RESET)
	IO_Config(PIN_AMP_RESET);
#endif
#if defined(PIN_I2C3_SDA) && defined(PIN_I2C3_SCL)
	IO_Config(PIN_I2C3_SDA);
	IO_Config(PIN_I2C3_SCL);
#endif
	i2c_init(CONFIG_SYS_I2C_SPEED, CONFIG_SYS_I2C_SLAVE); //init_func_i2c();
	AudioAmp_InitAudioOut2I2S_1st(); //init_amp();
	AudioAmp_InitAudioOut2I2S_2nd();
}
#endif
*/
void strboot_init_r (void)
{

#ifdef IS_ENABLE_STR
	if ((rtd_inl(REG_CRT_SUSPEND_reg) != KERNEL_RSV_MEM_MAGICNO) || (rtd_inl(REG_STR_JUMP_ADDR_reg) == 0))
	{
		return;
	}
	bIsSTR = 1; // it is STR flow
#ifdef CONFIG_BOOT_ATF
        rtd_outl(0x520,0x1234abcd);
#endif
#ifdef ENABLE_STRBOOT_DEUBG_PRINT
	enable_console(); // for debug
#else
	disable_console(); // default disable printf for speed up, but will be enable if occur error
#endif

	printf("rtd_inl(0x%x):%x\n", REG_STR_JUMP_ADDR_reg, rtd_inl(REG_STR_JUMP_ADDR_reg));
        printf("%s_%d\n", __FUNCTION__, __LINE__);
/*
#if defined(CONFIG_REALTEK_AMP_TAS5711) || defined(CONFIG_REALTEK_AMP_AD83586) || defined(CONFIG_REALTEK_AMP_TAS5707_TV013) || defined(CONFIG_REALTEK_AMP_ACL1310) ||defined(CONFIG_REALTEK_AMP_ALC1311)
	strboot_init_amp();
#endif
*/
#ifdef CONFIG_RTL8168
	rtl8168_clock_enable();//init_rtl8168();
#endif

	init_av_mem();
	init_av_cpu();
    	if(is_kcpu_fw_exist == 1){
	    	init_k_cpu ();
    	}

	strboot_putc('@'); // print '@' for STR debug
#if defined (Config_TPV_AMP_TRUE)
	AudioAmp_InitAudioOut2I2S_1st();
#endif	
#ifdef CONFIG_REALTEK_WATCHDOG
	printf("Watchdog Enabled\n");
	WATCHDOG_KICK();
#else
	if(g_wdog_env == 1){/* this case means no need disable watchdog. */
		printf("Watchdog Enabled\n");
		WATCHDOG_KICK();
	}
	else{
		printf("Close WatchDog\n");
		WATCHDOG_DISABLE();/* disable hw watchdog for debug */
	}
#endif

	if(rtk_check_ip_bisr() != 0)
	{
		enable_console();
		printf("\n\033[31m[ERR] BISR CHECK FAIL, PLEASE CHECK ABOVE LOG!!\033[0m\n");
		while(1);
	}

	bIsSTR = 0;

	strboot_resume_kernel();
#endif  //#ifdef IS_ENABLE_STR
}

