#ifndef __RTK_INTERRUPT_H__
#define __RTK_INTERRUPT_H__

#define MAX_IPI				16
#define MAX_IRQ				256

#define PPI_PHYS_SECURE		29

typedef unsigned int InterruptNum;
typedef void (*mhal_ipi_t)(unsigned int);
typedef int (*irq_func_t)(void);

#define E_INT_IPI_0x130_START 0
#define E_INT_IPI_NUM_1 1
#define E_INT_IPI_NUM_2 2
#define E_INT_IPI_NUM_3 3
#define E_INT_IPI_0x130_END 15

enum
{
    IPI_NEED_RESCHEDULE =  E_INT_IPI_NUM_1,
    IPI_TEST_1                    = E_INT_IPI_NUM_2,
    IPI_TEST_2                    = E_INT_IPI_NUM_3,
    IPI_MAX_NUM1              = E_INT_IPI_0x130_END
};

void RtOS_EnableInterrupt(InterruptNum irq_num);
void RtOS_DisableInterrupt(InterruptNum irq_num);
void RtOS_AttachInterrupt(InterruptNum irq_num, irq_func_t func);
void RtOS_DetachInterrupt(InterruptNum irq_num);

void mhal_ipi_attach(unsigned int ipi_num, mhal_ipi_t isr, unsigned int dat);
void mhal_ipi_detach(unsigned int ipi_num);
void mhal_ipi_mask(unsigned int ipi_num);
void mhal_ipi_unmask(unsigned int ipi_num);
void mhal_ipi_send(unsigned int cpu_mask, unsigned int ipi_num);
void mhal_ipi_broadcast(unsigned int ipi_num);
void mhal_ipi_send_target(unsigned int cpu_id, unsigned int ipi_num);
void mhal_ipi_send_self(unsigned int ipi_num);
void rtk_interrupt_gic_cpu_init(void);
#endif // __RTK_INTERRUPT_H__
