/*
 * Copyright (c) 2001-2007, Tom St Denis
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef TOMCRYPT_H_
#define TOMCRYPT_H_
//#include <assert.h>
//#include <stdio.h>
//#include <string.h>
//#include <stdlib.h>
//#include <time.h>
//#include <ctype.h>
//#include <limits.h>
//#include <stddef.h>  // for ptrdiff_t
#define CFG_CRYPTO 1 /* 'y' */
#define CFG_CRYPTO_AES 1 /* 'y' */
#define CFG_CRYPTO_AES_ARM32_CE 1 /* 'y' */
/* CFG_CRYPTO_AES_ARM64_CE is not set ('n') */
#define CFG_CRYPTO_CBC 1 /* 'y' */
#define CFG_CRYPTO_CBC_MAC 1 /* 'y' */
#define CFG_CRYPTO_CCM 1 /* 'y' */
#define CFG_CRYPTO_CMAC 1 /* 'y' */
#define CFG_CRYPTO_CONCAT_KDF 1 /* 'y' */
#define CFG_CRYPTO_CTR 1 /* 'y' */
#define CFG_CRYPTO_CTS 1 /* 'y' */
#define CFG_CRYPTO_DES 1 /* 'y' */
#define CFG_CRYPTO_DH 1 /* 'y' */
#define CFG_CRYPTO_DSA 1 /* 'y' */
#define CFG_CRYPTO_ECB 1 /* 'y' */
#define CFG_CRYPTO_ECC 1 /* 'y' */
#define CFG_CRYPTO_GCM 1 /* 'y' */
#define CFG_CRYPTO_HKDF 1 /* 'y' */
#define CFG_CRYPTO_HMAC 1 /* 'y' */
#define CFG_CRYPTO_MD5 1 /* 'y' */
#define CFG_CRYPTO_PBKDF2 1 /* 'y' */
#define CFG_CRYPTO_RSA 1 /* 'y' */
#define CFG_CRYPTO_SHA1 1 /* 'y' */
#define CFG_CRYPTO_SHA1_ARM32_CE 1 /* 'y' */
/* CFG_CRYPTO_SHA1_ARM64_CE is not set ('n') */
#define CFG_CRYPTO_SHA224 1 /* 'y' */
#define CFG_CRYPTO_SHA256 1 /* 'y' */
#define CFG_CRYPTO_SHA256_ARM32_CE 1 /* 'y' */
/* CFG_CRYPTO_SHA256_ARM64_CE is not set ('n') */
#define CFG_CRYPTO_SHA384 1 /* 'y' */
#define CFG_CRYPTO_SHA512 1 /* 'y' */
#define CFG_CRYPTO_SIZE_OPTIMIZATION 1 /* 'y' */
#define CFG_CRYPTO_WITH_CE 1 /* 'y' */
#define CFG_CRYPTO_XTS 1 /* 'y' */

/* use configuration data */
#include "tomcrypt_custom.h"

#ifdef __cplusplus
extern "C" {
#endif

/* version */
#define CRYPT   0x0117
#define SCRYPT  "1.17"

/* max size of either a cipher/hash block or symmetric key [largest of the two] */
#define MAXBLOCKSIZE  128

/* descriptor table size */
#define TAB_SIZE      32

/*optimize the AES ECB/CBC flow*/
#define USE_MCP_OPTIMIZED_MIN_LEN        192


struct tomcrypt_arm_neon_state {
	unsigned long state;
};

/* Temporarily enables neon instructions */
void inline tomcrypt_arm_neon_enable(struct tomcrypt_arm_neon_state *state)
{
}
/* Disables neon instructions after a call to tomcrypt_arm_neon_enable() */
void inline tomcrypt_arm_neon_disable(struct tomcrypt_arm_neon_state *state)
{
}


/* error codes [will be expanded in future releases] */
enum {
   CRYPT_OK=0,             /* Result OK */
   CRYPT_ERROR,            /* Generic Error */
   CRYPT_NOP,              /* Not a failure but no operation was performed */

   CRYPT_INVALID_KEYSIZE,  /* Invalid key size given */
   CRYPT_INVALID_ROUNDS,   /* Invalid number of rounds */
   CRYPT_FAIL_TESTVECTOR,  /* Algorithm failed test vectors */

   CRYPT_BUFFER_OVERFLOW,  /* Not enough space for output */
   CRYPT_INVALID_PACKET,   /* Invalid input packet given */

   CRYPT_INVALID_PRNGSIZE, /* Invalid number of bits for a PRNG */
   CRYPT_ERROR_READPRNG,   /* Could not read enough from PRNG */

   CRYPT_INVALID_CIPHER,   /* Invalid cipher specified */
   CRYPT_INVALID_HASH,     /* Invalid hash specified */
   CRYPT_INVALID_PRNG,     /* Invalid PRNG specified */

   CRYPT_MEM,              /* Out of memory */

   CRYPT_PK_TYPE_MISMATCH, /* Not equivalent types of PK keys */
   CRYPT_PK_NOT_PRIVATE,   /* Requires a private PK key */

   CRYPT_INVALID_ARG,      /* Generic invalid argument */
   CRYPT_FILE_NOTFOUND,    /* File Not Found */

   CRYPT_PK_INVALID_TYPE,  /* Invalid type of PK key */
   CRYPT_PK_INVALID_SYSTEM,/* Invalid PK system specified */
   CRYPT_PK_DUP,           /* Duplicate key already in key ring */
   CRYPT_PK_NOT_FOUND,     /* Key not found in keyring */
   CRYPT_PK_INVALID_SIZE,  /* Invalid size input for PK parameters */

   CRYPT_INVALID_PRIME_SIZE,/* Invalid size of prime requested */
   CRYPT_PK_INVALID_PADDING, /* Invalid padding on input */

   CRYPT_HASH_OVERFLOW      /* Hash applied to too many bits */
};

#include "tomcrypt_cfg.h"
#include "tomcrypt_macros.h"
#include "tomcrypt_cipher.h"
#include "tomcrypt_hash.h"
#include "tomcrypt_mac.h"
#include "tomcrypt_prng.h"
#include "tomcrypt_pk.h"
#include "tomcrypt_math.h"
#include "tomcrypt_misc.h"
#include "tomcrypt_argchk.h"
#include "tomcrypt_pkcs.h"

#ifdef __cplusplus
   }
#endif

#endif /* TOMCRYPT_H_ */


/* $Source: /cvs/libtom/libtomcrypt/src/headers/tomcrypt.h,v $ */
/* $Revision: 1.21 $ */
/* $Date: 2006/12/16 19:34:05 $ */
