/* Copyright (C) 2011
 *  * Corscience GmbH & Co. KG - Simon Schwarz <schwarz@corscience.de>
 *   *  - Added prep subcommand support
 *    *  - Reorganized source - modeled after powerpc version
 *     *
 *      * (C) Copyright 2002
 *       * Sysgo Real-Time Solutions, GmbH <www.elinos.com>
 *        * Marius Groeger <mgroeger@sysgo.de>
 *         *
 * Copyright (C) 2001  Erik Mouw (J.A.K.Mouw@its.tudelft.nl)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307	 USA
 *
 */

#include <common.h>
#include <command.h>
#include <image.h>
#include <u-boot/zlib.h>
#include <asm/byteorder.h>
#include <fdt.h>
#include <libfdt.h>
#include <fdt_support.h>
#include <asm/bootm.h>
#include <exports.h>
#include <environment.h>
#include <wakeup.h>
#include <asm/arch/system.h>
#include <asm/arch/fw_info.h>
#include <asm/arch/bootparam.h>
#include <platform_lib/board/pcb_mgr.h>
#include <watchdog.h>
#include <factorylib.h>

typedef enum {
	COLD_BOOT,
	WARM_BOOT,
	WATCHDOG,
	RECOVERY,
	KERNEL_PANIC,
	REBOOT_SHELL,
}eBootreason;
#define arg_next(p)             (char *)((unsigned long)p+strlen(p))

void set_boot_reason(char *kargs)
{
	extern uchar boot_ac_on;
	uchar wdt_count = 0;
	eBootreason ebr = COLD_BOOT;
	uint boot_cause = 0;
	if (boot_ac_on) {
		if (boot_mode != BOOT_RESCUE_MODE)
		{
			ebr = COLD_BOOT;
		}
		else
		{
			ebr = RECOVERY;
		}
	}
	else
	{
		ebr = WARM_BOOT;
	}
	wdt_count = (rtd_inl(ISO_MISC_TCWCR_reg)>> 8)&0x7;
	if(wdt_count)
	{
		boot_cause = rtd_inl(STB_WDOG_DATA4);
		if(boot_cause & 0x1)
			ebr = REBOOT_SHELL;
		else if(boot_cause & 2)
			ebr = KERNEL_PANIC;
		else
			ebr = WATCHDOG;
	}

	switch(ebr)
	{
		case COLD_BOOT:
			sprintf(arg_next(kargs),"androidboot.bootreason=cold ");
		break;
		case WARM_BOOT:
			sprintf(arg_next(kargs),"androidboot.bootreason=warm ");
		break;
		case WATCHDOG:
			sprintf(arg_next(kargs),"androidboot.bootreason=watchdog ");
		break;
		case RECOVERY:
			sprintf(arg_next(kargs),"androidboot.bootreason=recovery ");
		break;
		case KERNEL_PANIC:
			sprintf(arg_next(kargs),"androidboot.bootreason=kernel-panic ");
		break;
		case REBOOT_SHELL:
			sprintf(arg_next(kargs),"androidboot.bootreason=reboot ");
		break;
		default:
			printf("BOOT REASON UNKNOWN !!!! \n");
	}
}
