/*
 * cmd_rtk_otp.c - interface to One-Time-Programmable memory
 *
 * Copyright (c) 2008-2015 Analog Devices Inc.
 *
 * Licensed under the GPL-2 or later.
 */
#include <common.h>
#include <command.h>
#include <malloc.h>
#include <net.h>
#include <pcmcia/rtk_pcmcia.h>




typedef enum {
	PCMCIA_CMD_HELP,
	PCMCIA_CMD_CARD_ENABLE,	
	PCMCIA_CMD_CARD_DISABLE,
	PCMCIA_CMD_CARD_RESET,
	PCMCIA_CMD_READ_ATTR_MEM,
	PCMCIA_CMD_WRITE_ATTR_MEM,
	PCMCIA_CMD_READ_IO_MEM,
	PCMCIA_CMD_WRITE_IO_MEM,
	PCMCIA_CMD_READ_CIS,
}PCMCIA_CMD_ID;

typedef struct {
	PCMCIA_CMD_ID      id;
	int addr;
	int len;
	char *data;
}PCMCIA_CMD;
static void pcmcia_dump_mem(unsigned char* pData, unsigned short Len)
{
	int i = 0;
	int j = 0;
	printf("------------------------------------------------------------\n");
	printf("ADDRESS   00 01 02 03 04 05 06 07   08 09 0A 0B 0C 0D 0E 0F \n");
	printf("------------------------------------------------------------\n");    
	for (i=0; i<Len; i+=16)
	{                       
		printf("%08x  ", i);                          
                    
		for (j=0; j<16; j++)    
		{
			int c = i+j;
			if (c<Len)
				printf("%02x ", pData[c]);                        
			else
				printf("-- ");            
                            
			if ((c & 0x07)==0x07)
				printf("  ");            
		}
        
		printf(" ");
        
		for (j=0; j<16; j++)    
		{   
			int c = i+j;                    
			if(pData[c] <= 0x7E && pData[c] >= 0x20)                       
				printf("%c", (char) pData[c]);
			else                               
				printf(".");
		}
        
		printf("\n");               
	}
	printf("------------------------------------------------------------\n");
}

static int str_to_long(char *str, long* p_val, unsigned char base)
{
	char *p;
	long int  val;

	base = (base == 8 || base == 10 || base == 16 || base == 32) ? base : 0;
	val = simple_strtoul(str,&p,base);

	if (*p=='\0') {
		*p_val = val;
		return 0;
	}
	return -1;
}

static int str_to_hex_array(char *str, int len)
{
	int i = 0;
	if(!str || len <= 0)
		return 0;
	while(len >= 1)
	{
		char c = 0;
		if(len >= 2) {
			if(str[len - 1] <= '9' && str[len - 1] >= '0')
				c = str[len - 1] - '0';
			else if(str[len - 1] <= 'F' && str[len - 1] >= 'A')
				c = str[len -1] - 'A' + 0xA;
			else if(str[len - 1] <= 'f' && str[len - 1] >= 'a')
				c = str[len -1] - 'a' + 0xA;
			
			if(str[len - 2] <= '9' && str[len - 2] >= '0')
				c |= (str[len - 2] - '0') << 4;
			else if(str[len - 2] <= 'F' && str[len - 2] >= 'A')
				c |= (str[len - 2] - 'A' + 0xA) << 4;
			else if(str[len - 2] <= 'f' && str[len - 2] >= 'a')
				c |= (str[len - 2] - 'a' + 0xA) << 4;
			len = len -2;
		}
		else {
			if(str[len - 1] <= '9' && str[len - 1] >= '0')
				c = str[len - 1] - '0';
			else if(str[len - 1] <= 'F' && str[len - 1] >= 'A')
				c = str[len -1] - 'A' + 0xA;
			else if(str[len - 1] <= 'f' && str[len - 1] >= 'a')
				c = str[len -1] - 'a' + 0xA;
			len = len -1;
		}
		str[i++] = c;
	}
	str[i] = 0;
	return (i - 1);

}
static void do_pcmcia_card_enable()
{
	venus_pcmcia_init();
	if(venus_pcmcia_enable(1) != PCMCIA_OK){
		printf("Pcmcia enable failed\n");
	} else {
		printf("Pcmcia enable sucess\n");
	}
	
	
}

static void do_pcmcia_card_disable()
{
	venus_pcmcia_init();
	if(venus_pcmcia_enable(0) != PCMCIA_OK){
		printf("Pcmcia disable failed\n");
	} else {
		printf("Pcmcia disable sucess\n");
	}
	
	
}

static void do_pcmcia_card_reset()
{
	venus_pcmcia_init();
	if(venus_pcmcia_card_reset(3000) != PCMCIA_OK){
		printf("Pcmcia card reset failed\n");
	} else {
		printf("Pcmcia card reset sucess\n");
	}
	
	
}
static void do_pcmcia_read_attr_mem(int addr, int len)
{
	int i = 0;
	char *buf = NULL;
	if(addr < 0 || len <= 0)
	{
		printf("Parametes input error\n");
		return;
	}
	buf = malloc(len);
	if(!buf)
	{
		printf("Malloc fail\n");
		return;
	}
	venus_pcmcia_init();
	if(venus_pcmcia_read_attr_mem(addr, buf, len) != PCMCIA_OK)	
	{
		printf("pcmcia read attr mem failed\n");
		goto FAIL;
	}
	printf("pcmcia read attr mem sucess\n");
	pcmcia_dump_mem(buf, len);
FAIL:
	if(buf)
		free(buf);
}

static void do_pcmcia_write_attr_mem(int addr, char *buf, int len)
{
	int i = 0;
	if(addr < 0 || !buf || len <= 0)
	{
		printf("Parametes input error\n");
		return;
	}
	venus_pcmcia_init();
	if(venus_pcmcia_write_attr_mem(addr, buf, len) != PCMCIA_OK)	
	{
		printf("pcmcia write attr mem failed\n");
		goto FAIL;
	}
	printf("pcmcia write attr mem sucess\n");
	pcmcia_dump_mem(buf, len);
FAIL:
	return;
}


static void do_pcmcia_read_io_mem(int addr, int len)
{
	int i = 0;
	char *buf = NULL;
	if(addr < 0 || len <= 0)
	{
		printf("Parametes input error\n");
		return;
	}
	buf = malloc(len);
	if(!buf)
	{
		printf("Malloc fail\n");
		return;
	}
	venus_pcmcia_init();
	if(venus_pcmcia_read_io_mem(addr, buf, len) != PCMCIA_OK)	
	{
		printf("pcmcia read io mem failed\n");
		goto FAIL;
	}
	printf("pcmcia read io mem sucess\n");
	pcmcia_dump_mem(buf, len);
FAIL:
	if(buf)
		free(buf);
}

static void do_pcmcia_write_io_mem(int addr, char *buf, int len)
{
	int i = 0;
	if(addr < 0 || !buf || len <= 0)
	{
		printf("Parametes input error\n");
		return;
	}
	venus_pcmcia_init();
	if(venus_pcmcia_write_io_mem(addr, buf, len) != PCMCIA_OK)	
	{
		printf("pcmcia write io mem failed\n");
		goto FAIL;
	}
	printf("pcmcia write io mem sucess\n");
	pcmcia_dump_mem(buf, len);
FAIL:
	return;
}

static void do_pcmcia_read_cis()
{
	int i = 0;
	char buf[128]={0x00};
	memset(buf,0, 128);
	venus_pcmcia_init();
	if(venus_pcmcia_read_attr_mem(i, buf, 128) != PCMCIA_OK)	
	{
		printf("pcmcia read cis failed\n");
		return;
	}
	printf("pcmcia read cis sucess\n");
	pcmcia_dump_mem(buf, 128);
}


/*******************************************************************************
 * FUNC : do_help
 * DESC : 
 * PARM : 
 * RET  : 
 *******************************************************************************/
static void do_help()
{                        
	printf("                                                    \n");    
	printf(" @@@ Realtek PCMCIA Tool @@@                           \n");
	printf("                                                    \n");    
	printf(" Format : pcmcia <opcode> [ofst=<ofst>][data=<data>][len=<len>]\n");     
	printf("                                                    \n");    
	printf("   <opcode> : operations                            \n");  
	printf("          -s  : Card Enable              \n");  
	printf("          -f   : Card Disable              \n");  	
	printf("          -r   : Card Reset              \n");  		
	printf("          -a  : Read Attr Mem              \n");    
	printf("          -b  : Write Attr Mem          \n");
	printf("          -d  : Read IO Mem          \n");
	printf("          -e  : Write IO Mem    \n");
	printf("          -c  : Read CIS       \n");  
	printf("          -h  : print help                              \n");         
	printf("                                                    \n");    
	printf("   <addr>  : the offset of read/write action     \n");                
	printf("                                                    \n");   
	printf("   <data>  : the hex data array     \n");                
	printf("                                                    \n");   	
	printf("   <len>  : the len of data array     \n");                
	printf("                                                    \n");     
	printf(" Example: Card Enable                             \n");      
	printf("      pcmcia -s\n");  
	printf("                                                    \n");  	
	printf(" Example: Card Disable                             \n");      
	printf("      pcmcia -f\n");  
	printf("                                                    \n");  	
	printf(" Example: Card Reset                             \n");      
	printf("      pcmcia -r\n");  
	printf("                                                    \n");  		
	printf(" Example: Read Attr Mem                             \n");      
	printf("      pcmcia -a addr=<addr> len=<len>\n");  
	printf("                                                    \n");  
	printf(" Example: Write Attr Mem                             \n");      
	printf("      pcmcia -b addr=<addr> data=<data> len=<len>\n");  
	printf("                                                    \n");  	
	printf(" Example: Read IO Mem                             \n");      
	printf("      pcmcia -d addr=<addr> len=<len>\n");  
	printf("                                                    \n");  	
	printf(" Example: Write IO Mem                             \n");      
	printf("      pcmcia -e addr=<addr> data=<data> len=<len>\n");  
	printf("                                                    \n");  			
	printf(" Example: Read CIS                             \n");      
	printf("      pcmcia -c    \n");  
	printf("                                                    \n");  	
}

static int check_cmd_range(PCMCIA_CMD *p_cmd)
{
	if(p_cmd->addr < 0 ||p_cmd->len < 0)
	{
		printf("Parameters input error\n");
		return -1;
	}
	return 0;
}

/*******************************************************************************
 * FUNC : do_command
 * DESC : 
 * PARM : 
 * RET  : 
 *******************************************************************************/
static int do_command(PCMCIA_CMD *p_cmd)
{
	switch (p_cmd->id) {
	case PCMCIA_CMD_CARD_ENABLE:
		do_pcmcia_card_enable();
		break;
	case PCMCIA_CMD_CARD_DISABLE:
		do_pcmcia_card_disable();
		break;
	case PCMCIA_CMD_CARD_RESET:
		do_pcmcia_card_reset();
		break;		
	case PCMCIA_CMD_READ_CIS:
		if (check_cmd_range(p_cmd)<0)
			return 0;
		do_pcmcia_read_cis();              
		break;
	case PCMCIA_CMD_READ_ATTR_MEM:
		if (check_cmd_range(p_cmd)<0)
			return 0;
		do_pcmcia_read_attr_mem(p_cmd->addr,p_cmd->len);              
		break;
	case PCMCIA_CMD_WRITE_ATTR_MEM:
		if (check_cmd_range(p_cmd)<0)
			return 0;
		do_pcmcia_write_attr_mem(p_cmd->addr, p_cmd->data, p_cmd->len);              
		break;		
	case PCMCIA_CMD_READ_IO_MEM:
		if (check_cmd_range(p_cmd)<0)
			return 0;
		do_pcmcia_read_io_mem(p_cmd->addr,p_cmd->len);              
		break;	
	case PCMCIA_CMD_WRITE_IO_MEM:
		if (check_cmd_range(p_cmd)<0)
			return 0;
		do_pcmcia_write_io_mem(p_cmd->addr, p_cmd->data ,p_cmd->len);              
		break;			
	case PCMCIA_CMD_HELP:  
		do_help();              
		break;
	}
}

/*******************************************************************************
 * FUNC : do_command
 * DESC : 
 * PARM : 
 * RET  : 
 *******************************************************************************/
static int parse_command(unsigned char idx, const char* str, PCMCIA_CMD *p_cmd)
{    
	if (idx == 1) {
		if (strncmp(str, "-a", 2) == 0)
			p_cmd->id = PCMCIA_CMD_READ_ATTR_MEM;
		else if (strncmp(str, "-b", 2) == 0)
			p_cmd->id = PCMCIA_CMD_WRITE_ATTR_MEM;	
		else if (strncmp(str, "-d", 2) == 0)
			p_cmd->id = PCMCIA_CMD_READ_IO_MEM;
		else if (strncmp(str, "-e", 2) == 0)
			p_cmd->id = PCMCIA_CMD_WRITE_IO_MEM;		
		else if (strncmp(str, "-c", 2) == 0)
			p_cmd->id = PCMCIA_CMD_READ_CIS;
		else if (strncmp(str, "-s", 2) == 0)
			p_cmd->id = PCMCIA_CMD_CARD_ENABLE;
		else if (strncmp(str, "-f", 2) == 0)
			p_cmd->id = PCMCIA_CMD_CARD_DISABLE;	
		else if (strncmp(str, "-r", 2) == 0)
			p_cmd->id = PCMCIA_CMD_CARD_RESET;		
		else if (strncmp(str, "-h", 2) == 0)
			p_cmd->id = PCMCIA_CMD_HELP;
		else
			return -1;
	} else {
		if (strncmp(str, "addr=", 5) == 0) {                    
			if (str_to_long(str + 5, &p_cmd->addr, 16) == -1) {                
				printf("invalid data addr- %s\n", str);          
				return -1;
			} 
		} else if (strncmp(str, "len=", 4) == 0) {
			if (str_to_long(str + 4, &p_cmd->len, 10) == -1) {                
				printf("invalid len - %s\n", str);          
				return -1;
			}   
		}  else if (strncmp(str, "data=", 5) == 0) {
			p_cmd->data = str + 5;
			if(strlen(p_cmd->data) >= 2 &&(str[0] == '0' && (str[1] == 'x' || str[1] == 'X')))
				p_cmd->data = p_cmd->data + 2;
			p_cmd->len = str_to_hex_array(p_cmd->data, strlen(p_cmd->data));
		} else {
			printf("unknown parameter - %s\n", str);            
			return -1;
		}
	}
	return 0;   
}

static int do_pcmcia(cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
	PCMCIA_CMD cmd = {
		.id    = PCMCIA_CMD_HELP,
		.addr = 0,
		.len = 0,
		.data = NULL
	};         
	int i;
    	if (argc > 1)
	{        
		for (i=1; i<argc ; i++)
		{
			if (parse_command(i, argv[i], &cmd)<0)
			{            
				cmd.id = PCMCIA_CMD_HELP;              
				break;
			}
		}
	}     
	do_command(&cmd);      
	return 1; 
}
U_BOOT_CMD(pcmcia, 6, 0, do_pcmcia,
	"Realtek PCMCIA Tool",
	"Format : pcmcia <opcode> [ofst=<ofst>][data=<data>][len=<len>]\n"
	"<opcode> : operations     \n"
	"  -s  : Card Enable              \n"
	"  -f  : Card Disable              \n"
	"  -r  : Card Reset          \n"
	"  -d  : Read IO Mem          \n"
	"  -e  : Write IO Mem    \n"
	"  -c  : Read CIS       \n"
	"  -h  : print help              \n"
	"                                     \n"
	"   <addr>  : the offset of read/write action     \n"                
	"   <data>  : the hex data array     \n"	
	"   <len>  : the len of data array     \n"
	"                                     \n"
	" Example: Card Enable                             \n"	
	"      pcmcia -s                 \n"		
	" Example: Card Disable                             \n"	
	"      pcmcia -f                 \n"	
	" Example: Card Reset                             \n"	
	"      pcmcia -r                 \n"		
	" Example: Read Attr Mem                             \n"	
	"      pcmcia -a addr=<addr> len=<len>\n"	
	" Example: Write Attr Mem                             \n"	
	"      pcmcia -b addr=<addr> data=<data> len=<len>\n"	
	" Example: Read IO Mem                             \n"	
	"      pcmcia -d addr=<addr> len=<len>\n"		
	" Example: Write IO Mem                             \n"	
	"      pcmcia -e addr=<addr> data=<data> len=<len>\n"		
	" Example: Read CIS                             \n"	
	"      pcmcia -c                 \n"
	);
