#include <linux/ctype.h>
#include <common.h>
#include <parse_h.h>
#include <asm/arch/bootparam.h>
#include <asm/arch/fw_info.h>
#include <asm/arch/system.h>
#include <exports.h>
#include <factorylib.h>
#include <platform_lib/board/pcb.h>
#include "cus_common.h"

cus_bootparam_t cus_bootparam;

void CusApi_get_bootparam(char *dst_addr, int dst_length)
{
	char *retAddr = NULL, *firstpos = NULL, *lastpos = NULL;
	uint retVal;

	if (parse_h("BOOT_KEYPAD_LOCK", dst_addr, (uint)dst_length, &retVal, &retAddr) == 0) {
		cus_bootparam.isKeypadLock = retVal;
		printf("BOOT_KEYPAD_LOCK=%d\n", retVal);
	}
	else {
		cus_bootparam.isKeypadLock = 0;
		printf("can't find Keypad Lock\n");
	}

	if (parse_h("BOOT_MAC_ADDRESS", dst_addr, (uint)dst_length, &retVal, &retAddr) == 0) {
		firstpos = strchr(retAddr, '\"');
		if (firstpos)
			lastpos = strchr(firstpos+1, '\"');
		if (firstpos && lastpos)
			strncpy((char *)cus_bootparam.macaddress, firstpos+1, lastpos-firstpos-1);
		else
			strncpy((char *)cus_bootparam.macaddress, retAddr, retVal);

		{
			int str_len = strlen((char *)cus_bootparam.macaddress);
			int i;
			for (i = str_len - 1; i > 0 ; i--) {
				if (isprint(cus_bootparam.macaddress[i]))
					break;
				else
					cus_bootparam.macaddress[i] = '\0';
			}
		}

		printf("BOOT_MAC_ADDRESS=%s\n", cus_bootparam.macaddress);
	}
	else {
		sprintf(cus_bootparam.macaddress,"%s","00:01:02:03:04:05");
		printf("can't find mac address\n");
	}

	if (parse_h("BOOT_BACKLIGHT_MODE", dst_addr, (uint)dst_length, &retVal, &retAddr) == 0) {
		cus_bootparam.backlight_mode = retVal;
		printf("BOOT_BACKLIGHT_MODE=%d\n", retVal);
	}
	else {
		cus_bootparam.backlight_mode = 1;
		printf("can't find backlight mode\n");
	}

	//bt and wifi enable
	if (parse_h("BOOT_BT_ENABLE", dst_addr, (uint)dst_length, &retVal, &retAddr) == 0) {
		cus_bootparam.boot_bt_enable = retVal;
		printf("BOOT_BT_ENABLE=%d\n", retVal);
	}
	else {
		cus_bootparam.boot_bt_enable = 1;
		printf("can't find boot_bt_enable\n");
	}

	if (parse_h("BOOT_WIFI_ENABLE", dst_addr, (uint)dst_length, &retVal, &retAddr) == 0) {
		cus_bootparam.boot_wifi_enable = retVal;
		printf("BOOT_WIFI_ENABLE=%d\n", retVal);
	}
	else {
		cus_bootparam.boot_wifi_enable = 1;
		printf("can't find boot_wifi_enable\n");
	}

	if (parse_h("BOOT_LED_TYPE", dst_addr, (uint)dst_length, &retVal, &retAddr) == 0) {
		char tmp_str[8] = {0};
		char tmp[4] = {0};
		firstpos = strstr(retAddr, "0x");
		if (firstpos)
			lastpos = strchr(firstpos+2, '\n');
		
		if (firstpos && lastpos)
			strncpy(tmp_str, firstpos+2, lastpos-firstpos-2);
		else
			strncpy(tmp_str, retAddr, retVal);

		StrToHex(tmp,tmp_str,4);
		cus_bootparam.boot_led_type = 0;

		for (int i = 0; i < 4 ; i++) 
			cus_bootparam.boot_led_type = (cus_bootparam.boot_led_type<<8)|tmp[i];		
		
		printf("BOOT_LED_TYPE=0x%08x\n", cus_bootparam.boot_led_type);
	}
	else {
		cus_bootparam.boot_led_type = 0;
		printf("can't find boot_led_type\n");
	}

	if (parse_h("STANDY_LED_TYPE", dst_addr, (uint)dst_length, &retVal, &retAddr) == 0) {
		char tmp_str[8] = {0};
		char tmp[4] = {0};
		firstpos = strstr(retAddr, "0x");
		if (firstpos)
			lastpos = strchr(firstpos+2, '\n');
		if (firstpos && lastpos)
			strncpy(tmp_str, firstpos+2, lastpos-firstpos-2);
		else
			strncpy(tmp_str, retAddr, retVal);
		
		StrToHex(tmp,tmp_str,4);
		cus_bootparam.standy_led_type= 0;

		for (int i = 0; i < 4 ; i++) 
			cus_bootparam.standy_led_type = (cus_bootparam.standy_led_type<<8)|tmp[i];
			
		printf("STANDY_LED_TYPE=0x%08x\n", cus_bootparam.standy_led_type);
	}
	else {
		cus_bootparam.standy_led_type= 0;
		printf("can't find standy_led_type\n");
	}

	if (parse_h("LED_PARAM1", dst_addr, (uint)dst_length, &retVal, &retAddr) == 0) {
		char tmp_str[8] = {0};
		char tmp[4] = {0};
		firstpos = strstr(retAddr, "0x");
		if (firstpos)
			lastpos = strchr(firstpos+2, '\n');
		if (firstpos && lastpos)
			strncpy(tmp_str, firstpos+2, lastpos-firstpos-2);
		else
			strncpy(tmp_str, retAddr, retVal);
		
		StrToHex(tmp,tmp_str,4);
		cus_bootparam.led_param1= 0;

		for (int i = 0; i < 4 ; i++) 
			cus_bootparam.led_param1 = (cus_bootparam.led_param1<<8)|tmp[i];

		printf("LED_PARAM1=0x%08x\n", cus_bootparam.led_param1);
	}
	else {
		cus_bootparam.led_param1= 0;
		printf("can't find led_param1\n");
	}
}

void CusApi_decide_power_on_mode(void)
{
	// RTK config don't care prev_power_state and always power on
	boot_power_on_mode = POWER_ON;
	prev_power_on_status = bootparam.prev_power_state;

#if defined(CONFIG_BOOTUP_BY_PREV_POWER_STATUS) || defined(ENABLE_BOOTUP_POLICY)
	if(boot_ac_on)
		boot_power_on_mode = bootparam.prev_power_state;
#endif

#ifdef ENABLE_BOOTUP_POLICY
	printf("bootup_policy=%d\n", bootparam.bootup_policy);
	if(bootparam.bootup_policy != 2)	//0: always powerOff, 1: always powerOn, 2:auto by prev_status
	{
		if(boot_ac_on)
			boot_power_on_mode = bootparam.bootup_policy;
		else
			boot_power_on_mode = POWER_ON;
	}
#endif
#if 1	
	rtd_outl(SHARE_MEMORY_FOR_LED_STANDBY_TYPE,cus_bootparam.boot_led_type);
	//rtd_outl(SHARE_MEMORY_FOR_LED_STANDBY_PARAM1,cus_bootparam.led_param1);
	//refine by Ravi  
//Auxcode disbale EMCU clk by CLKEN=0 > RSTN=0
//so here enable EMCU clk should as the following:
//CLKEN=1 > CLKEN=0 > RSTN=1 > CLKEN=1

	rtd_outl(REG_MAGIC_51, 0x9021afbe);
	rtd_outl(REG_EMCU_CLKEN, _BIT9 | _BIT0);
	rtd_outl(REG_EMCU_CLKEN, _BIT9);
	rtd_outl(REG_EMCU_RESET, _BIT9 | _BIT0);
	rtd_outl(REG_EMCU_CLKEN, _BIT9 | _BIT0);
#endif	
}
