/**********************************************************
***
*** 	include header 
***
***********************************************************/
#include "../../src/include/project_config.h"

#include <common.h>
#include <exports.h>
#include <mmc.h>
#include <command.h>
#include "part_efi.h"
#include <gpt_parse.h>

//#define __DEBBUG_ENABLE__

#if defined(CONFIG_SYS_RTK_EMMC_FLASH)
/**********************************************************
***
*** 	global variable 
***
***********************************************************/
/*record GPT header infomation*/
gpt_header *g_gpt_header_info = NULL;
/*record all GPT partition Information*/
gpt_entry *g_gpt_entry_array = NULL;
/*size of all GPT partition Information*/
unsigned int g_pgt_entry_size = 0;


/**********************************************************
***
***		function body
***
***********************************************************/

/* Convert char[2] in little endian format to the host format integer
 */
static inline unsigned short le16_to_int(unsigned char *le16)
{
	return ((le16[1] << 8) + le16[0]);
}

/* Convert char[4] in little endian format to the host format integer
 */
static inline unsigned long le32_to_int(unsigned char *le32)
{
	return ((le32[3] << 24) + (le32[2] << 16) + (le32[1] << 8) + le32[0]);
}

/* Convert char[8] in little endian format to the host format integer
 */
static inline unsigned long long le64_to_int(unsigned char *le64)
{
	return (((unsigned long long)le64[7] << 56) +
		((unsigned long long)le64[6] << 48) +
		((unsigned long long)le64[5] << 40) +
		((unsigned long long)le64[4] << 32) +
		((unsigned long long)le64[3] << 24) +
		((unsigned long long)le64[2] << 16) +
		((unsigned long long)le64[1] << 8) +
		(unsigned long long)le64[0]);
}

static inline void int_to_le32(unsigned char *le32, unsigned int value)
{
	unsigned int loop = 0;

	for(loop = 0; loop < sizeof(unsigned int); loop++)
		le32[loop] = (unsigned char)((value >> (8*loop)) & 0xff);

	return;
}

static inline void int_to_le64(unsigned char *le64, unsigned long long value)
{
	unsigned int loop = 0;

	for(loop = 0; loop < sizeof(unsigned long long); loop++)
		le64[loop] = (unsigned char)((value >> (8*loop)) & 0xff);

	return;
}

static struct mmc* mmc_info_init(void)
{
	struct mmc *mmc;
	int curr_mmc = -1;
	
	if (get_mmc_num() > 0) {
		curr_mmc = 0;
	} else {
		printf("[%s][%d]No MMC device available\n", __func__, __LINE__);
		return NULL;
	}

	mmc = find_mmc_device(curr_mmc);
	if (!mmc) {
		printf("[%s][%d]no mmc device at slot %x\n", __func__, __LINE__, curr_mmc);
		return NULL;
	}

	return mmc;
}

static int check_pmbr_LBA0(void)
{
	int ret = -1;
	legacy_mbr mbr_table;
	
	ret = rtk_eMMC_read(0, GPT_BLOCK_SIZE, (unsigned int *)&mbr_table);
	if(ret == 0){
		printf("[%s][%d]Read pmbr LBA0 from emmc fail!\r\n", __func__, __LINE__);
		return -1;
	}

	if(le16_to_int(mbr_table.signature) != MSDOS_MBR_SIGNATURE || \
		mbr_table.partition_record[0].sys_ind != EFI_PMBR_OSTYPE_EFI_GPT){
		printf("[%s][%d]Current platfrom partition table format is not GPT!\r\n",__func__, __LINE__);	
		return -1;
	}

	return 0;
}

static int check_gpt_header_LBA1(gpt_header * rtk_gpt_header, gpt_entry * rtk_gpt_entry, 
									unsigned int rtk_gpt_entry_size, unsigned char is_backup)
{
	int ret = -1;
	struct mmc *mmc = NULL;
	unsigned long long total_sector = 0;
	unsigned long long gpt_head_blk = 0;
	unsigned long long lastlba = 0;

	unsigned char crc32_backup[4] = { 0 };
	unsigned long calc_crc32 = 0;
	unsigned long curr_crc32 = 0;

	unsigned int main_part_entry_invalid = 0;
	unsigned long long part_entry_blkoff = 0;
	unsigned int entry_size = 0;

	/*Init mmc info*/
	mmc = mmc_info_init();
	if(mmc == NULL){
		printf("[%s][%d]mmc info init fail!\r\n", __func__, __LINE__);
		return -1;
	}
	
	total_sector = mmc->capacity >> 9;
	if(is_backup)
		gpt_head_blk = total_sector -1;
	else
		gpt_head_blk = 0x1;

	ret = rtk_eMMC_read((unsigned int)gpt_head_blk, GPT_BLOCK_SIZE, (unsigned int*)rtk_gpt_header);
	if(ret == 0){
		printf("[%s][%d]Read GPT header from emmc fail!\r\n", __func__, __LINE__);
		return -1;
	}

	/* Check the GPT header signature */
	if (le64_to_int(rtk_gpt_header->signature) != GPT_HEADER_SIGNATURE) {
		printf("GUID Partition Table Header signature is wrong:"
			"0x%llx != 0x%llx\n",
			(unsigned long long)le64_to_int(rtk_gpt_header->signature),
			(unsigned long long)GPT_HEADER_SIGNATURE);
		return -1;
	}

	/* Check the GPT header  size*/
	if(le32_to_int(rtk_gpt_header->header_size) != \
		(sizeof(gpt_header) - sizeof(rtk_gpt_header->reserved2))){
		printf("GUID Partition Table Header Size is wrong:"
			"0x%x != 0x%x\n",
			(unsigned int)le32_to_int(rtk_gpt_header->header_size),
			(unsigned int)(sizeof(gpt_header) - sizeof(rtk_gpt_header->reserved2)));
		return -1;
	}

	/* Check the GUID Partition Table CRC */
	memcpy(crc32_backup, rtk_gpt_header->header_crc32, sizeof(crc32_backup));
	memset(rtk_gpt_header->header_crc32, 0, sizeof(rtk_gpt_header->header_crc32));
	
	calc_crc32 = crc32(0, (const unsigned char *)rtk_gpt_header, le32_to_int(rtk_gpt_header->header_size));
	memcpy(rtk_gpt_header->header_crc32, crc32_backup, sizeof(crc32_backup));
	if (calc_crc32 != le32_to_int(crc32_backup)) {
		printf("GUID Partition Table Header CRC is wrong:"
			"0x%08lX != 0x%08lX\n",
			le32_to_int(crc32_backup), calc_crc32);
		return -1;
	}

	/* Check that the my_lba entry points to the LBA that contains the GPT */
	if (le64_to_int(rtk_gpt_header->my_lba) != gpt_head_blk) {
		printf("GPT: my_lba incorrect: %llX != %llX\n",
			(unsigned long long)le64_to_int(rtk_gpt_header->my_lba),
			(unsigned long long)gpt_head_blk);
		return -1;
	}

	/* Check the first_usable_lba and last_usable_lba are within the disk. */
	lastlba = total_sector -1;
	if (le64_to_int(rtk_gpt_header->first_usable_lba) > lastlba) {
		printf("GPT: first_usable_lba incorrect: %llX > %llX\n",
			le64_to_int(rtk_gpt_header->first_usable_lba), lastlba);
		return -1;
	}
	if (le64_to_int(rtk_gpt_header->last_usable_lba) > lastlba) {
		printf("GPT: last_usable_lba incorrect: %llX > %llX\n",
			le64_to_int(rtk_gpt_header->last_usable_lba), lastlba);
		return -1;
	}

#ifdef __DEBBUG_ENABLE__
	printf("mmc totol count: 0x%llx\r\n", total_sector);

	printf("my_lba: 0x%llx \r\n", le64_to_int(rtk_gpt_header->my_lba));
	printf("alternate_lba: 0x%llx \r\n", le64_to_int(rtk_gpt_header->alternate_lba));

	printf("first_usable_lba: 0x%llx \r\n", le64_to_int(rtk_gpt_header->first_usable_lba));
	printf("last_usable_lba: 0x%llx \r\n", le64_to_int(rtk_gpt_header->last_usable_lba));
	
	printf("partition_entry_lba: 0x%llx \r\n", le64_to_int(rtk_gpt_header->partition_entry_lba));
	printf("num_partition_entries: 0x%lx \r\n", le32_to_int(rtk_gpt_header->num_partition_entries));

	printf("header_crc32: 0x%lx \r\n", le32_to_int(rtk_gpt_header->header_crc32));
	printf("header_size: 0x%lx \r\n", le32_to_int(rtk_gpt_header->header_size));
	printf("partition_entry_array_crc32: 0x%lx \r\n", le32_to_int(rtk_gpt_header->partition_entry_array_crc32));
#endif

retry:	
	/* Check the GUID Partition Table Entry Array CRC */
	part_entry_blkoff = main_part_entry_invalid ? \
				(le64_to_int(rtk_gpt_header->last_usable_lba) + 1):le64_to_int(rtk_gpt_header->partition_entry_lba);
	entry_size = le32_to_int(rtk_gpt_header->num_partition_entries) *  \
					le32_to_int(rtk_gpt_header->sizeof_partition_entry);
	entry_size = (entry_size % GPT_BLOCK_SIZE) ? ((entry_size / GPT_BLOCK_SIZE + 1) * GPT_BLOCK_SIZE): entry_size;
	if (entry_size > rtk_gpt_entry_size)
		entry_size = rtk_gpt_entry_size;
	ret = rtk_eMMC_read((unsigned int)part_entry_blkoff, entry_size, (unsigned int *)rtk_gpt_entry);
	if(ret == 0){
		printf("[%s][%d]Read GPT partition entry from emmc fail!\r\n", __func__, __LINE__);
		return -1;
	}

	curr_crc32 = le32_to_int(rtk_gpt_header->partition_entry_array_crc32);
	calc_crc32 = crc32(0, (const unsigned char *)rtk_gpt_entry, 
			le32_to_int(rtk_gpt_header->num_partition_entries) * le32_to_int(rtk_gpt_header->sizeof_partition_entry));	
	if(curr_crc32 != calc_crc32){
		printf("%s GUID Partition Table Entry Array CRC is wrong:"
					"0x%08lX != 0x%08lX\n",
					main_part_entry_invalid ? "Primiry": "Main",
					le32_to_int(rtk_gpt_header->partition_entry_array_crc32),
					calc_crc32);

		if(main_part_entry_invalid == 0){
			main_part_entry_invalid = 1;
			goto retry;
		}else{
			return -1;
		}
	}

	return 0;
}

static int check_gpt_context(gpt_header * rtk_gpt_header, gpt_entry * rtk_gpt_entry, unsigned int rtk_gpt_entry_size, unsigned int *is_primiry_valid)
{
	int ret = -1;

	ret = check_pmbr_LBA0();
	if(ret < 0){
		printf("[%s][%d]check pmbr LBA0 fail!\r\n",__func__, __LINE__);
		return -1;
	}

	ret = check_gpt_header_LBA1(rtk_gpt_header, rtk_gpt_entry, rtk_gpt_entry_size, 0);
	if(ret < 0){
		printf("[%s][%d]check main GPT fail!\r\n",__func__, __LINE__);
		ret = check_gpt_header_LBA1(rtk_gpt_header, rtk_gpt_entry, rtk_gpt_entry_size, 1);
		if(ret < 0){
			printf("[%s][%d]check primiry GPT fail!\r\n",__func__, __LINE__);
			return -1;
		}else{
			*is_primiry_valid = 1;
		}
	}else{
		*is_primiry_valid = 0;
	}

	return 0;
}

static void parse_part_name(char *gpt_name, char *str_name, int gpt_name_len)
{
	int i = 0;

	for(i = 0; i< gpt_name_len; i++){
		str_name[i] = gpt_name[2*i];
	}

	return;
}

static int init_global_gpt_partition_info(void)
{
	int ret = 0;
	static unsigned char s_gpt_info_init_done = 0;
	unsigned int is_primiry_valid = 0;

	/*check gpt info init finish or not*/
	if(s_gpt_info_init_done){
		return 0;
	}

	/* malloc buffer to store gpt header and part entry */ 
	g_gpt_header_info = (gpt_header *)malloc(sizeof(gpt_header));
	if(g_gpt_header_info == NULL){
		printf("[%s][%d]malloc buffer to store gpt header fail!\r\n",__func__, __LINE__);
		ret = -1;
		goto fail;
	}else{
		memset(g_gpt_header_info, 0x00, sizeof(gpt_header));
	}

	g_pgt_entry_size = GPT_SUPPORT_MAX_PART_NUM * sizeof(gpt_entry);
	g_gpt_entry_array = (gpt_entry *)malloc(g_pgt_entry_size);
	if(g_gpt_entry_array == NULL){
		printf("[%s][%d]malloc buffer to store gpt part entry fail!\r\n",__func__, __LINE__);
		ret =  -1;
		goto fail;
	}else{
		memset(g_gpt_entry_array, 0x00, g_pgt_entry_size);
	}

	/* check GPT context is valid or not  and get GPT info*/
	ret = check_gpt_context(g_gpt_header_info, g_gpt_entry_array, g_pgt_entry_size, &is_primiry_valid);
	if(ret < 0){
		printf("[%s][%d]GPT context check fail!\r\n",__func__, __LINE__);
		goto fail;
	}

	s_gpt_info_init_done++;
	return 0;
	
fail:
	s_gpt_info_init_done = 0;
	if(g_gpt_header_info)
		free(g_gpt_header_info);
	if(g_gpt_entry_array)
		free(g_gpt_entry_array);
	g_gpt_header_info = NULL;
	g_gpt_entry_array = NULL;
	return -1;
}
#endif

int get_gpt_entry_num(unsigned int *entry_num)
{
#if defined(CONFIG_SYS_RTK_EMMC_FLASH)
	int ret = 0;

	ret = init_global_gpt_partition_info();
	if(ret < 0){
		printf("[%s][%d]parse gpt partition info fail!\r\n", __func__, __LINE__);
		return -1;
	}

	*entry_num = le32_to_int(g_gpt_header_info->num_partition_entries);
	return 0;
#else
	printf("[ERROR] Not support %s!\r\n", __func__);
	return -1;
#endif
}

int get_gpt_entry_by_index(gpt_entry *entry, unsigned int part_index)
{
#if defined(CONFIG_SYS_RTK_EMMC_FLASH)
	int ret = 0;
	unsigned int part_total_num = 0;

	if(entry == NULL){
		printf("[%s][%d]check parameter fail!\r\n",__func__, __LINE__);
		return -1;
	}

	ret = init_global_gpt_partition_info();
	if(ret < 0){
		printf("[%s][%d]parse gpt partition info fail!\r\n", __func__, __LINE__);
		return -1;
	}

	part_total_num = le32_to_int(g_gpt_header_info->num_partition_entries);
	if(part_index >= part_total_num){
		printf("[%s][%d]part index beyond the max index!\r\n", __func__, __LINE__);
		return -1;
	}

	memcpy((void *)entry, (void *)&g_gpt_entry_array[part_index], sizeof(gpt_entry));
	return 0;
#else
	printf("[ERROR] Not support %s!\r\n", __func__);
	return -1;
#endif
}

int get_gpt_entry_by_name(gpt_entry *entry, char *part_name)
{
#if defined(CONFIG_SYS_RTK_EMMC_FLASH)
	int ret = 0;
	unsigned int part_num = 0;
	char cur_part_name[72] = {0};
	unsigned int loop = 0;
	int hit_flag = 0;

	if(entry == NULL || part_name == NULL){
		printf("[%s][%d]check parameter fail!\r\n",__func__, __LINE__);
		return -1;
	}

	ret = init_global_gpt_partition_info();
	if(ret < 0){
		printf("[%s][%d]parse gpt partition info fail!\r\n", __func__, __LINE__);
		return -1;
	}

	part_num = le32_to_int(g_gpt_header_info->num_partition_entries);

	/* check target partition is exist or not*/
	for(loop = 0; loop < part_num; loop++){
		memset(cur_part_name, 0x00, sizeof(cur_part_name));
		parse_part_name((char *)&(g_gpt_entry_array[loop].partition_name), cur_part_name, sizeof(g_gpt_entry_array[loop].partition_name));
		if(strcmp(part_name, cur_part_name) == 0){
			memcpy((void *)entry, (void *)&g_gpt_entry_array[loop], sizeof(gpt_entry));
			hit_flag = 1;
			break;
		}
	}

	if(hit_flag)
		return 0;
	else
		return -1;
#else
	printf("[ERROR] Not support %s!\r\n", __func__);
	return -1;
#endif
}

int get_partition_partname_by_gpt_entry(gpt_entry entry, char *part_name)
{
#if defined(CONFIG_SYS_RTK_EMMC_FLASH)

	char cur_part_name[72] = {0};

	memset(cur_part_name, 0x00, sizeof(cur_part_name));
	parse_part_name((char *)entry.partition_name, cur_part_name, sizeof(entry.partition_name));

	strcpy(part_name, (const char *)cur_part_name);
	return 0;
#else
	printf("[ERROR] Not support %s!\r\n", __func__);
	return -1;
#endif
}

unsigned long long get_partition_starting_by_gpt_entry(gpt_entry entry)
{
#if defined(CONFIG_SYS_RTK_EMMC_FLASH)
	return le64_to_int(entry.starting_lba)*0x200; 
#else
	printf("[ERROR] Not support %s!\r\n", __func__);
	return 0;
#endif
}

unsigned long long get_partition_ending_by_gpt_entry(gpt_entry entry)
{
#if defined(CONFIG_SYS_RTK_EMMC_FLASH)
	return (le64_to_int(entry.ending_lba) + 1)*0x200;	
#else
	printf("[ERROR] Not support %s!\r\n", __func__);
	return 0;
#endif
}

int get_dev_index_by_name(char *part_name, unsigned int *dev_index)
{
#if defined(CONFIG_SYS_RTK_EMMC_FLASH)	
	int ret = 0;
	unsigned int part_num = 0;
	char cur_part_name[72] = {0};
	unsigned int loop = 0;
	int hit_flag = 0;

	if(dev_index == NULL || part_name == NULL){
		printf("[%s][%d]check parameter fail!\r\n",__func__, __LINE__);
		return -1;
	}

	ret = init_global_gpt_partition_info();
	if(ret < 0){
		printf("[%s][%d]parse gpt partition info fail!\r\n", __func__, __LINE__);
		return -1;
	}

	part_num = le32_to_int(g_gpt_header_info->num_partition_entries);

	/* check target partition is exist or not*/
	for(loop = 0; loop < part_num; loop++){
		memset(cur_part_name, 0x00, sizeof(cur_part_name));
		parse_part_name((char *)&(g_gpt_entry_array[loop].partition_name), cur_part_name, sizeof(g_gpt_entry_array[loop].partition_name));
		if(strcmp(part_name, cur_part_name) == 0){
			hit_flag = 1;
			break;
		}
	}

	if(hit_flag){
		*dev_index = loop + 1;
		return 0;
	}else{
		return -1;
	}
#else
	printf("[ERROR] Not support %s!\r\n", __func__);
	return -1;
#endif
}


#ifdef __DEBBUG_ENABLE__
static int do_part_parse(cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
	int ret = -1;
	unsigned int part_num = 0;
	unsigned int loop = 0;
	gpt_entry gpt_entry;
	char partname[256] = {0};
	unsigned long long start = 0, end = 0;
	
	if((argc != 2) && (strcmp(argv[1], "parse") != 0))
		return CMD_RET_USAGE;

	ret = get_gpt_entry_num(&part_num);
	if(ret < 0){
		printf("[%s][%d]get gpt entry num fail!\r\n", __func__, __LINE__);
		return CMD_RET_FAILURE;
	}

	printf("part num = 0x%x \r\n", part_num);

	for(loop = 0; loop < part_num; loop++){
		get_gpt_entry_by_index(&gpt_entry, loop);
		
		start = get_partition_starting_by_gpt_entry(gpt_entry);
		end = get_partition_ending_by_gpt_entry(gpt_entry);
		get_partition_partname_by_gpt_entry(gpt_entry, partname);

		printf("partname: %s  0x%llx~0x%llx \r\n", partname, start, end);
	}


	get_gpt_entry_by_name(&gpt_entry, "data");
	start = get_partition_starting_by_gpt_entry(gpt_entry);
	end = get_partition_ending_by_gpt_entry(gpt_entry);
	get_partition_partname_by_gpt_entry(gpt_entry, partname);

	printf("partname: %s  0x%llx~0x%llx \r\n", partname, start, end);
		
	
	get_gpt_entry_by_name(&gpt_entry, "vendor");
	start = get_partition_starting_by_gpt_entry(gpt_entry);
	end = get_partition_ending_by_gpt_entry(gpt_entry);
	get_partition_partname_by_gpt_entry(gpt_entry, partname);
	
	printf("partname: %s  0x%llx~0x%llx \r\n", partname, start, end);

	/*test get dev index*/
	ret = get_dev_index_by_name("vendor", &part_num);
	if(ret < 0){
		printf("get vendor block device index fail!\r\n");
	}else{
		printf("vendor block device index is %d \r\n", part_num);
	}

	ret = get_dev_index_by_name("system", &part_num);
	if(ret < 0){
		printf("get system block device index fail!\r\n");
	}else{
		printf("system block device index is %d \r\n", part_num);
	}

	ret = get_dev_index_by_name("fault", &part_num);
	if(ret < 0){
		printf("get fault block device index fail!\r\n");
	}else{
		printf("fault block device index is %d \r\n", part_num);
	}
	
	return CMD_RET_SUCCESS;
}

U_BOOT_CMD(
	rtkpart, 2, 1, do_part_parse,
	"parse partition information",
	"\n"
	"rtkpart parse - parse partition info\n"
);
#endif
