#include <common.h>
#include <drivers/pwm/pwm.h>
#include <asm/arch/io.h>
#include <platform_lib/board/pcb_mgr.h>

int pwm_init(int index, int type, int pwm_freq_hz, int duty_max, int duty, int invert)
{
	int i;
	long  source_clock, fclk, dutyMax, pwmFreqHz, error, error_best;
	unsigned short N, N_best = N_MAX, M_best = M_MAX;
	pwm0_ctrl_RBUS pwm_ctrl_reg;
	pwm0_timing_ctrl_RBUS pwm_timing_ctrl_reg;
	pwm0_duty_set_RBUS 	pwm_duty_set_reg;
	int regAddr = 0;
	int regval = 0;

	if (type == PCB_PIN_TYPE_PWM)
	{
		if (index >= 8) return -1;
		printf("pwm_freq_hz = %d\n", pwm_freq_hz);
	} else {
		if (index >= 2) return -1;
		printf("iso_pwm_freq_hz = %d\n", pwm_freq_hz);
	}

	source_clock = CONFIG_CPU_XTAL_FREQ;
	//fclk = duty_max * pwm_freq_hz;
	// fclk = (Crystal clock) / 2^M / (N+1)
	dutyMax = duty_max + 1;
	pwmFreqHz = pwm_freq_hz;
	fclk = dutyMax  * pwmFreqHz;
	//printf("-- fclk = 0x%llx,  %lld, %lld\n", fclk, dutyMax, pwmFreqHz);

	printf("fclk max(M=0,N=0)=0x%lx, min(M=3,N=0xFFF)=0x%lx\n", source_clock, ((source_clock / ( 1 << M_MAX)) / (N_MAX+1)));
	if (fclk > CONFIG_CPU_XTAL_FREQ)
	{
		N_best = 0x0;
		M_best = 0x0;
	}

	error_best = fclk;
	for (i=0; i<=M_MAX; i++) {
		N = ((source_clock / ( 1 << i)) / fclk);
		printf("(M=%d)-- N = 0x%x,  %d, 0x%lx, 0x%lx\n", i, N, source_clock, (source_clock / ( 1 << i) ), fclk);
		if (N == 0)
			N=1;
		if (N > (N_MAX+1))
			N=(N_MAX+1);
		error = ABS((source_clock / ( 1 << i )) / (N), fclk);

		printf("-- error CLK = 0x%lx, fclk = 0x%lx\n", (source_clock / ( 1 << i )) / (N),  fclk);
		printf("-- error = 0x%lx, error_best = 0x%lx\n", error,  error_best);
		if (error < error_best) {
			error_best = error;
			N_best = (unsigned short)N;;
			M_best = i;
		}
		printf("-- N_best = 0x%x, M_best = 0x%x\n", N_best,  M_best);
	}

	printf("M = 0x%x\n", M_best);
	printf("N = 0x%x\n", N_best);
	if (type == PCB_PIN_TYPE_PWM)
	{
		pwm_timing_ctrl_reg.regValue = rtd_inl(pwm_mis_phy[index].p_reg_map->TIMING_CTRL);
		pwm_duty_set_reg.regValue = rtd_inl(pwm_mis_phy[index].p_reg_map->DUTY_SET);
		pwm_ctrl_reg.regValue = rtd_inl(pwm_mis_phy[index].p_reg_map->CTRL);

	} else {
		pwm_timing_ctrl_reg.regValue = rtd_inl(pwm_iso_phy[index].p_reg_map->TIMING_CTRL);
		pwm_duty_set_reg.regValue = rtd_inl(pwm_iso_phy[index].p_reg_map->DUTY_SET);
		pwm_ctrl_reg.regValue = rtd_inl(pwm_iso_phy[index].p_reg_map->CTRL);
	}

	pwm_timing_ctrl_reg.pwm0_cnt_mode = 1;
	pwm_timing_ctrl_reg.pwm0_ck_sel = 0;
	pwm_timing_ctrl_reg.pwm0_m = M_best;
	pwm_timing_ctrl_reg.pwm0_n = N_best;

	pwm_duty_set_reg.pwm0_totalcnt = duty_max;
	pwm_duty_set_reg.pwm0_dut = duty;

	pwm_ctrl_reg.regValue = 0; //rtd_inl(PWM0_CTRL_reg + index * 12);
	pwm_ctrl_reg.pwm0_en = 1;
	pwm_ctrl_reg.pwm0l = (invert != 0);
	pwm_ctrl_reg.pwm0_ck = 1;
	pwm_ctrl_reg.pwm0_dut_8b = 0;

	if (type == PCB_PIN_TYPE_PWM)
	{
		rtd_outl(pwm_mis_phy[index].p_reg_map->CTRL,pwm_ctrl_reg.regValue);
		rtd_outl(pwm_mis_phy[index].p_reg_map->TIMING_CTRL,pwm_timing_ctrl_reg.regValue);
		rtd_outl(pwm_mis_phy[index].p_reg_map->DUTY_SET,pwm_duty_set_reg.regValue);

		regAddr = pwm_mis_phy[index].p_reg_map->CTRL;
		regval = rtd_inl(regAddr);
		pwmDebug("\033[1;32m reg 0x%x=0x%x\033[m\n", regAddr, regval);
		regAddr = pwm_mis_phy[index].p_reg_map->TIMING_CTRL;
		regval = rtd_inl(regAddr);
		pwmDebug("\033[1;32m reg 0x%x=0x%x\033[m\n", regAddr, regval);
		regAddr = pwm_mis_phy[index].p_reg_map->DUTY_SET;
		regval = rtd_inl(regAddr);
		pwmDebug("\033[1;32m reg 0x%x=0x%x\033[m\n", regAddr, regval);

	} else {
		rtd_outl(pwm_iso_phy[index].p_reg_map->CTRL,pwm_ctrl_reg.regValue);
		rtd_outl(pwm_iso_phy[index].p_reg_map->TIMING_CTRL,pwm_timing_ctrl_reg.regValue);
		rtd_outl(pwm_iso_phy[index].p_reg_map->DUTY_SET,pwm_duty_set_reg.regValue);

		regAddr = pwm_iso_phy[index].p_reg_map->CTRL;
		regval = rtd_inl(regAddr);
		pwmDebug("\033[1;32m reg 0x%x=0x%x\033[m\n", regAddr, regval);
		regAddr = pwm_iso_phy[index].p_reg_map->TIMING_CTRL;
		regval = rtd_inl(regAddr);
		pwmDebug("\033[1;32m reg 0x%x=0x%x\033[m\n", regAddr, regval);
		regAddr = pwm_iso_phy[index].p_reg_map->DUTY_SET;
		regval = rtd_inl(regAddr);
		pwmDebug("\033[1;32m reg 0x%x=0x%x\033[m\n", regAddr, regval);
	}

	return 0;
}

void rtk_hw_get_divider(int pwm_freq_hz, int duty_max,
			unsigned short *M_best, unsigned short *N_best)
{
	int i = 0;
	unsigned long ftmp = 0, error = 0, error_best = 0;
	unsigned short N = 0;
	if (pwm_freq_hz == 3) {
		*M_best = 0x3;
		*N_best = 0xfff;
	} else {
		ftmp = (duty_max + 1) * pwm_freq_hz + 1;
		error_best = ftmp;
		for (i = 0; i <= M_MAX; i++) {
			N = (CONFIG_CPU_XTAL_FREQ / (1 << i)) / ftmp;
			if (N <= 0)
				N = 1;

			if (N > 0x1000)
				continue;

			error = ABS(CONFIG_CPU_XTAL_FREQ / (1 << i) / N, ftmp);
			if (error < error_best) {
				error_best = error;
				*N_best = N - 1;
				*M_best = i;
			}
		}
	}
	printf("%s M_best:%d N_best:%d freq:%d duty_max:%d\n",__FUNCTION__,*M_best,*N_best,pwm_freq_hz,duty_max);
}


int get_pwm_frequency(RTK_PWM_TYPE type, int index)
{
	int freq = MIN_PWM_FREQ;
	int duty_max = 255;
	unsigned long ftmp = 0;
	unsigned short N = 0, M = 0;

	unsigned int registerAddress = 0;

	pwm0_timing_ctrl_RBUS pwm_timing_ctrl_reg;

	if( PWM_ISO == type  && index < ISO_PWM_MAX ) {
		registerAddress = pwm_iso_phy[index].p_reg_map->TIMING_CTRL;
	} else if( PWM_MISC == type && index < MISC_PWM_MAX ) {
		registerAddress = pwm_mis_phy[index].p_reg_map->TIMING_CTRL;
	} else {
		printf("%s parameter fail register address 0x%X !\n",  __func__ , registerAddress );
		printf("%s %d  but  ISO_PWM_MAX =%d !\n", __func__ , index , ISO_PWM_MAX );
		printf("%s %d  but MISC_PWM_MAX =%d !\n",  __func__ , index , MISC_PWM_MAX );
		return -1;
	}

	pwm_timing_ctrl_reg.regValue =rtd_inl(registerAddress);
	M = pwm_timing_ctrl_reg.pwm0_m;
	N = pwm_timing_ctrl_reg.pwm0_n;

	freq = (CONFIG_CPU_XTAL_FREQ /(1 << M)) / (N+1)/duty_max;

	if (freq <= MIN_PWM_FREQ || freq >= MAX_PWM_FREQ)
		printf("get pwm freq=%d invalid!!\n", freq);
	else
		printf("get pwm freq=%d success!!\n", freq);

	return 0;
}

int set_pwm_frequency(RTK_PWM_TYPE type, int index,int freq)
{
	int duty_max = 255;
	unsigned short N = 0, M = 0;

	unsigned int registerAddress = 0;

	pwm0_timing_ctrl_RBUS pwm_timing_ctrl_reg;


	if (freq <= MIN_PWM_FREQ)
		freq = MIN_PWM_FREQ;/* minimal is 3.2, AP won't set this?! */
	else if (freq >= MAX_PWM_FREQ)
		freq = MAX_PWM_FREQ;

	rtk_hw_get_divider(freq,duty_max, &M, &N);

	if( PWM_ISO == type  && index < ISO_PWM_MAX ) {
		registerAddress = pwm_iso_phy[index].p_reg_map->TIMING_CTRL;
	} else if( PWM_MISC == type && index < MISC_PWM_MAX ) {
		registerAddress = pwm_mis_phy[index].p_reg_map->TIMING_CTRL;
	}else{
		printf("%s parameter fail register address 0x%X !\n",  __func__ , registerAddress );
		printf("%s %d  but  ISO_PWM_MAX =%d !\n", __func__ , index , ISO_PWM_MAX );
		printf("%s %d  but MISC_PWM_MAX =%d !\n",  __func__ , index , MISC_PWM_MAX );
		return -1;
	}


	pwm_timing_ctrl_reg.regValue =rtd_inl(registerAddress);
	pwm_timing_ctrl_reg.pwm0_m = M;
	pwm_timing_ctrl_reg.pwm0_n = N;

	rtd_outl(registerAddress,pwm_timing_ctrl_reg.regValue);

	return 0;
}


int get_pwm_duty(RTK_PWM_TYPE type, int index)
{
	pwm0_duty_set_RBUS pwm_duty_set_reg;
	int duty;
	int duty_max;

	unsigned int registerAddress = 0;

	if( PWM_ISO == type  && index < ISO_PWM_MAX ) {
		registerAddress = pwm_iso_phy[index].p_reg_map->DUTY_SET;
	} else if( PWM_MISC == type && index < MISC_PWM_MAX ) {
		registerAddress = pwm_mis_phy[index].p_reg_map->DUTY_SET;
	}else{
		printf("%s parameter fail register address 0x%X !\n",  __func__ , registerAddress );
		printf("%s %d  but  ISO_PWM_MAX =%d !\n", __func__ , index , ISO_PWM_MAX );
		printf("%s %d  but MISC_PWM_MAX =%d !\n",  __func__ , index , MISC_PWM_MAX );
		return -1;
	}

	pwm_duty_set_reg.regValue = rtd_inl( registerAddress );
 	duty = pwm_duty_set_reg.pwm0_dut;
	duty_max = pwm_duty_set_reg.pwm0_totalcnt;

	if( duty > 255 || duty < 0 )
		printf("get pwm duty=%d duty_max=%d invalid!!\n\n", duty , duty_max );
	else
		printf("get pwm duty=%d duty_max=%d success!!\n\n", duty , duty_max);

	return 0;
}

int set_pwm_duty(RTK_PWM_TYPE type, int index, int duty)
{
	pwm0_duty_set_RBUS pwm_duty_set_reg;

	int duty_max;

	unsigned int registerAddress = 0;

	if( duty > 255 || duty < 0 ){
		printf("set pwm duty=%d  failed!!\n\n", duty );
		return -1;
	}else
		printf("set pwm duty=%d  success!!\n\n", duty );


	if( PWM_ISO == type  && index < ISO_PWM_MAX ) {
		registerAddress = pwm_iso_phy[index].p_reg_map->DUTY_SET;
	} else if( PWM_MISC == type && index < MISC_PWM_MAX ) {
		registerAddress = pwm_mis_phy[index].p_reg_map->DUTY_SET;
	}else{
		printf("%s parameter fail register address 0x%X !\n",  __func__ , registerAddress );
		printf("%s %d  but  ISO_PWM_MAX =%d !\n", __func__ , index , ISO_PWM_MAX );
		printf("%s %d  but MISC_PWM_MAX =%d !\n",  __func__ , index , MISC_PWM_MAX );
		return -1;
	}


	pwm_duty_set_reg.regValue = rtd_inl( registerAddress );
	duty_max = pwm_duty_set_reg.pwm0_totalcnt;
	if (duty >= duty_max) duty = duty_max;
	if (duty <= 0) duty = 0;
	pwm_duty_set_reg.pwm0_dut = duty;
	rtd_outl(registerAddress , pwm_duty_set_reg.regValue);

	return 0;
}

int get_pwm_enable(RTK_PWM_TYPE type, int index)
{
	pwm0_ctrl_RBUS pwm_ctrl_reg;
	int enable;

	unsigned int registerAddress = 0;

	if( PWM_ISO == type  && index < ISO_PWM_MAX ) {
		registerAddress = pwm_iso_phy[index].p_reg_map->CTRL;
	} else if( PWM_MISC == type && index < MISC_PWM_MAX ) {
		registerAddress = pwm_mis_phy[index].p_reg_map->CTRL;
	}else{
		printf("%s parameter fail register address 0x%X !\n",  __func__ , registerAddress );
		printf("%s %d  but  ISO_PWM_MAX =%d !\n", __func__ , index , ISO_PWM_MAX );
		printf("%s %d  but MISC_PWM_MAX =%d !\n",  __func__ , index , MISC_PWM_MAX );
		return -1;
	}

	pwm_ctrl_reg.regValue = rtd_inl( registerAddress );
	enable = pwm_ctrl_reg.pwm0_en;

	printf("get pwm enable=%d success!!\n",enable);
	return 0;
}

int set_pwm_enable(RTK_PWM_TYPE type, int index, int enable)
{
	pwm0_ctrl_RBUS pwm_ctrl_reg;

	unsigned int registerAddress = 0;

	if( PWM_ISO == type  && index < ISO_PWM_MAX ) {
		registerAddress = pwm_iso_phy[index].p_reg_map->CTRL;
	} else if( PWM_MISC == type && index < MISC_PWM_MAX ) {
		registerAddress = pwm_mis_phy[index].p_reg_map->CTRL;
	}else{
		printf("%s parameter fail register address 0x%X !\n",  __func__ , registerAddress );
		printf("%s %d  but  ISO_PWM_MAX =%d !\n", __func__ , index , ISO_PWM_MAX );
		printf("%s %d  but MISC_PWM_MAX =%d !\n",  __func__ , index , MISC_PWM_MAX );
		return -1;
	}


	if (enable == 0 || enable == 1){
		pwm_ctrl_reg.pwm0_en = enable;
		rtd_outl( registerAddress , pwm_ctrl_reg.regValue);
	}
	else
		printf("ser pwm enable=%d invalid!!\n",enable);

	return 0;
}


int get_pwm_sync(RTK_PWM_TYPE type, int index)
{
	pwm0_ctrl_RBUS pwm_ctrl_reg;
	int rtk_vsync;

	unsigned int registerAddress = 0;

	if( PWM_ISO == type  && index < ISO_PWM_MAX ) {
		registerAddress = pwm_iso_phy[index].p_reg_map->CTRL;
	} else if( PWM_MISC == type && index < MISC_PWM_MAX ) {
		registerAddress = pwm_mis_phy[index].p_reg_map->CTRL;
	}else{
		printf("%s parameter fail register address 0x%X !\n",  __func__ , registerAddress );
		printf("%s %d  but  ISO_PWM_MAX =%d !\n", __func__ , index , ISO_PWM_MAX );
		printf("%s %d  but MISC_PWM_MAX =%d !\n",  __func__ , index , MISC_PWM_MAX );
		return -1;
	}

	pwm_ctrl_reg.regValue = rtd_inl(registerAddress);
 	rtk_vsync = pwm_ctrl_reg.pwm0_vs_rst_en;

	printf("get pwm sync=%d success!!\n",rtk_vsync);

	return 0;
}

int set_pwm_sync(RTK_PWM_TYPE type, int index,int rtk_vsync)
{
	pwm0_ctrl_RBUS pwm_ctrl_reg;

	unsigned int registerAddress = 0;

	if( PWM_ISO == type  && index < ISO_PWM_MAX ) {
		registerAddress = pwm_iso_phy[index].p_reg_map->CTRL;
	} else if( PWM_MISC == type && index < MISC_PWM_MAX ) {
		registerAddress = pwm_mis_phy[index].p_reg_map->CTRL;
	}else{
		printf("%s parameter fail register address 0x%X !\n",  __func__ , registerAddress );
		printf("%s %d  but  ISO_PWM_MAX =%d !\n", __func__ , index , ISO_PWM_MAX );
		printf("%s %d  but MISC_PWM_MAX =%d !\n",  __func__ , index , MISC_PWM_MAX );
		return -1;
	}


	if (rtk_vsync == 0 || rtk_vsync == 1){
		pwm_ctrl_reg.pwm0_vs_rst_en = rtk_vsync;
		rtd_outl( registerAddress , pwm_ctrl_reg.regValue);
	}
	else
		printf("ser pwm sync=%d invalid!!\n",rtk_vsync);

	return 0;
}

#if defined(CONFIG_RTD289X) || defined(CONFIG_RTD284X)
int rtk_pwm_set_para(RTK_PWM_INFO_T* param) {
	unsigned short M = 0, N = 0;
	unsigned int phase_us= 0, pos =0, tfreq=0;
	const rtk_pwm_reg_map* pp =NULL;
	int i = 0;

	pwmDebug("[pwm] %s\n",__func__);

	pwm0_ctrl_RBUS pwm_ctrl_reg;
	pwm0_duty_set_RBUS pwm_duty_set_reg;
	pwm0_db_ctrl_RBUS dbReg;
	pwm0_timing_ctrl_RBUS pwm_timing_ctrl_reg;

	if(param == NULL) {
		printf("[pwm] %s, get null value.\n",__func__);
		return -1;
	}


	if( PWM_ISO == param->m_type  && param->m_index < ISO_PWM_MAX ){

		pp = pwm_iso_phy[param->m_index].p_reg_map;
	} else if( PWM_MISC == param->m_type  && param->m_index <  MISC_PWM_MAX ){

		pp = pwm_mis_phy[param->m_index].p_reg_map;
	}else{
		printf("%s %d  but  ISO_PWM_MAX =%d !\n", __func__ , param->m_index , ISO_PWM_MAX );
		printf("%s %d  but MISC_PWM_MAX =%d !\n",  __func__ , param->m_index , MISC_PWM_MAX );
		return -1;
	}

	if( pp == NULL )
		return -1;

	/*set timing ctrl , freq */
	tfreq = param->m_freq;
	if (tfreq <= 4)
		tfreq = 4;/* minimal is 3.2, AP won't set this?! */
	else if (tfreq >= 105600)
		tfreq = 105600;

	rtk_hw_get_divider(tfreq, 255 , &M, &N);

	pwm_timing_ctrl_reg.regValue = rtd_inl(pp->TIMING_CTRL);
	pwm_timing_ctrl_reg.pwm0_m = M;
	pwm_timing_ctrl_reg.pwm0_n = N;
	rtd_outl(pp->TIMING_CTRL, pwm_timing_ctrl_reg.regValue);

	/*set duty */
	pwm_duty_set_reg.regValue = rtd_inl(pp->DUTY_SET);
	pwm_duty_set_reg.pwm0_dut = param->m_duty;
	rtd_outl(pp->DUTY_SET, pwm_duty_set_reg.regValue);


	/*set CTRL */
	pwm_ctrl_reg.regValue = rtd_inl(pp->CTRL);
	pwm_ctrl_reg.pwm0_en = param->m_run;
	pwm_ctrl_reg.pwm0_vs_rst_en = param->m_vsync;

	/*get v delay*/
	if(param->m_pos > 0) {
		phase_us=(1000000000/CONFIG_VSYNC_FREQ)*CONFIG_VSYNC_FREQ/param->m_freq;
		pos = param->m_pos*phase_us/255/CONFIG_CPU_XTAL_NS;
		if(pos >=0x1fffff) {   //[148139] the tpye of pos is unsigned int
			printf("[pwm] %s vsync overflow\n",__func__);
			printf("adapt_fre = %d, freq = %d, pos = %d, phase_us = %d\n",
				CONFIG_VSYNC_FREQ, tfreq,
				pos, phase_us);
			pos=0;
		}
	}else{
		pos=0;
	};


	/*vsync delay*/
	if(pos > 0 && param->m_vsync) {
		pwm_ctrl_reg.pwm0_vs_delay_thr = pos;
	}else{
		pwm_ctrl_reg.pwm0_vs_delay_thr = 0;
	}
	rtd_outl(pp->CTRL, pwm_ctrl_reg.regValue);

	/*set DB */
	dbReg.regValue = rtd_inl(MIS_PWM_DB_CTRL_VADDR);
	dbReg.pwm_w_db_mode = param->m_vsync;
	dbReg.pwm_w_db_en = param->m_vsync;
	rtd_outl(MIS_PWM_DB_CTRL_VADDR, dbReg.regValue);

	//Write DB (move from DB_CTRL to PWM_CTRL)
	if(param->m_vsync) {
		dbReg.pwm_w_db_wr = 1;		
		rtd_outl(pp->CTRL, dbReg.regValue);
	}

	pwmDebug("[pwm-%d] %s freq = %d, duty = %d , vsync = 0x%X , vs_dealy_en = 0x%X ,  reg_pos = 0x%X , M+=%d , codeOption_pos = 0x%X  success.\n",
			param->m_index,
			__func__,
			tfreq,
			pwm_duty_set_reg.pwm0_dut ,
			pwm_ctrl_reg.pwm0_vs_rst_en ,
			(pwm_ctrl_reg.pwm0_vs_delay_thr)?1:0,
			pwm_ctrl_reg.pwm0_vs_delay_thr ,
			pwm_ctrl_reg.pwm0_dim_duty_en ,
			pos);

	return 0;

};
#endif

