/*
 * (C) Copyright 2013
 * EJ Hsu <ejhsu@realtek.com>
 *
 * SPDX-License-Identifier:	GPL-2.0+
 */

#include "macro.h"

/*
 * Enter Exception.
 * This will save the processor state that is ELR/X0~X30
 * to the stack frame.
 */
.macro	exception_entry
	stp	x29, x30, [sp, #-16]!
	stp	x27, x28, [sp, #-16]!
	stp	x25, x26, [sp, #-16]!
	stp	x23, x24, [sp, #-16]!
	stp	x21, x22, [sp, #-16]!
	stp	x19, x20, [sp, #-16]!
	stp	x17, x18, [sp, #-16]!
	stp	x15, x16, [sp, #-16]!
	stp	x13, x14, [sp, #-16]!
	stp	x11, x12, [sp, #-16]!
	stp	x9, x10, [sp, #-16]!
	stp	x7, x8, [sp, #-16]!
	stp	x5, x6, [sp, #-16]!
	stp	x3, x4, [sp, #-16]!
	stp	x1, x2, [sp, #-16]!

	/* Could be running at EL3/EL2/EL1 */
	switch_el x11, 3f, 2f, 1f
3:	mrs	x1, esr_el3
	mrs	x2, elr_el3
	b	0f
2:	mrs	x1, esr_el2
	mrs	x2, elr_el2
	b	0f
1:	mrs	x1, esr_el1
	mrs	x2, elr_el1
0:
	stp	x2, x0, [sp, #-16]!
	mov	x0, sp
.endm

/*
 * Exception vectors.
 */
	.align	11
	.globl	vectors
vectors:
	.align	7
	b	_do_bad_entry		/* Current EL Synchronous Handler */

	.align	7
	b	_do_bad_entry		/* Current EL IRQ Handler */

	.align	7
	b	_do_bad_entry		/* Current EL FIQ Handler */

	.align	7
	b	_do_bad_entry		/* Current EL Error Handler */

	.align	7
#	b	_do_bad_entry		/* Current EL Synchronous Handler */
	b	do_bad_1

	.align	7
#	b	_do_bad_entry		/* Current EL IRQ Handler */
	b	do_bad_2

	.align	7
#	b	_do_bad_entry		/* Current EL FIQ Handler */
	b	do_bad_2

	.align	7
#	b	_do_bad_entry		/* Current EL Error Handler */
	b	do_bad_2

	.align	7
	b	_do_bad_entry		/* Lower EL Synchronous Handler using A64 */

	.align	7
	b	_do_bad_entry		/* Lower EL IRQ Handler using A64 */

	.align	7
	b	_do_bad_entry		/* Lower EL FIQ Handler using A64 */

	.align	7
	b	_do_bad_entry		/* Lower EL Error Handler using A64 */

	.align	7
	b	jump_to_el1_kernel	/* Lower EL Synchronous Handler using A32 */

	.align	7
	b	_do_bad_entry		/* Lower EL IRQ Handler using A32 */

	.align	7
	b	_do_bad_entry		/* Lower EL FIQ Handler using A32 */

	.align	7
	b	_do_bad_entry		/* Lower EL Error Handler using A32 */

_do_bad_entry:
#	exception_entry
1:
	b	1b

do_bad_1:
#	exception_entry
	mrs	x1, esr_el3
	mrs	x2, elr_el3
1:
	b	1b

do_bad_2:
#	exception_entry
	mrs	x1, esr_el3
	mrs	x2, elr_el3
1:
	b	1b
