#include <common.h>
#include <drivers/amp/rtk_amp_interface.h>
#include <drivers/amp/rtk_amp_device_id_table.h>
#include <platform_lib/board/pcb.h>


#if defined(CONFIG_REALTEK_AMP_ALC1312)
#include "ALC1312/alc1312.h"
#endif

struct AMP_IC
{
    char amp_name[64];
    int id ;
    unsigned short slave_addr;
    void (*ops)(int bus_id, unsigned short addr);
};

extern void alc1310_func(int bus_id, unsigned short addr);
extern void ad82010_func(int bus_id, unsigned short addr);
extern void tas5707_func(int bus_id, unsigned short addr);
extern void tas5711_func(int bus_id, unsigned short addr);
extern void tas5751_func(int bus_id, unsigned short addr);
extern void ad82088_func(int bus_id, unsigned short addr);
extern void alc1311_func(int bus_id, unsigned short addr);
extern void alc1312_func(int bus_id, unsigned short addr);
#if defined(CONFIG_AMP_RT9119)
extern void rt9119_func(int bus_id, unsigned short addr);
#endif

#if defined(Config_Board_Demo) || defined(Config_Board_Demo_MT) ||defined(Config_Board_Demo_19x19) || defined(Config_Board_TV036) || defined(Config_Board_TV050)
    // TV001
    const struct AMP_IC amp_list[] =
    {
        // amp_name          , id            , slave_addr                           , operation function 
        { "Clear Settting   ",  0            , AMP_BOOTCODE_USE_THIS_I2C_ADDRESS ,alc1310_func},
        { "ALC1310          ",  RTK_ALC1310  , AMP_BOOTCODE_USE_THIS_I2C_ADDRESS ,alc1310_func},
        { "AD82010          ",  ESMT_AD82010 , AMP_AD82010_ADDRESS               ,ad82010_func},
        { "TAS5751          ",  TI_TAS5751   , AMP_TAS5751_ADDRESS               ,tas5751_func},
        { "TAS5707          ",  TI_TAS5707   , AMP_TAS5707_ADDRESS               ,tas5707_func},
        { "TAS5711          ",  TI_TAS5711   , AMP_TAS5711_ADDRESS               ,tas5711_func},
        { "ALC1312          ",  RTK_ALC1312  , AMP_ALC1312_ADDRESS               ,alc1312_func},
        #if defined(CONFIG_AMP_RT9119)
        { "RT9119           ",  RICHTEK_9119 , AMP_RT9119_ADDRESS                ,rt9119_func },
        #endif
    };
#elif defined(Config_Board_Demo_pfg_ref)|| defined(Config_Board_TV051_Demo_pfg_ref)
    // For Odin
    const struct AMP_IC amp_list[] =
    {
        // amp_name          , id            , slave_addr                           , operation function 
        { "Clear Settting   ",  0            , AMP_TAS5707_ADDRESS               ,tas5707_func},
        { "ALC1310          ",  RTK_ALC1310  , AMP_BOOTCODE_USE_THIS_I2C_ADDRESS ,alc1310_func},
        { "AD82010          ",  ESMT_AD82010 , AMP_AD82010_ADDRESS               ,ad82010_func},
        { "TAS5751          ",  TI_TAS5751   , AMP_TAS5751_ADDRESS               ,tas5751_func},
        { "TAS5707          ",  TI_TAS5707   , AMP_TAS5707_ADDRESS               ,tas5707_func},
        { "TAS5711          ",  TI_TAS5711   , AMP_TAS5711_ADDRESS               ,tas5711_func},
        { "ALC1312          ",  RTK_ALC1312  , AMP_ALC1312_ADDRESS               ,alc1312_func},
    };
#elif defined(Config_Board_TV030)
    const struct AMP_IC amp_list[] =
    {
        // amp_name          , id           , slave_addr
        { "Clear Settting   ",  0           , AMP_BOOTCODE_USE_THIS_I2C_ADDRESS ,alc1310_func},
        { "ACL1310          ",  RTK_ALC1310 , AMP_BOOTCODE_USE_THIS_I2C_ADDRESS ,alc1310_func},
        { "AD82010          ",  ESMT_AD82010, AMP_AD82010_ADDRESS               ,ad82010_func},
        { "TAS5751          ",  TI_TAS5751  , AMP_TAS5751_ADDRESS               ,tas5751_func},
        { "AD82088          ",  ESMT_AD82088, AMP_BOOTCODE_USE_THIS_I2C_ADDRESS ,ad82088_func},
    };

#elif defined(Config_Board_TV035)
    const struct AMP_IC amp_list[] =
    {
        // amp_name          , id            , slave_addr
        { "Clear Settting   ",  0            , AMP_BOOTCODE_USE_THIS_I2C_ADDRESS ,alc1310_func},
        { "ALC1310          ",  RTK_ALC1310  , AMP_BOOTCODE_USE_THIS_I2C_ADDRESS ,alc1310_func},
        { "AD82010          ",  ESMT_AD82010 , AMP_AD82010_ADDRESS               ,ad82010_func},
        { "TAS5751          ",  TI_TAS5751   , AMP_TAS5751_ADDRESS               ,tas5751_func},
        { "ALC1311          ",  RTK_ALC1311  , AMP_BOOTCODE_USE_THIS_I2C_ADDRESS ,alc1311_func},
    };
#elif defined(Config_Board_TV002_V001)
    const struct AMP_IC amp_list[] =
    {
        // amp_name          , id            , slave_addr                           , operation function
        #if defined(CONFIG_AMP_RT9119)
        { "Clear Settting   ",  0            , AMP_BOOTCODE_USE_THIS_I2C_ADDRESS ,rt9119_func},
        { "RT9119           ",  RICHTEK_9119 , AMP_RT9119_ADDRESS                ,rt9119_func },
        #endif
    };
#else
    const struct AMP_IC amp_list[] =
    {
        // amp_name          , id            , slave_addr                           , operation function·
        { "Clear Settting   ",  0            , AMP_BOOTCODE_USE_THIS_I2C_ADDRESS ,alc1310_func},
        { "ALC1310          ",  RTK_ALC1310  , AMP_BOOTCODE_USE_THIS_I2C_ADDRESS ,alc1310_func},
    };
#endif


const int AMP_LIST_SIZE  = (sizeof(amp_list) /sizeof(amp_list[0]));


/**used in cmd_rtk_amp.c**/
int getAmpListLength( void )
{
    return AMP_LIST_SIZE;
}


const char *getAmpName(unsigned char index )
{
    if( index > getAmpListLength() )
    {
        printf("%s index = %d > AMP_LIST_SIZE = %d \n" , __func__  , index , getAmpListLength() );
        return NULL;
    }

    return amp_list[index].amp_name ;
}

int getAmpI2cAddress(unsigned char index )
{
    if( index > getAmpListLength() )
    {
        printf("%s index = %d > AMP_LIST_SIZE = %d \n" , __func__  , index , getAmpListLength() );
        return -1;
    }

    return amp_list[index].slave_addr;
}

int getAmpDeviceId(unsigned char index )
{
    if( index > getAmpListLength() )
    {
        printf("%s index = %d > AMP_LIST_SIZE = %d \n" , __func__  , index , getAmpListLength() );
        return -1;
    }

    return amp_list[index].id;
}
/**used in cmd_rtk_amp.c  end **/


/**used in rtk_amp_interface.c **/
int exeAmpInitFunc(unsigned int deviceId ,unsigned int bus_id, unsigned short addr)
{
    int i = 0 ;
    int index = 0 ;

    for( i = 0 ; i < AMP_LIST_SIZE ; ++i )
    {
        if(amp_list[i].id == deviceId )
        {
            index = i ;
            break;
        }
    }

    if( i >= AMP_LIST_SIZE )
    {
        printf("%s deviceId = %d is not existed in the AMP device table , run default AMP init() \n" , __func__  , deviceId );
        amp_list[0].ops(amp_list[0].id,amp_list[0].slave_addr);
        return 0;
    }

    amp_list[index].ops(bus_id,addr);

    return 0;
}


int	setAmpInitFunc( unsigned int deviceId , struct amp_controller *amp  )
{
    int i = 0 ;
    int index = 0 ;

    for( i = 0 ; i < AMP_LIST_SIZE ; ++i )
    {
        if(amp_list[i].id == deviceId )
        {
            index = i ;
            break;
        }
    }

    if( i >= AMP_LIST_SIZE )
    {
        printf("%s deviceId = %d is not existed in the AMP device table , set to default AMP init() \n" , __func__  , deviceId );
        amp->ops = amp_list[0].ops;
        return -1;
    }

    amp->ops = amp_list[index].ops;

    return 0;
}




