#include <common.h>
#include <rbus/stb_reg.h>
#include <drivers/lsadc/lsadc_reg.h>
#include <drivers/lsadc/lsadc.h>
#include <asm/arch/io.h>

#define LSADC_PIN_NUM 12

#define BIT(x)                                          (1 << (x))
#define printf(fmt,args...)

/*========================Functions===============================*/

static int g_lsadc_init = 0;

extern void rtd_part_outl(unsigned int addr, unsigned int B, unsigned int C, unsigned int D);

static void lsadc_clock_enable(void)
{
    if((rtd_inl(STB_ST_CLKEN1_reg) & BIT(STB_ST_CLKEN1_clken_lsadc_shift)) == 0) {
        rtd_outl(STB_ST_SRST1_reg, BIT(STB_ST_SRST1_rstn_lsadc_shift) |
                                                    BIT(STB_ST_SRST1_write_data_shift));
        udelay(10);
        rtd_outl(STB_ST_CLKEN1_reg, BIT(STB_ST_CLKEN1_clken_lsadc_shift) |
                                                    BIT(STB_ST_CLKEN1_write_data_shift));
        udelay(10);
    }
}

int lsadc_init(int index, int voltage_mode)
{
    st_pad0_RBUS st_pad0_reg;
    if (index >= LSADC_PIN_NUM)
            return -1;
            
    if(voltage_mode == 0)
            return -1;

    printf("%s idx:%d\n", __FUNCTION__, index);

    if (g_lsadc_init == 0) {
        /*enable lsadc clock*/
        lsadc_clock_enable();
        rtd_outl(ST_LSADC_ANALOG_CTRL_reg, 0x00010101); //Analog Enable
        rtd_outl(ST_LSADC_CTRL_reg, 0x16100000); //Digital setting
        rtd_outl(ST_LSADC_CTRL_reg, 0x16100001); //Digital Enable
        rtd_outl(ST_LSADC_PERI_TOP_DEBUG_reg, 0x00000000);

        g_lsadc_init = 1;
    }

    st_pad0_reg.regValue = 0;
    st_pad0_reg.pad0_active = 1;
    st_pad0_reg.pad0_sw = index;

    if (voltage_mode) {     // if voltage mode
            st_pad0_reg.pad0_ctrl = 0;
    } else {
            st_pad0_reg.pad0_ctrl = 1;
    }
    if(index < 8){
        rtd_outl(ST_PAD0_reg + index * 4, st_pad0_reg.regValue);
    }else{
        index = index -8;
        rtd_outl(ST_PAD8_reg + index * 4, st_pad0_reg.regValue);
    }
    return 0;
}


int lsadc_get(int index)
{
    st_lsadc_ctrl_RBUS st_lsadc_ctrl_reg;
    st_pad0_RBUS st_pad0_reg;

    if (index >= LSADC_PIN_NUM) {
            return -1;
    }

    st_lsadc_ctrl_reg.regValue = rtd_inl(ST_LSADC_CTRL_reg);
    if (st_lsadc_ctrl_reg.enable == 0) {
            st_lsadc_ctrl_reg.enable = 1;
            rtd_outl(ST_LSADC_CTRL_reg, st_lsadc_ctrl_reg.regValue);
    }

    udelay(1000);
    if(index < 8){
        st_pad0_reg.regValue = rtd_inl(ST_PAD0_reg + index * 4);
    }else{
        index = index -8;
        st_pad0_reg.regValue = rtd_inl(ST_PAD8_reg + index * 4);
    }
    printf("%s get value = %d\n", __func__, st_pad0_reg.regValue);

    return st_pad0_reg.adc_val0;
}

