/******************************************************************************
*
*   Copyright(c) 2016 Realtek Semiconductor Corp. All rights reserved.
*
*   @author realtek.com
*
******************************************************************************/
#include <rbus_types.h>
#include <sys_reg_reg.h>
#include <io.h>
#include <common.h>
#include "malloc.h"
#include <drivers/md/rtk_md_reg.h>
#include <drivers/md/rtk_md_hw.h>


int g_md_init_ok = 0;
static spin_lock_t rtk_md_lock = INIT_SPIN_LOCK;


#ifdef RTK_MD_SELF_CHECKING
static void rtk_md_self_checking(void);
#endif


static void rtk_md_reset(void)
{
    spin_lock(&rtk_md_lock);
    rtk_md_dump_reg();
    rtk_md_hw_init();
    spin_unlock(&rtk_md_lock);
}

void rtk_md_init(void)
{
    
    spin_lock(&rtk_md_lock);
    rtk_md_hw_init();
    g_md_init_ok = 1;
    spin_unlock(&rtk_md_lock);
    
#ifdef RTK_MD_SELF_CHECKING
    rtk_md_self_checking();
#endif
    printf("[MD]%s success\n", __func__);
}


int32_t rtk_md_memcpy_seq(uint32_t addrdst, uint32_t addrsrc, 
                                                    int32_t len, int32_t op, int32_t movebyte,
                                                    int32_t datamode, int32_t dir, uint32_t is_wait)
{
    uint32_t len_tmp = MD_SEQ_LEN_LIMIT;
    uint32_t dwcmdword[4] = {0};
    int32_t wret = 0;
    spin_lock(&rtk_md_lock);
    dwcmdword[0] = (datamode | movebyte | (op ? MD_CONST : 0) | (dir ? BIT(7) : 0));
    do {
        len_tmp = (len > MD_SEQ_LEN_LIMIT) ? MD_SEQ_LEN_LIMIT : len;

        dwcmdword[1] = addrdst;
        dwcmdword[2] = addrsrc;
        dwcmdword[3] = (len_tmp & MD_SEQ_LEN_LIMIT_MASK);

        wret = rtk_md_write_cmd_wrapper((uint8_t *)dwcmdword, sizeof(dwcmdword));
        if(wret != MD_SUCCESS) {
            printf("[MD]%s fail(%d).\n", __func__, wret);
            break;
        }
        len -= len_tmp;
        addrdst += len_tmp;
        addrsrc += len_tmp;
    } while(len);

    if(wret == MD_SUCCESS && is_wait)
        rtk_md_wait_hw_stopped();
    spin_unlock(&rtk_md_lock);     
    return wret;
}


int32_t rtk_md_memcpy_pitch(uint32_t addrdst, uint32_t dstpitch, uint32_t addrsrc,
                                                    uint32_t srcpitch, uint32_t hlen, uint32_t wlen, 
                                                    int32_t movebyte, uint32_t is_wait)
{
    uint32_t dwcmdword[4] = {0};
    int32_t wret = 0;
    dwcmdword[0] = MD_SS_PITCH | movebyte | MD_DST_PITCH(dstpitch) | MD_SRC_PITCH(srcpitch);

    dwcmdword[1] = addrdst;
    dwcmdword[2] = addrsrc;
    dwcmdword[3] = MD_HEIGHT_BLK(hlen) | MD_WIDTH_BLK(wlen);

    spin_lock(&rtk_md_lock);
    wret = rtk_md_write_cmd_wrapper((uint8_t *)dwcmdword, sizeof(dwcmdword));
    if(wret != MD_SUCCESS) {
        printf("[MD]%s fail(%d).\n", __func__, wret);
    }

    if(wret == MD_SUCCESS && is_wait)
        rtk_md_wait_hw_stopped();
    spin_unlock(&rtk_md_lock);
    return wret;
}


uint32_t rtk_md_get_r_crc(uint32_t addrsrc, uint32_t len)
{
    uint32_t ret = 0;  
    spin_lock(&rtk_md_lock);
    ret = rtk_md_do_hw_r_crc(addrsrc, len);
    spin_unlock(&rtk_md_lock);
    return ret;
}


uint16_t smd_memfill(uint32_t addrDst, int32_t len, uint32_t value)
{
    rtk_md_memcpy_seq(addrDst, value & 0xff, len, MD_CONST, MD_1B, MD_SS, 0, 1);
    return rtk_md_get_issued_inst_count();
}



uint16_t smd_memcpy(uint32_t addrDst, uint32_t addrSrc, int32_t len)
{
    rtk_md_memcpy_seq(addrDst, addrSrc, len, 0, MD_1B, MD_SS, 0, 1);
    return rtk_md_get_issued_inst_count();
}




bool smd_checkFinish(uint16_t sw_cnt)
{
    bool ret = false;
    spin_lock(&rtk_md_lock);
    ret = rtk_md_check_finish(sw_cnt);
    spin_unlock(&rtk_md_lock);
    return ret;
}

bool smd_checkComplete(uint16_t sw_cnt)
{
    bool ret = false;
    uint32_t u32_err_cnt = 0;
    while(1) {
        if(smd_checkFinish(sw_cnt)) {
            ret = true;
            break;
        }
        udelay(200);
        u32_err_cnt++;
        /*add 200ms timeout*/
        if(u32_err_cnt > 1000) {
            printf("[MD] wait timeout!!\n");    
            rtk_md_reset();
            break;
        }
    }
    return ret;
}


#ifdef RTK_MD_SELF_CHECKING
static void rtk_md_self_checking(void)
{
    int retry_cnt = 0;
    int len = 0x40;
    unsigned char *src = NULL;
    unsigned char *dst = NULL;
    uint16_t inst_count = 0;
    src = malloc(len * 2);
    if(!src) {
        printf("[MD] %s malloc failed\n", __FUNCTION__);
        return;
    }
    dst = src + len;
    
    memset(src, 0x22, len);
    memset(dst, 0x11, len);
    
    flush_dcache_all();
    for(retry_cnt = 0; retry_cnt <= 2; retry_cnt++) {
        inst_count = smd_memcpy((unsigned long)dst, (unsigned long)src, len);
        if(smd_checkComplete(inst_count)) {
            flush_dcache_all();
           if(memcmp(src, dst, len) == 0) {
                printf("[MD]%s success\n", __func__);
                break;
            }        
        }
    }
    if(src)
        free(src);
}
#endif



