/******************************************************************************
*
*   Copyright(c) 2016 Realtek Semiconductor Corp. All rights reserved.
*
*   @author realtek.com
*
******************************************************************************/
#include <rbus_types.h>
#include <sys_reg_reg.h>
#include <io.h>
#include <common.h>
#include "malloc.h"
#include <drivers/md/rtk_md_reg.h>
#include <drivers/md/rtk_md_hw.h>


static uint16_t g_u16IssuedInstCnt = 0;
static uint32_t g_CmdBase = 0;
static uint32_t g_CmdLimit = 0;
extern int g_md_init_ok;


void rtk_md_dump_reg(void)
{
    uint32_t* rp = NULL;
    printf(" ======================================== \n");
    printf("MdCtrl         = %08x\n", (unsigned int)GET_MD_SMQ_CNTL());
    printf("MdInts         = %08x\n", (unsigned int)GET_MD_SMQ_INT_STATUS());
    printf("MdInte         = %08x\n", (unsigned int)GET_MD_SMQ_INT_ENABLE() );
    printf("MdCmdBase      = %08x\n", (unsigned int)GET_MD_SMQBASE());
    printf("MdCmdLimit     = %08x\n", (unsigned int)GET_MD_SMQLIMIT());
    printf("MdCmdReadPtr   = %08x\n", (unsigned int)GET_MD_SMQRDPTR());
    printf("MdCmdWritePtr  = %08x\n", (unsigned int)GET_MD_SMQWRPTR());
    printf("MdInsRemainder = %08x\n", (unsigned int)GET_MD_SMQFIFOSTATE());
    printf("MdInstCnt      = %08x\n", (unsigned int)GET_MD_SMQ_INSTCNT());
    if(GET_MD_SMQRDPTR()) {
        /*This is a case for GDMA resume flow.K2L/p only. each cmd from GDMA is 4 words*/
        rp =  (uint32_t*)(GET_MD_SMQRDPTR());//|0xC0000000);
        printf("CMD            = (%08x, %08x, %08x, %08x)\n", *(rp - 8), *(rp - 7), *(rp - 6), *(rp - 5));
        printf("CMD            = (%08x, %08x, %08x, %08x)\n", *(rp - 4), *(rp - 3), *(rp - 2), *(rp - 1));
        printf("CMD            = (%08x, %08x, %08x, %08x)\n", *(rp), *(rp + 1), *(rp + 2), *(rp + 3));
    }
    printf(" ======================================== \n");
}


void rtk_md_clk_on_off(int on_off)
{
    if(on_off == 1) {
        rtd_outl(SYS_REG_SYS_CLKEN1_reg, BIT(SYS_REG_SYS_CLKEN1_clken_md_shift));
        rtd_outl(SYS_REG_SYS_SRST1_reg, BIT(SYS_REG_SYS_SRST1_rstn_md_shift));
        rtd_outl(SYS_REG_SYS_CLKEN1_reg, BIT(SYS_REG_SYS_CLKEN1_clken_md_shift) | SYS_REG_SYS_CLKEN1_write_data_mask);
        rtd_outl(SYS_REG_SYS_CLKEN1_reg, BIT(SYS_REG_SYS_CLKEN1_clken_md_shift));
        rtd_outl(SYS_REG_SYS_SRST1_reg, BIT(SYS_REG_SYS_SRST1_rstn_md_shift) | SYS_REG_SYS_SRST1_write_data_mask);
        rtd_outl(SYS_REG_SYS_CLKEN1_reg, BIT(SYS_REG_SYS_CLKEN1_clken_md_shift) | SYS_REG_SYS_CLKEN1_write_data_mask);
    } else {
        rtd_outl(SYS_REG_SYS_CLKEN1_reg, BIT(SYS_REG_SYS_CLKEN1_clken_md_shift));
    }
}

void rtk_md_hw_init(void)
{

    if(g_CmdBase == 0) {
        g_CmdBase = (unsigned int)malloc(MD_CMD_BUF);
        g_CmdLimit = g_CmdBase + MD_CMD_BUF;
    }

    /* Clock & reset */
    rtk_md_clk_on_off(1);
    SET_MD_SMQBASE((uint32_t)g_CmdBase);
    SET_MD_SMQLIMIT((uint32_t)g_CmdLimit);
    SET_MD_SMQRDPTR((uint32_t)g_CmdBase);
    SET_MD_SMQWRPTR((uint32_t)g_CmdBase);
    SET_MD_SMQ_INSTCNT(0);
    SET_MD_SMQ_CNTL(MdEndianSwap | MdWriteData);
    SET_MD_DUMMY1(0x1);
    SET_MD_MULTI_REQ(MD_MULTI_REQ_EN(0));
    g_u16IssuedInstCnt = 0;
    
}


static bool rtk_md_check_hw_stopped(void)
{
    if((GET_MD_SMQRDPTR() == GET_MD_SMQWRPTR())
        && GET_MD_SMQ_IDLE()) 
        return true;
    return false;
}


static int32_t rtk_md_write_cmd(uint8_t *pCmdBuf, uint32_t lCmdLen)
{
    uint32_t dcnt = 0;
    uint32_t WPointer = GET_MD_SMQWRPTR();
    uint32_t u32_err_cnt = 0;

    while(1) {
        uint32_t RPointer = GET_MD_SMQRDPTR();
        if(RPointer <= WPointer)
            RPointer += MD_CMD_BUF;
        if((WPointer + lCmdLen) < RPointer)
            break;
        udelay(1);
        /*add 200ms timeout*/
        if(u32_err_cnt > 200000)
            goto WRITEFAIL;
    }
    
    for(dcnt = 0; dcnt < lCmdLen; dcnt += sizeof(uint32_t)) {
        *((uint32_t *)(WPointer)) = *(uint32_t *)(pCmdBuf + dcnt);
        WPointer += sizeof(uint32_t);
        if(WPointer >= (uint32_t)g_CmdLimit)
            WPointer = (uint32_t)g_CmdBase;
    }
    
    flush_dcache_all();
    
    SET_MD_SMQWRPTR(WPointer);
    SET_MD_SMQ_CNTL(MdGo | MdWriteData);
    g_u16IssuedInstCnt += lCmdLen / MD_ONE_CMD_LEN;
    
    return MD_SUCCESS;
    
WRITEFAIL:
    rtk_md_dump_reg();
    return MD_ERR_WRITE_FAIL;
}


int32_t rtk_md_write_cmd_wrapper(uint8_t *pbyCommandBuffer, uint32_t lCommandLength)
{
    int32_t wret = -1;
    int32_t timeoutcnt = 0;
    int32_t errcnt = 3;
    int32_t icnt = 3;
    uint32_t ori_rptr = 0;

    if(g_md_init_ok == 0) {
        rtk_md_hw_init();
        g_md_init_ok = 1;
    }  
    do {
        ori_rptr = GET_MD_SMQRDPTR();
        /*must be enable go bit!*/
        wret = rtk_md_write_cmd(pbyCommandBuffer, lCommandLength);

        if(wret == MD_ERR_WRITE_FAIL) {
            udelay(1000);
            timeoutcnt += 3;
        } else {
            /*if md is busy, should we need more delay?*/
            for(icnt = 0; icnt < errcnt; icnt++) {
                udelay(50);
                if(ori_rptr == GET_MD_SMQRDPTR()) {
                    timeoutcnt += 10;
                } else {
                    goto WSUCCESS;
                }
            }
        }
        if(timeoutcnt >= 10) {
            errcnt--;
            timeoutcnt = 0;
        }
    } while(wret != 0 || errcnt < 0);
WSUCCESS:
    return wret;
}


bool rtk_md_wait_hw_stopped(void)
{
    bool ret = false;
    uint32_t u32_cnt = 0;
    while(1) {
        if(rtk_md_check_hw_stopped()) {
            ret = true;
            break;
        }
        udelay(200);
        u32_cnt++;
        /*add 200ms timeout*/
        if(u32_cnt > 1000) {
            break;
        }
    }
    if(ret == false) {
        printf("[%s]wait hw stopped failed\n", __func__);
    }
    return ret;
}


bool rtk_md_check_finish(uint16_t sw_cnt)
{
    bool ret = false;
    uint16_t hw_cnt = GET_MD_SMQ_INST_CNT();
    if(hw_cnt >= sw_cnt)
        ret = true;
    else if(rtk_md_check_hw_stopped() == true)
        ret = true;

    return ret;
}




/*CRC */
uint32_t rtk_md_do_hw_r_crc(uint32_t addrsrc, uint32_t len)
{
    uint32_t ret = 0;
    int32_t timeout = 0;

    if(len == 0x40000000)
        len = 0;
    else if (len > 0x40000000)
        return ret;

    SET_MD_REG_CMD1(MD_1B); //seq2seq
    SET_MD_REG_CMD2(addrsrc);
    SET_MD_REG_CMD3(addrsrc);
    SET_MD_REG_CMD4(len);
    SET_MD_R_CRC_CTRL(MD_R_CRC_START(1));

    SET_MD_REG_CTRL(MD_REG_POLL(1));
    SET_MD_REG_MODE(MD_REG_MODE(1));
    while(GET_MD_REG_MODE_MODE()) {
        udelay(1000);
        timeout++;
        if(timeout > 100) {
            printf("[%s]timeout,size=%d", __func__, timeout);
            SET_MD_REG_MODE(MD_REG_MODE(0));
            ret =  0;
            goto err;
        }
    }
    ret = GET_MD_R_CRC_RESULT();
err:
    return ret;
}

uint16_t rtk_md_get_issued_inst_count(void)
{
    return g_u16IssuedInstCnt;
}



