/******************************************************************************
*
*   Copyright(c) 2016 Realtek Semiconductor Corp. All rights reserved.
*
*   @author realtek.com
*
******************************************************************************/
#include <rbus_types.h>
#include <sys_reg_reg.h>
#include <io.h>
#include <common.h>
#include "malloc.h"
#include <drivers/md/rtk_md.h>
#include <drivers/md/rtk_md_reg.h>

//======================================
//for public test
#define MD_TEST_SRC_ADDR         (0x01500000)
#define MD_TEST_DST_ADDR         (0x01500000+0x100000)

#define TIMER_REG_VAL_ADDR  0xb801b6b8
#define TIMER_REG_CTRL_ADDR  0xb801b6b0


static int rtk_md_check(
    uint32_t addr,
    uint32_t len,
    uint32_t val,
    uint32_t bytemode)
{
    uint32_t size = len;
    uint32_t cnt = 0;

    if(bytemode == MD_2B) {
        size /= 2;

    } else if(bytemode == MD_4B) {
        size /= 4;
    }

    flush_dcache_all();

    for(cnt = 0; cnt < size; cnt++) {
        if(bytemode == MD_1B) {
            if(*((uint8_t*)addr + cnt) != (uint8_t)(val & 0xff)) {
                printf("[MD]%s error. address = %08x, shift = %d, addr value = 0x%2x, it should be 0x%2x!\n", __func__, addr, cnt, *((uint8_t*)addr + cnt), (uint8_t)(val & 0xff));
                return -1;
            }
        } else if(bytemode == MD_2B) {
            if(*((uint16_t*)addr + cnt) != (uint16_t)(val & 0xffff)) {
                printf("[MD]%s error. address = %08x, shift = %d, addr value = 0x%4x, it should be 0x%4x!\n", __func__, addr, cnt, *((uint16_t*)addr + cnt), (uint16_t)(val & 0xffff));
                return -1;
            }
        } else if(bytemode == MD_4B) {
            if(*((uint32_t*)addr + cnt) != (uint32_t)(val & 0xffffffff)) {
                printf("[MD]%s error. address = %08x, shift = %d, addr value = 0x%8x, it should be 0x%8x!\n", __func__, addr, cnt, *((uint32_t*)addr + cnt), (uint32_t)(val & 0xffffffff));
                return -1;
            }
        } else {
            printf("[MD]%s parameter error!\n", __func__);
            return -1;
        }
    }
    return 0;
}
#if !CONFIG_FT
#if 0
static uint32_t rtk_md_compare(
    uint32_t addr1,
    uint32_t addr2,
    uint32_t len,
    uint32_t bytemode)
{
    uint32_t size = len;
    uint32_t cnt = 0;

    if(bytemode == MD_2B) {
        size /= 2;

    } else if(bytemode == MD_4B) {
        size /= 4;
    }

    for(cnt = 0; cnt < size; cnt++) {
        if(bytemode == MD_1B) {
            if(*((uint8_t*)addr1 + cnt) != *((uint8_t*)addr2 + cnt)) {
                printf("[MD]%s error. shift = %d, addr1(%08x)=0x%2x, addr2(%08x)=0x%2x\n",
                       __func__, cnt, addr1, *((uint8_t*)addr1 + cnt), addr2, *((uint8_t*)addr2 + cnt));
                return -1;
            }
        } else if(bytemode == MD_2B) {
            if(*((uint16_t*)addr1 + cnt) != *((uint16_t*)addr2 + cnt)) {
                printf("[MD]%s error. shift = %d, addr1(%08x)=0x%4x, addr2(%08x)=0x%4x\n",
                       __func__, cnt, addr1, *((uint16_t*)addr1 + cnt), addr2, *((uint16_t*)addr1 + cnt));
                return -1;
            }
        } else if(bytemode == MD_4B) {
            if(*((uint32_t*)addr1 + cnt) != *((uint32_t*)addr2 + cnt)) {
                printf("[MD]%s error. shift = %d, addr1(%08x)=0x%8x, addr2(%08x)=0x%8x\n",
                       __func__, cnt, addr1, *((uint32_t*)addr1 + cnt), addr2, *((uint32_t*)addr1 + cnt));
                return -1;
            }
        } else {
            printf("[MD]%s parameter error!\n", __func__);
            return -1;
        }
    }
    return 0;
}
#endif
#endif
static void rtk_md_gen_data(
    uint32_t addr,
    uint32_t len,
    uint32_t val,
    uint32_t bytemode)
{

    uint32_t size = len;
    uint32_t cnt = 0;
    if(bytemode == MD_2B) {
        size /= 2;
    } else if(bytemode == MD_4B) {
        size /= 4;
    }

    for(cnt = 0; cnt < len; cnt++) {
        if(bytemode == MD_1B) {
            *((uint8_t*)addr + cnt) = (uint8_t)(val & 0xff);
        } else if(bytemode == MD_2B) {
            *((uint16_t*)addr + cnt) = (uint16_t)(val & 0xffff);
        } else if(bytemode == MD_4B) {
            *((uint32_t*)addr + cnt) = (uint32_t)(val & 0xffffffff);
        } else {
            printf("[MD]%s parameter error!\n", __func__);
        }
    }
    flush_dcache_all();
    rtk_md_check(addr, len, val, bytemode);
}


static int rtk_md_testcases_seq2seq(
    uint32_t addrSrc,
    uint32_t addrDst,
    uint32_t len)
{
    uint32_t pts_s = 0;
    uint32_t pts_e = 0;
    int32_t result = 0;


    /*CASE - FILL 1B 0x1c*/
    pts_s = rtd_inl(TIMER_REG_VAL_ADDR);
    rtk_md_memcpy_seq(addrDst, 0x83, len, MD_CONST, MD_1B, MD_SS, 0, 1);
    pts_e = rtd_inl(TIMER_REG_VAL_ADDR);
    result = rtk_md_check((uint32_t)addrDst, len, 0x83, MD_1B);

    if(result != 0) {
        printf("[MD] fill %d bytes with 1B mode fail!\n", len);
        return result;
    } else {
        printf("[MD] fill %d bytes with 1B mode success. elapsed %d us\n", len, (pts_e - pts_s) * 1000000 / 90090);
    }


    /*CASE - COPY 1B*/
    rtk_md_memcpy_seq(addrSrc, 0x00, len, MD_CONST, MD_1B, MD_SS, 0, 1);
    rtk_md_memcpy_seq(addrDst, 0x00, len, MD_CONST, MD_1B, MD_SS, 0, 1);
    rtk_md_gen_data((uint32_t)addrSrc, len, 0x45, MD_1B);

    pts_s = rtd_inl(TIMER_REG_VAL_ADDR);
    rtk_md_memcpy_seq(addrDst, addrSrc, len, MD_VECTOR, MD_1B, MD_SS, 0, 1);
    pts_e = rtd_inl(TIMER_REG_VAL_ADDR);
    result = rtk_md_check((uint32_t)addrDst, len, 0x45, MD_1B);

    if(result != 0) {
        printf("[MD] COPY %d bytes with 1B mode fail!\n", len);
        return result;
    } else {
        printf("[MD] COPY %d bytes with 1B mode success. elapsed %d us\n", len, (pts_e - pts_s) * 1000000 / 90090);
    }


    /*CASE - COPY 2B*/
    rtk_md_memcpy_seq(addrSrc, 0x00, len, MD_CONST, MD_1B, MD_SS, 0, 1);
    rtk_md_memcpy_seq(addrDst, 0x00, len, MD_CONST, MD_1B, MD_SS, 0, 1);
    rtk_md_gen_data((uint32_t)addrSrc, len, 0xf123, MD_2B);

    pts_s = rtd_inl(TIMER_REG_VAL_ADDR);
    rtk_md_memcpy_seq(addrDst, addrSrc, len, MD_VECTOR, MD_2B, MD_SS, 0, 1);
    pts_e = rtd_inl(TIMER_REG_VAL_ADDR);
    result = rtk_md_check((uint32_t)addrDst, len, 0x23f1, MD_2B);

    if(result != 0) {
        printf("[MD] COPY %d bytes with 2B mode fail!\n", len);
        return result;
    } else {
        printf("[MD] COPY %d bytes with 2B mode success. elapsed %d us\n", len, (pts_e - pts_s) * 1000000 / 90090);
    }


    /*CASE - COPY 4B*/
    rtk_md_memcpy_seq(addrSrc, 0x00, len, MD_CONST, MD_1B, MD_SS, 0, 1);
    rtk_md_memcpy_seq(addrDst, 0x00, len, MD_CONST, MD_1B, MD_SS, 0, 1);
    rtk_md_gen_data((uint32_t)addrSrc, len, 0xff9abcde, MD_4B);

    pts_s = rtd_inl(TIMER_REG_VAL_ADDR);
    rtk_md_memcpy_seq(addrDst, addrSrc, len, MD_VECTOR, MD_4B, MD_SS, 0, 1);
    pts_e = rtd_inl(TIMER_REG_VAL_ADDR);
    result = rtk_md_check((uint32_t)addrDst, len, 0xdebc9aff, MD_4B);

    if(result != 0) {
        printf("[MD] COPY %d bytes with 4B mode fail!\n", len);
        return result;
    } else {
        printf("[MD] COPY %d bytes with 4B mode success. elapsed %d us\n", len, (pts_e - pts_s) * 1000000 / 90090);
    }
#if 1
    /*CASE - COPY 1B forward*/
    rtk_md_memcpy_seq(addrSrc, 0x00, len, MD_CONST, MD_1B, MD_SS, 0, 1);
    rtk_md_memcpy_seq(addrDst, 0x00, len, MD_CONST, MD_1B, MD_SS, 0, 1);
    rtk_md_gen_data((uint32_t)addrSrc, 1024, 0x77, MD_1B);
    rtk_md_gen_data((uint32_t)addrSrc + 1024, 1024, 0x11, MD_1B);

    pts_s = rtd_inl(TIMER_REG_VAL_ADDR);
    rtk_md_memcpy_seq(addrSrc, addrSrc + 512, 1024, MD_VECTOR, MD_1B, MD_SS, 0, 1);
    pts_e = rtd_inl(TIMER_REG_VAL_ADDR);
    result = rtk_md_check((uint32_t)addrSrc, 512, 0x77, MD_1B);
    result += rtk_md_check((uint32_t)addrSrc + 512, 512, 0x11, MD_1B);

    if(result != 0) {
        printf("[MD] COPY %d bytes with 1B forward mode fail!\n", len);
        return result;
    } else {
        printf("[MD] COPY %d bytes with 1B forward mode success. elapsed %d us\n", len, (pts_e - pts_s) * 1000000 / 90090);
    }
#endif

    /*CASE - COPY 1B backward*/
    rtk_md_memcpy_seq(addrSrc, 0x00, len, MD_CONST, MD_1B, MD_SS, 0, 1);
    rtk_md_memcpy_seq(addrDst, 0x00, len, MD_CONST, MD_1B, MD_SS, 0, 1);
    rtk_md_gen_data((uint32_t)addrSrc, 1024, 0x33, MD_1B);
    rtk_md_gen_data((uint32_t)addrSrc + 1024, 1024, 0x23, MD_1B);

    pts_s = rtd_inl(TIMER_REG_VAL_ADDR);
    rtk_md_memcpy_seq(addrSrc, addrSrc + 512, 1024, MD_VECTOR, MD_1B, MD_SS, MOVE_BACKWARD, 1);
    pts_e = rtd_inl(TIMER_REG_VAL_ADDR);
    result = rtk_md_check((uint32_t)addrSrc, 1024, 0x23, MD_1B);

    if(result != 0) {
        printf("[MD] COPY %d bytes with 1B backward mode fail!\n", len);
        return result;
    } else {
        printf("[MD] COPY %d bytes with 1B backward mode success. elapsed %d us\n", len, (pts_e - pts_s) * 1000000 / 90090);
    }

    /*CASE - COPY 2B*/
    rtk_md_memcpy_seq(addrSrc, 0x00, len, MD_CONST, MD_1B, MD_SS, 0, 1);
    rtk_md_memcpy_seq(addrDst, 0x00, len, MD_CONST, MD_1B, MD_SS, 0, 1);
    rtk_md_gen_data((uint32_t)addrSrc, 1024, 0x2211, MD_2B);
    rtk_md_gen_data((uint32_t)addrSrc + 1024, 1024, 0xf123, MD_2B);

    pts_s = rtd_inl(TIMER_REG_VAL_ADDR);
    rtk_md_memcpy_seq(addrSrc, addrSrc + 512, 1024, MD_VECTOR, MD_2B, MD_SS, MOVE_BACKWARD, 1);
    pts_e = rtd_inl(TIMER_REG_VAL_ADDR);
    result = rtk_md_check((uint32_t)addrSrc, 512, 0xf123, MD_2B);
    result += rtk_md_check((uint32_t)addrSrc + 512, 512, 0x23f1, MD_2B);

    if(result != 0) {
        printf("[MD] COPY %d bytes with 2B backward mode fail!\n", len);
        return result;
    } else {
        printf("[MD] COPY %d bytes with 2B backward mode success. elapsed %d us\n", len, (pts_e - pts_s) * 1000000 / 90090);
    }


    /*CASE - COPY 4B*/
    rtk_md_memcpy_seq(addrSrc, 0x00, len, MD_CONST, MD_1B, MD_SS, 0, 1);
    rtk_md_memcpy_seq(addrDst, 0x00, len, MD_CONST, MD_1B, MD_SS, 0, 1);
    rtk_md_gen_data((uint32_t)addrSrc, len, 0xff9abcde, MD_4B);
    rtk_md_gen_data((uint32_t)addrSrc + 1024, 1024, 0xff9abcde, MD_4B);

    pts_s = rtd_inl(TIMER_REG_VAL_ADDR);
    rtk_md_memcpy_seq(addrSrc, addrSrc + 512, 1024, MD_VECTOR, MD_4B, MD_SS, MOVE_BACKWARD, 1);
    pts_e = rtd_inl(TIMER_REG_VAL_ADDR);
    result = rtk_md_check((uint32_t)addrSrc, 512, 0xff9abcde, MD_4B);
    result = rtk_md_check((uint32_t)addrSrc + 512, 512, 0xdebc9aff, MD_4B);

    if(result != 0) {
        printf("[MD] COPY %d bytes with 4B  backward mode fail!\n", len);
        return result;
    } else {
        printf("[MD] COPY %d bytes with 4B backward mode success. elapsed %d us\n", len, (pts_e - pts_s) * 1000000 / 90090);
    }

    return result;
}


#if !CONFIG_FT
#if 0
static void rtk_md_testcases_pitch(
    uint32_t addrSrc,
    uint32_t addrDst,
    uint32_t len)
{
    uint32_t pts_s = 0;
    uint32_t pts_e = 0;
    int32_t result = 0;


    /*CASE - pitch 1B */
    rtk_md_memcpy_seq(addrSrc, 0x00, len, MD_CONST, MD_1B, MD_SS, 0, 1);
    rtk_md_memcpy_seq(addrDst, 0x00, len, MD_CONST, MD_1B, MD_SS, 0, 1);
    rtk_md_gen_data((uint32_t)addrSrc, 1024, 0x77, MD_1B);

    pts_s = rtd_inl(TIMER_REG_VAL_ADDR);

    rtk_md_memcpy_pitch(addrDst, 256, addrSrc, 256, 3, 16, MD_1B, 1);
    pts_e = rtd_inl(TIMER_REG_VAL_ADDR);
    result = rtk_md_check((uint32_t)addrDst, 16, 0x77, MD_1B);
    result += rtk_md_check((uint32_t)addrDst + 16, 256 - 16, 0x00, MD_1B);
    result += rtk_md_check((uint32_t)addrDst + 256, 16, 0x77, MD_1B);
    result += rtk_md_check((uint32_t)addrDst + 256 + 16, 256 - 16, 0x00, MD_1B);
    result += rtk_md_check((uint32_t)addrDst + 256 * 2, 16, 0x77, MD_1B);
    result += rtk_md_check((uint32_t)addrDst + 256 * 2 + 16, 256 - 16, 0x00, MD_1B);

    if(result < 0) {
        printf("[MD] COPY %d bytes with 1B pitch mode fail!\n", len);
        return;
    } else {
        printf("[MD] COPY %d bytes with 1B pitch mode success. elapsed %d us\n", len, (pts_e - pts_s) * 1000000 / 90090);
    }


    /*CASE - pitch 2B */
    rtk_md_memcpy_seq(addrSrc, 0x00, len, MD_CONST, MD_1B, MD_SS, 0, 1);
    rtk_md_memcpy_seq(addrDst, 0x00, len, MD_CONST, MD_1B, MD_SS, 0, 1);
    rtk_md_gen_data((uint32_t)addrSrc, 1024, 0x7117, MD_2B);

    pts_s = rtd_inl(TIMER_REG_VAL_ADDR);

    rtk_md_memcpy_pitch(addrDst, 256, addrSrc, 256, 3, 16, MD_2B, 1);
    pts_e = rtd_inl(TIMER_REG_VAL_ADDR);
    result = rtk_md_check((uint32_t)addrDst, 16, 0x1771, MD_2B);
    result += rtk_md_check((uint32_t)addrDst + 16, 256 - 16, 0x00, MD_2B);
    result += rtk_md_check((uint32_t)addrDst + 256, 16, 0x1771, MD_2B);
    result += rtk_md_check((uint32_t)addrDst + 256 + 16, 256 - 16, 0x00, MD_2B);
    result += rtk_md_check((uint32_t)addrDst + 256 * 2, 16, 0x1771, MD_2B);
    result += rtk_md_check((uint32_t)addrDst + 256 * 2 + 16, 256 - 16, 0x00, MD_2B);

    if(result < 0) {
        printf("[MD] COPY %d bytes with 2B pitch mode fail!\n", len);
        return;
    } else {
        printf("[MD] COPY %d bytes with 2B pitch mode success. elapsed %d us\n", len, (pts_e - pts_s) * 1000000 / 90090);
    }

    /*CASE - pitch 4B */
    rtk_md_memcpy_seq(addrSrc, 0x00, len, MD_CONST, MD_1B, MD_SS, 0, 1);
    rtk_md_memcpy_seq(addrDst, 0x00, len, MD_CONST, MD_1B, MD_SS, 0, 1);
    rtk_md_gen_data((uint32_t)addrSrc, 1024, 0x12345678, MD_4B);

    pts_s = rtd_inl(TIMER_REG_VAL_ADDR);

    rtk_md_memcpy_pitch(addrDst, 256, addrSrc, 256, 3, 16, MD_4B, 1);
    pts_e = rtd_inl(TIMER_REG_VAL_ADDR);
    result = rtk_md_check((uint32_t)addrDst, 16, 0x78563412, MD_4B);
    result += rtk_md_check((uint32_t)addrDst + 16, 256 - 16, 0x00, MD_4B);
    result += rtk_md_check((uint32_t)addrDst + 256, 16, 0x78563412, MD_4B);
    result += rtk_md_check((uint32_t)addrDst + 256 + 16, 256 - 16, 0x00, MD_4B);
    result += rtk_md_check((uint32_t)addrDst + 256 * 2, 16, 0x78563412, MD_4B);
    result += rtk_md_check((uint32_t)addrDst + 256 * 2 + 16, 256 - 16, 0x00, MD_4B);

    if(result < 0) {
        printf("[MD] COPY %d bytes with 4B pitch mode fail!\n", len);
        return;
    } else {
        printf("[MD] COPY %d bytes with 4B pitch mode success. elapsed %d us\n", len, (pts_e - pts_s) * 1000000 / 90090);
    }
    return;
}
#endif
#endif



int rtk_md_unit_test(void)
{
    void *srcdata = NULL;
    void *dstdata = NULL;
    uint32_t mdcrc = 0;
    int ret = 0;
    
    srcdata = (void *)MD_TEST_SRC_ADDR;
    if(srcdata == 0) {
        printf("[MD]alloc src fail.\n");
        return -1;
    }
    
    dstdata = (void *)MD_TEST_DST_ADDR;
    if(dstdata == 0) {
        printf("[MD]alloc dst fail.\n");
        return -1;
    }

    printf("[MD]test src addr = %08x(seq), dst addr = %08x(seq).\n", (uint32_t)srcdata, (uint32_t)dstdata);
    
    /*enable timer 90k */
    rtd_outl(TIMER_REG_CTRL_ADDR, 0x1);
    
    printf("=====================Seq to Seq test======================\n");
    /*CASE - Seq to Seq test*/
    ret += rtk_md_testcases_seq2seq((uint32_t)srcdata, (uint32_t)dstdata, 300);
#if !CONFIG_FT
    //printf("=====================Pitch mode test======================\n");
    /*CASE - pitch test*/
    //rtk_md_testcases_pitch((uint32_t)srcdata, (uint32_t)dstdata, 16384);

#endif
    mdcrc = rtk_md_get_r_crc((uint32_t)srcdata, 0x60);
    printf("[MD]src addr = %08x, crc = %08x.\n", (uint32_t)srcdata, (uint32_t)mdcrc);
    return ret;
}


int rtk_md_performance_test(void)
{
    int ret = 0;
    unsigned int BlockSize = 1024*1024;
    unsigned char *src = NULL;
    unsigned char *dst = NULL;
    uint16_t issued_cnt = 0;
    uint32_t pts_s = 0;
    uint32_t pts_e = 0;
    unsigned int b_mbyte;
    unsigned int u_time;
    
    rtd_outl(TIMER_REG_CTRL_ADDR, 0x1);
    src = malloc(BlockSize * 2);
    if(src == NULL) {
        printf("%s malloc failed\n",  __FUNCTION__);
        return -1;
    }
    
    dst = src + BlockSize;
    pts_s = rtd_inl(TIMER_REG_VAL_ADDR);
    issued_cnt = smd_memcpy((uint32_t)dst, (uint32_t)src, BlockSize);
    smd_checkComplete(issued_cnt);
    pts_e = rtd_inl(TIMER_REG_VAL_ADDR);
    b_mbyte = BlockSize/1024/1024;
    u_time = (pts_e - pts_s) * 1000000 / 90090;
    printf("[MD]smd_memcpy %dMByte. elapsed %d us, throughput=%dMB/s\n",
                                            b_mbyte, u_time, b_mbyte*1000000/u_time);
                                            
    pts_s = rtd_inl(TIMER_REG_VAL_ADDR);
    issued_cnt = smd_memfill((uint32_t)dst, (uint32_t)0x33, BlockSize);
    smd_checkComplete(issued_cnt);
    pts_e = rtd_inl(TIMER_REG_VAL_ADDR);
    b_mbyte = BlockSize/1024/1024;
    u_time = (pts_e - pts_s) * 1000000 / 90090;
    printf("[MD]smd_memfill %dMByte. elapsed %d us, throughput=%dMB/s\n",
                                            b_mbyte, u_time, b_mbyte*1000000/u_time); 
    if(src)
        free(src);
    return ret;
}

