#ifndef __XHCI_PLAT_RTD287O_CONFIG_H__
#define __XHCI_PLAT_RTD287O_CONFIG_H__


typedef struct {
    unsigned char  page;
    unsigned char  reg;
    unsigned short val;
} U3_PHY_REGISTER;

// USB3.0 Phy initial table

static U3_PHY_REGISTER xhci_u3_phy_config[]  __attribute__((unused)) =
{
    { 0x0, 0x00, 0x400c },     // ANA00
    { 0x0, 0x01, 0xa846 },     // ANA01
    { 0x0, 0x02, 0x6042 },     // ANA02
    { 0x0, 0x03, 0x27F1 },     // ANA03
    { 0x0, 0x04, 0xF2F5 },     // ANA04
    { 0x0, 0x05, 0x2AD7 },     // ANA05
    { 0x0, 0x06, 0x0006 },     // ANA06
    { 0x0, 0x07, 0x3210 },     // ANA07
    { 0x0, 0x08, 0x3590 },     // ANA08
    { 0x0, 0x09, 0x367C },     // ANA09
    { 0x0, 0x0A, 0x9642 },     // ANA0A /* (Auto mode) */
    { 0x0, 0x0B, 0xEb09 },     // ANA0B
    { 0x0, 0x0C, 0xC000 },     // ANA0C
    { 0x0, 0x0D, 0xF508 },     // ANA0D  //20170619 Modify [5:2]0100->0010
    { 0x0, 0x0E, 0x2010 },     // ANA0E
    { 0x0, 0x0F, 0x8D90 },     // ANA0F
    { 0x0, 0x10, 0x000C },     // DMR
    { 0x0, 0x11, 0x4C10 },     // BACR
    { 0x0, 0x12, 0xFC00 },     // IER
    { 0x0, 0x13, 0x0C81 },     // BCSR
    { 0x0, 0x14, 0xDE01 },     // BPR
    { 0x0, 0x15, 0x0000 },     // BPNR2
    { 0x0, 0x16, 0x0000 },     // BFNR
    { 0x0, 0x17, 0x0000 },     // BRNR2
    { 0x0, 0x18, 0x0000 },     // BENR
    { 0x0, 0x19, 0x3004 },     // REG_REV0  : bits[15:9] REG_FILTER_OUT
                               // modify   dr.save eq 9-->5
    { 0x0, 0x1A, 0x0085 },     // REG_REV1
    { 0x0, 0x1B, 0x2014 },     // REG_REV2
    { 0x0, 0x1C, 0xC900 },     // REG_REV3
    { 0x0, 0x1D, 0xA03F },     // REG_FLD_0
    { 0x0, 0x1E, 0xC200 },     // REG_FLD_1
    { 0x0, 0x1F, 0x4803 },     // ANA1F
    { 0x1, 0x00, 0xD47A },     // page1 0x00 --> xxxx20xx
    { 0x1, 0x01, 0x88AA },     // PAGE1_REG01 bit[15:12] : Normal operation Amplitude control
                               // PAGE1_REG01 bit[11: 8] : Beacon LFPS Amplitude Control deamphasis--> 8-->a (dr.save)
                               // PAGE1_REG01 bit[ 7: 4] : Normal operation Amplitude control
                               // PAGE1_REG01 bit[ 3: 0] : Beacon LFPS Amplitude Control deamphasis

    { 0x1, 0x02, 0x0057 },
    { 0x1, 0x03, 0xBB64 },     // 20170620 [15:12]=1101->1011
    { 0x1, 0x04, 0x0070 },     // 0719 modify bit 15=1,U1/U2/U3 can enter.
    { 0x1, 0x05, 0x126C },     //
    { 0x1, 0x06, 0x846A },     //
    { 0x1, 0x07, 0x01D6 },     //
    { 0x1, 0x08, 0xf802 },     // PAGE1_REG08 bit[15:11] REG_EQ_UPPER_LIMIT,
                               // PAGE1_REG08 bit[10:6] REG_EQOUT_OFFSET,
                               // PAGE1_REG08 bit[4:0] REG_TIMER_EQ
    { 0x1, 0x09, 0xff00 },     //
    { 0x1, 0x0A, 0x3040 },     //
    { 0x1, 0x0B, 0x8028 },     //
    { 0x1, 0x0C, 0xFFFF },     //
    { 0x1, 0x0D, 0xFFFF },     //
    { 0x1, 0x0E, 0x0000 },     //
    { 0x1, 0x0F, 0x8635 },     //
};

#define U3_PHY_SETTING_SIZE      (sizeof(xhci_u3_phy_config) / sizeof(unsigned long))

/////////////////////////////////////////////////////////////////
// U2 Phy Setting
/////////////////////////////////////////////////////////////////

typedef struct {
    int           port1;
    unsigned char addr;
    unsigned char val;
} U2_PHY_REGISTER;


/////////////////////////////////////////////
// for all ports
U2_PHY_REGISTER u2_phy_setting[] =
{
    {1, 0xF4, 0xbb},   // zap to page1
    {2, 0xF4, 0xbb},
    {3, 0xF4, 0xbb},

    {1, 0XE5, 0x00},   // E5 Page 1
    {2, 0XE5, 0x0f},
    {3, 0XE5, 0x00},

    {1, 0xE6, 0x40},   // E6 Page 1
    {2, 0xE6, 0x58},
    {3, 0xE6, 0x40},

    {1, 0xE7, 0x00},   // E7 Page 1
    {2, 0xE7, 0xe3},
    {3, 0xE7, 0x00},

    {1, 0xF4, 0x9b},   // zap to page0
    {2, 0xF4, 0x9b},
    {3, 0xF4, 0x9b},

    {1, 0xE0, 0x0f},   // E0 Page 0
    {2, 0xE0, 0x0f},
    {3, 0xE0, 0x0f},

    {1, 0xE1, 0x18},
    {2, 0xE1, 0x18},
    {3, 0xE1, 0x18},

    {1, 0xE2, 0x5f},   // XHCI CMU
    {2, 0xE2, 0x5f},
    {3, 0xE2, 0x5f},

    {1, 0xE3, 0xcd},
    {2, 0xE3, 0xcd},
    {3, 0xE3, 0xcd},

    {1, 0xE5, 0x63}, // REG_DBS_SEL[1]=1: Double sensitivity 1.5X
    {2, 0xE5, 0x63},
    {3, 0xE5, 0x63},

    {1, 0xF4, 0xdb},   // page 2
    {2, 0xF4, 0xdb},
    {3, 0xF4, 0xdb},

    {1, 0xE6, 0x53},   // DPLL modify REG_DPLL_VDS[5:4]=10
    {2, 0xE6, 0x53},
    {3, 0xE6, 0x53},

    {1, 0xE7, 0x30},
    {2, 0xE7, 0x30},
    {3, 0xE7, 0x30},

    {1, 0xF4, 0x9b},   // page 0
    {2, 0xF4, 0x9b},
    {3, 0xF4, 0x9b},

    {1, 0xE4, 0x6a},   //bit[7:4] disconnt level,bit[3:0]=swing	//disconnect first stage= 6 //20180428->0507->0510 swing c->a->7->9->a
    {2, 0xE4, 0x6a},
    {3, 0xE4, 0x6a},

    {1, 0xE7, 0x61},   // bit[7:4] device disconnt level
    {2, 0xE7, 0x61},
    {3, 0xE7, 0x61},

    {1, 0xF4, 0xbb},   // page 1
    {2, 0xF4, 0xbb},
    {3, 0xF4, 0xbb},

    {1, 0xE0, 0x23},
    {2, 0xE0, 0x23},
    {3, 0xE0, 0x23},

    {1, 0xE0, 0x27},
    {2, 0xE0, 0x27},
    {3, 0xE0, 0x27},

    {1, 0xF4, 0x9b},   // page 0
    {2, 0xF4, 0x9b},
    {3, 0xF4, 0x9b},

    {1, 0xE4, 0x7a},   //bit[7:4] disconnt level,bit[3:0]=swing/ //20180531 open disconnect manual mode=5->7
    {2, 0xE4, 0x7a},
    {3, 0xE4, 0x7a},

    {1, 0xE7, 0x61},  // bit[7:4] squelch level
    {2, 0xE7, 0x61},
    {3, 0xE7, 0x61},

    {1, 0xE6, 0x01},
    {2, 0xE6, 0x01},
    {3, 0xE6, 0x01},

    {1, 0xF0, 0xfc},
    {2, 0xF0, 0xfc},
    {3, 0xF0, 0xfc},

    {1, 0xF1, 0x88},
    {2, 0xF1, 0x88},
    {3, 0xF1, 0x88},

    {1, 0xF2, 0x00},
    {2, 0xF2, 0x00},
    {3, 0xF2, 0x00},

    {1, 0xF3, 0x31},
    {2, 0xF3, 0x31},
    {3, 0xF3, 0x31},

    {1, 0xF5, 0x15},
    {2, 0xF5, 0x15},
    {3, 0xF5, 0x15},

    {1, 0xF6, 0x01},
    {2, 0xF6, 0x01},
    {3, 0xF6, 0x01},

    {1, 0xF7, 0x8a},
    {2, 0xF7, 0x8a},
    {3, 0xF7, 0x8a},

    {1, 0xF4, 0xbb},   // zap to page1
    {2, 0xF4, 0xbb},
    {3, 0xF4, 0xbb},

    {1, 0xE1, 0x70},
    {2, 0xE1, 0x70},
    {3, 0xE1, 0x70},

    {1, 0xE2, 0x00},
    {2, 0xE2, 0x00},
    {3, 0xE2, 0x00},

    {1, 0xE3, 0x12},
    {2, 0xE3, 0x12},
    {3, 0xE3, 0x12},

    {1, 0xE4, 0x00},
    {2, 0xE4, 0x00},
    {3, 0xE4, 0x00},

    {1, 0xE5, 0x0f},
    {2, 0xE5, 0x0f},
    {3, 0xE5, 0x0f},

    {1, 0xE6, 0x58},
    {2, 0xE6, 0x58},
    {3, 0xE6, 0x58},

    {1, 0xE7, 0xe3},
    {2, 0xE7, 0xe3},
    {3, 0xE7, 0xe3},

    {1, 0xF4, 0xdb},   // zap to page2
    {2, 0xF4, 0xdb},
    {3, 0xF4, 0xdb},

    {1, 0xE0, 0xa8},
    {2, 0xE0, 0xa8},
    {3, 0xE0, 0xa8},

    {1, 0xE1, 0x11},
    {2, 0xE1, 0x11},
    {3, 0xE1, 0x11},

    {1, 0xE2, 0xe3},
    {2, 0xE2, 0xe3},
    {3, 0xE2, 0xe3},

    {1, 0xE3, 0xf8},  // REG_DPLL_FLQ[6:5]=11
    {2, 0xE3, 0xf8},
    {3, 0xE3, 0xf8},

    {1, 0xE4, 0x15},
    {2, 0xE4, 0x15},
    {3, 0xE4, 0x15},

    {1, 0xE5, 0x81},
    {2, 0xE5, 0x81},
    {3, 0xE5, 0x81},

    {1, 0xF4, 0x9b},   // zap to page0
    {2, 0xF4, 0x9b},
    {3, 0xF4, 0x9b},
};


typedef struct {
    U2_PHY_REGISTER* p_reg_table;
    unsigned int     n_reg;
} U2_PHY_CONFIG;


U2_PHY_CONFIG u2_phy_config = {
    .p_reg_table = u2_phy_setting,
    .n_reg       = sizeof(u2_phy_setting) / sizeof(U2_PHY_REGISTER),
};

#define U2_PHY_CONFIG_COUNT     3

enum CRT_CLK {
	CLK_ON = 0,
	CLK_OFF = 1,
};

enum CRT_RET {
	CRT_SUCCESS = 0,
	CRT_FAIL = 1,
};
#endif // __XHCI_PLAT_RTD287O_CONFIG_H__
