/*
 *
 *  file:rtk_mcp_reg.h
 *
 *  Define the register setting of KCPU-MCP or SCPU-MCP
 *
 */

#ifndef __RTK_MCP_REG_H__
#define __RTK_MCP_REG_H__

#define CP_REG_BASE                          0xb8015000
#if defined(CONFIG_KCPU_MCP) || defined(CONFIG_K_AUX_MCP)
#define K_MCP_CTRL                            (CP_REG_BASE + 0x900)
#define K_MCP_STATUS				(CP_REG_BASE + 0x904)
#define K_MCP_EN					(CP_REG_BASE + 0x908)
#define K_MCP_CTRL1                           (CP_REG_BASE + 0x998)
#define K_MCP_BASE					(CP_REG_BASE + 0x90c)
#define K_MCP_LIMIT				       (CP_REG_BASE + 0x910)
#define K_MCP_RDPTR				(CP_REG_BASE + 0x914)
#define K_MCP_WRPTR				(CP_REG_BASE + 0x918)
#define K_MCP_DES_COUNT			(CP_REG_BASE + 0x934)
#define K_MCP_DES_COMPARE			(CP_REG_BASE + 0x938)
#define K_MCP_DES_INI_KEY			(CP_REG_BASE + 0x91C)
#define K_MCP_AES_INI_KEY			(CP_REG_BASE + 0x924)
#define K_MCP_MULTI2_SYS_KEY         (CP_REG_BASE + 0x978)
#define  CP_OTP_LOAD                         (CP_REG_BASE + 0x19c)
#define K_MCP_RSA_SEL_REG               (CP_REG_BASE + 0x59604)

/* Nagra Key write agent register */
#define CP_CW_AGENT     (CP_REG_BASE + 0xe0)

#define CW_TP_REG_INFO      (0xb8014118)
#define CW_TP_REG_CTRL      (0xb8014120)

#define WRITE_REG32(reg, val)            (REG32(reg) = (val))
#define READ_REG32(reg)                     (REG32(reg))
#else
#include <asm/arch/system.h>
#define K_MCP_CTRL                            (CP_REG_BASE + 0x100)
#define K_MCP_STATUS                        (CP_REG_BASE + 0x104)
#define K_MCP_EN					(CP_REG_BASE + 0x108)
#define K_MCP_CTRL1                           (CP_REG_BASE + 0x198)
#define K_MCP_BASE					(CP_REG_BASE + 0x10c)
#define K_MCP_LIMIT				       (CP_REG_BASE + 0x110)
#define K_MCP_RDPTR				(CP_REG_BASE + 0x114)
#define K_MCP_WRPTR				(CP_REG_BASE + 0x118)
#define K_MCP_DES_COUNT			(CP_REG_BASE + 0x134)
#define K_MCP_DES_COMPARE			(CP_REG_BASE + 0x138)
#define K_MCP_DES_INI_KEY			(CP_REG_BASE + 0x11C)
#define K_MCP_AES_INI_KEY			(CP_REG_BASE + 0x124)
#define K_MCP_MULTI2_SYS_KEY         (CP_REG_BASE + 0x178)
#define  CP_OTP_LOAD                         (CP_REG_BASE + 0x19c)
#define K_MCP_RSA_SEL_REG               (CP_REG_BASE + 0x59504)


#define WRITE_REG32(reg, val)            rtd_outl(reg, val)
#define READ_REG32(reg)                     rtd_inl(reg)

#endif

/* MCP descriptor define */
#define MCP_IV_FROM_CW               (1 << 14)

#define MCP_KEY_SEL(x)                       (((x) & 0x3) << 12)
#define MCP_KEY_FROM_DSCPT            MCP_KEY_SEL(0)
#define MCP_KEY_FROM_OTP                MCP_KEY_SEL(1)
#define MCP_KEY_FROM_CW                 MCP_KEY_SEL(2)
#define MCP_KEY_FROM_DDR                MCP_KEY_SEL(3)

#define MCP_BCM_SEL(x)                       (((x) & 0xf) << 6)
#define MCP_BCM_ECB                          MCP_BCM_SEL(0)
#define MCP_BCM_CBC                          MCP_BCM_SEL(1)
#define MCP_BCM_CTR                          MCP_BCM_SEL(2)
#define MCP_BCM_RCBC_CS                  MCP_BCM_SEL(3)

#define MCP_ENCRYPT                          BIT(5)
#define MCP_DECRYPT                           0x0

#define MCP_MODE_SEL(x)                    ((x) & 0x1f)
#define MCP_MODE_DES                       MCP_MODE_SEL(0x0)
#define MCP_MODE_3DES                     MCP_MODE_SEL(0x1)
#define MCP_MODE_MD5                      MCP_MODE_SEL(0x3)
#define MCP_MODE_SHA1                     MCP_MODE_SEL(0x4)
#define MCP_MODE_AES                       MCP_MODE_SEL(0x5)
#define MCP_MODE_AES_G                   MCP_MODE_SEL(0x6)
#define MCP_MODE_AES_H                   MCP_MODE_SEL(0x7)
#define MCP_MODE_CMAC                    MCP_MODE_SEL(0x8)
#define MCP_MODE_CSA                       MCP_MODE_SEL(0x9)
#define MCP_MODE_MULTI2                 MCP_MODE_SEL(0xa)
#define MCP_MODE_SHA256                 MCP_MODE_SEL(0xb)
#define MCP_MODE_CRC                       MCP_MODE_SEL(0xc)
#define MCP_MODE_CRC_SHA256          MCP_MODE_SEL(0xd)
#define MCP_MODE_AES_192                MCP_MODE_SEL(0x10)
#define MCP_MODE_AES_256                MCP_MODE_SEL(0x11)
#define MCP_MODE_CAM                      MCP_MODE_SEL(0x1d)
#define MCP_MODE_CAM_192               MCP_MODE_SEL(0x1e)
#define MCP_MODE_CAM_256               MCP_MODE_SEL(0x1f)

#define REG_KEY_CW_ENTRY(x)           (((x) & 0x7F)<<8)
#define REG_IV_CW_ENTRY(x)           (((x) & 0x7F)<<0)

#endif
