/************************************************************************
 *
 *  mcp.h
 *
 *  Public header file for CP module
 *
 ************************************************************************/

#ifndef __RTK_MCP_SCPU_H__
#define __RTK_MCP_SCPU_H__
#include <asm/arch/system.h>
#include <drivers/mcp/rtk_mcp_core.h>
#include <drivers/mcp/rtk_mcp_rsa.h>

/***********Memory usage**********/
#define SECURE_IMAGE2HASH_BUF           (rtk_mem_info->secure_mcp_mem_buffer_addr)
#define SECURE_SIGN2HASH_BUF             (rtk_mem_info->secure_mcp_mem_buffer_addr + 0x200)
#define SECURE_SIGN2HASH_TEMP_BUF   (rtk_mem_info->secure_mcp_mem_buffer_addr + 0x800)
#define SECURE_MAX_ALLOC_SIZE            (UBOOT_SECURE_MCP_MEMORY_SIZE)  // (32UL << 20)
#define SECURE_MALLOC_BASE                 SECURE_SIGN2HASH_TEMP_BUF

/************************************************************************
 *  Definition
 ************************************************************************/
#define DEFAULT_KEY_PTR                      NULL // Kh_key_default
#define UBOOT_DDR_OFFSET                    0xA0000000  //for RTD299X

#define CP_DESCRIPTOR_ADDR                (0xa000ff00 - UBOOT_DDR_OFFSET) /* CP descriptor address */
#define CP_DSCPT_POOL_BASE_ADDR      (0xa0010000 - UBOOT_DDR_OFFSET) /* CP descriptor pool base address */
#define CP_DSCPT_POOL_SIZE                 0x800 /* CP descriptor pool size */
#define CP_DSCPT_POOL_MAX_ADDR       (CP_DSCPT_POOL_BASE_ADDR + CP_DSCPT_POOL_SIZE)
#define CP_DESCRIPTOR_HW_ALIGN    0x60


#define SECURE_KH_KEY_STR                  "abcdef1213572468a1b2c3d49090babe"
#define SECURE_KH_KEY0                        0xabcdef12
#define SECURE_KH_KEY1                        0x13572468
#define SECURE_KH_KEY2                        0xa1b2c3d4
#define SECURE_KH_KEY3                        0x9090babe

#define RSA_SIGNATURE_LENGTH            256
#define PHYS(addr)                                  ((uint)(addr))

#define S_RSA_SEL                             0xB806E504
#define S_RSA_ST                              0xB806E508

/************************************************************************
 *  For QC Verify Test functions
 ************************************************************************/
int MCP_DES_ECB_Test(void);
int MCP_DES_CBC_Test(void);
int MCP_TDES_ECB_Test(void);
int MCP_TDES_CBC_Test(void);
int MCP_AES_128_ECB_Test(void);
int MCP_AES_128_CBC_Test(void);
int MCP_AES_128_CBC_NO_IV_Test(void);
int MCP_AES_128_CTR_Test(void);

int MCP_AES_128_ECB_CW_Test(void);
int MCP_AES_128_CBC_CW_Test(void);
int MCP_AES_128_CTR_CW_Test(void);
int MCP_AES_192_ECB_CW_Test(void);
int MCP_AES_192_CBC_CW_Test(void);
int MCP_AES_192_CTR_CW_Test(void);
int MCP_AES_256_ECB_CW_Test(void);
int MCP_AES_256_CBC_CW_Test(void);
int MCP_AES_256_CTR_CW_Test(void);
int MCP_AES_128_ECB_DDR_Test(void);
int MCP_AES_128_CBC_DDR_Test(void);
int MCP_AES_128_CTR_DDR_Test(void);
int MCP_AES_192_ECB_DDR_Test(void);
int MCP_AES_192_CBC_DDR_Test(void);
int MCP_AES_192_CTR_DDR_Test(void);
int MCP_AES_256_ECB_DDR_Test(void);
int MCP_AES_256_CBC_DDR_Test(void);
int MCP_AES_256_CTR_DDR_Test(void);

int MCP_CAM_128_ECB_Test(void);
int MCP_CAM_128_CBC_Test(void);
int MCP_CAM_128_CTR_Test(void);
int MCP_CAM_128_ECB_CW_Test(void);
int MCP_CAM_128_CBC_CW_Test(void);
int MCP_CAM_128_CTR_CW_Test(void);
int MCP_CAM_192_ECB_CW_Test(void);
int MCP_CAM_192_CBC_CW_Test(void);
int MCP_CAM_192_CTR_CW_Test(void);
int MCP_CAM_256_ECB_CW_Test(void);
int MCP_CAM_256_CBC_CW_Test(void);
int MCP_CAM_256_CTR_CW_Test(void);
int MCP_CAM_128_ECB_DDR_Test(void);
int MCP_CAM_128_CBC_DDR_Test(void);
int MCP_CAM_128_CTR_DDR_Test(void);
int MCP_CAM_192_ECB_DDR_Test(void);
int MCP_CAM_192_CBC_DDR_Test(void);
int MCP_CAM_192_CTR_DDR_Test(void);
int MCP_CAM_256_ECB_DDR_Test(void);
int MCP_CAM_256_CBC_DDR_Test(void);
int MCP_CAM_256_CTR_DDR_Test(void);

int MCP_SHA1_HASH_Test(void);
int MCP_SHA256_HASH_Test(void);
int MCP_AES_HASH_Test(void);
int Verify_SHA256_hash( unsigned char * src_addr, unsigned int length, unsigned char * ref_sha256, unsigned int do_recovery );
void enable_SCPU_RSA (void);

#endif
